@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  */
 
 #include <config.h>
@@ -45,6 +45,7 @@
 #define GstStateChangeReturn GstElementStateReturn
 #define GstState GstElementState
 #define GST_STATE_CHANGE_FAILURE GST_STATE_FAILURE
+#define gst_caps_unref gst_caps_free
 #endif
 
 
@@ -55,8 +56,13 @@ static void rb_encoder_init (RBEncoderIface *iface);
 
 
 struct _RBEncoderGstPrivate {
+	GstElement *enc;
 	GstElement *pipeline;
 	
+	gboolean transcoding;
+	gint decoded_pads;
+	
+	gboolean error_emitted;
 	gboolean completion_emitted;
 
 	gulong total_length;
@@ -103,11 +109,6 @@ static void
 rb_encoder_gst_finalize (GObject *object)
 {
 	RBEncoderGst *encoder = RB_ENCODER_GST (object);
-
-	if (encoder->priv->completion_emitted == FALSE) {
-		g_warning ("completion not emitted for encoder");
-		rb_encoder_gst_emit_completed (encoder);
-	}
 	
 	if (encoder->priv->progress_id != 0)
 		g_source_remove (encoder->priv->progress_id);	
@@ -127,14 +128,39 @@ rb_encoder_gst_new (void)
 	return RB_ENCODER (g_object_new (RB_TYPE_ENCODER_GST, NULL));
 }
 
+static void
+rb_encoder_gst_emit_error (RBEncoderGst *encoder, GError *error)
+{
+	encoder->priv->error_emitted = TRUE;
+	_rb_encoder_emit_error (RB_ENCODER (encoder), error);
+}
+
 static void
 rb_encoder_gst_emit_completed (RBEncoderGst *encoder)
 {
+	GError *error = NULL;
+
 	g_return_if_fail (encoder->priv->completion_emitted == FALSE);
 
 	if (encoder->priv->progress_id != 0)
 		g_source_remove (encoder->priv->progress_id);	
 
+
+	/* emit an error if no audio pad has been found and it wasn't due to an
+	 * error */
+	if (encoder->priv->error_emitted == FALSE &&
+			encoder->priv->transcoding &&
+			encoder->priv->decoded_pads == 0) {
+		rb_debug ("received EOS and no decoded pad");
+		g_set_error (&error,
+				RB_ENCODER_ERROR,
+				RB_ENCODER_ERROR_FORMAT_UNSUPPORTED,
+				"no decodable audio pad found");
+
+		rb_encoder_gst_emit_error (encoder, error);
+		g_error_free (error);
+	}
+
 	encoder->priv->completion_emitted = TRUE;
 	_rb_encoder_emit_completed (RB_ENCODER (encoder));
 }
@@ -154,7 +180,7 @@ bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
 	switch (GST_MESSAGE_TYPE (message)) {
 	case GST_MESSAGE_ERROR:
 		gst_message_parse_error (message, &error, &string);
-		_rb_encoder_emit_error (RB_ENCODER (encoder), error);
+		rb_encoder_gst_emit_error (encoder, error);
 		rb_debug ("received error %s", string);
 		g_error_free (error);
 		g_free (string);
@@ -211,7 +237,7 @@ gst_error_cb (GstElement *element,
 			  gchar *debug,
 			  RBEncoderGst *encoder)
 {
-	_rb_encoder_emit_error (RB_ENCODER (encoder), error);
+	rb_encoder_gst_emit_error (encoder, error);
 	rb_debug ("received error %s", debug);
 		
 	rb_encoder_cancel (RB_ENCODER (encoder));
@@ -251,7 +277,7 @@ progress_timeout_cb (RBEncoderGst *encoder)
 #endif
 
 	secs = nanos / GST_SECOND;
-	rb_debug ("encoding progress at %d out of %d", secs, encoder->priv->total_length);
+	rb_debug ("encoding progress at %d out of %lu", secs, encoder->priv->total_length);
 	_rb_encoder_emit_progress (RB_ENCODER (encoder), ((double)secs) / encoder->priv->total_length);
 
 	return TRUE;
@@ -264,13 +290,16 @@ start_pipeline (RBEncoderGst *encoder, GError **error)
 #ifdef HAVE_GSTREAMER_0_10
 	GstBus *bus;
 
-	g_assert (encoder->priv->pipeline);
+	g_return_val_if_fail (encoder->priv->pipeline != NULL, FALSE);
 	
 	bus = gst_pipeline_get_bus (GST_PIPELINE (encoder->priv->pipeline));
 	gst_bus_add_watch (bus, bus_watch_cb, encoder);
 
 	result = gst_element_set_state (encoder->priv->pipeline, GST_STATE_PLAYING);
 #elif HAVE_GSTREAMER_0_8
+	
+	g_return_val_if_fail (encoder->priv->pipeline != NULL, FALSE);
+	
 	g_signal_connect_object (G_OBJECT (encoder->priv->pipeline),
 				 "error", G_CALLBACK (gst_error_cb),
 				 encoder, 0);
@@ -327,7 +356,6 @@ const char *GST_ENCODING_PROFILE = "audioresample ! audioconvert ! %s";
 
 static GstElement*
 add_encoding_pipeline (RBEncoderGst *encoder,
-		       GstElement *start,
 		       GMAudioProfile *profile,
 		       GError **error)
 {
@@ -338,10 +366,13 @@ add_encoding_pipeline (RBEncoderGst *encoder,
 	queue = gst_element_factory_make ("queue", NULL);
 	if (queue == NULL)
 		return NULL;
+	gst_bin_add (GST_BIN (encoder->priv->pipeline), queue);
+	
 	queue2 = gst_element_factory_make ("queue", NULL);
 	if (queue2 == NULL)
 		return NULL;
-
+	gst_bin_add (GST_BIN (encoder->priv->pipeline), queue2);
+	
 	/* Nice big buffers... */
 	g_object_set (queue, "max-size-time", 120 * GST_SECOND, NULL);
 
@@ -359,9 +390,15 @@ add_encoding_pipeline (RBEncoderGst *encoder,
 	if ((pad = gst_bin_find_unconnected_pad (GST_BIN (encoding_bin), GST_PAD_SINK)))
 		gst_element_add_pad (encoding_bin, gst_ghost_pad_new ("sink", pad));
 
-	gst_bin_add_many (GST_BIN (encoder->priv->pipeline), queue, encoding_bin, queue2, NULL);
-	gst_element_link_many (start, queue, encoding_bin, queue2, NULL);
+	gst_bin_add (GST_BIN (encoder->priv->pipeline), encoding_bin);
 
+	if (gst_element_link_many (queue, encoding_bin, queue2, NULL) == FALSE)
+		return NULL;
+
+	/* store the first element of the encoding graph. new_decoded_pad_cb
+	 * will link to this once a decoded pad is found */
+	encoder->priv->enc = queue;
+	
 	return queue2;
 }
 
@@ -473,6 +510,67 @@ gnomevfs_allow_overwrite_cb (GstElement *element, GnomeVFSURI *uri, RBEncoderGst
 	return (response == GTK_RESPONSE_YES);				 
 }
 
+static void
+new_decoded_pad_cb (GstElement *decodebin,
+		GstPad *new_pad, gboolean arg1,
+		RBEncoderGst *encoder)
+{
+	GstPad *enc_sinkpad;
+	GstCaps *caps;
+	gchar *caps_string;
+
+	rb_debug ("new decoded pad");
+
+	/* transcode only the first audio track. multitrack audio files are not
+	 * so common anyway */
+	if (encoder->priv->decoded_pads > 0)
+		return;
+	
+	caps = gst_pad_get_caps (new_pad);
+	caps_string = gst_caps_to_string (caps);
+	gst_caps_unref (caps);
+
+	/* only process audio data */
+	if (strncmp (caps_string, "audio/", 6) == 0) {
+		encoder->priv->decoded_pads++;
+		enc_sinkpad = gst_element_get_pad (encoder->priv->enc,
+				"sink");
+		if (gst_pad_link (new_pad, enc_sinkpad) != GST_PAD_LINK_OK)
+			rb_debug ("error linking pads");
+	}
+
+	g_free (caps_string);
+}
+
+static GstElement *
+add_decoding_pipeline (RBEncoderGst *encoder,
+			GError **error)
+{
+	GstElement *decodebin;
+
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	encoder->priv->transcoding = TRUE;
+	decodebin = gst_element_factory_make ("decodebin", NULL);
+	if (decodebin == NULL) {
+		g_set_error (error,
+				RB_ENCODER_ERROR,
+				RB_ENCODER_ERROR_INTERNAL,
+				"Could not create decodebin");
+		
+		return NULL;
+	}
+	
+	gst_bin_add (GST_BIN (encoder->priv->pipeline), decodebin);
+	
+	g_signal_connect_object (decodebin, 
+			"new-decoded-pad",
+			G_CALLBACK (new_decoded_pad_cb),
+			encoder, 0);
+
+	return decodebin;
+}
+
 static gboolean
 attach_output_pipeline (RBEncoderGst *encoder,
 			GstElement *end,
@@ -498,11 +596,111 @@ attach_output_pipeline (RBEncoderGst *encoder,
 	return TRUE;
 }
 
+static gboolean
+encoder_match_mime (GstElement *encoder, const gchar *mime_type)
+{
+	GstPad *srcpad;
+	GstCaps *caps;
+	GstStructure *structure;
+	const gchar *pad_mime;
+	gboolean match;
+
+	srcpad = gst_element_get_pad (encoder, "src");
+	caps = gst_pad_get_caps (srcpad);
+	structure = gst_caps_get_structure (caps, 0);
+	pad_mime = gst_structure_get_name (structure);
+	match = strcmp (mime_type, pad_mime) == 0;
+	gst_caps_unref (caps);
+	gst_object_unref (GST_OBJECT (srcpad));
+
+	return match;
+}
+
+static GstElement *
+profile_bin_find_encoder (GstBin *profile_bin)
+{
+#ifdef HAVE_GSTREAMER_0_10
+	GstElementFactory *factory;
+	GstElement *encoder = NULL;
+	GstIterator *iter;
+	gboolean done = FALSE;
+	
+	iter = gst_bin_iterate_elements (profile_bin);
+	while (!done) {
+		gpointer data;
+
+		switch (gst_iterator_next (iter, &data)) {
+			case GST_ITERATOR_OK:
+				factory = gst_element_get_factory (GST_ELEMENT (data));
+				if (strcmp (factory->details.klass,
+						"Codec/Encoder/Audio") == 0) {
+					encoder = GST_ELEMENT (data);
+					done = TRUE;
+				}
+				break;
+			case GST_ITERATOR_RESYNC:
+				gst_iterator_resync (iter);
+				break;
+			case GST_ITERATOR_ERROR:
+				/* !?? */
+				rb_debug ("iterator error");
+				done = TRUE;
+				break;
+			case GST_ITERATOR_DONE:
+				done = TRUE;
+				break;
+		}
+	}
+	gst_iterator_free (iter);
+	
+	return encoder;
+#else
+	return NULL;
+#endif
+}
+
 static GMAudioProfile*
 get_profile_from_mime_type (const char *mime_type)
 {
-	/* TODO */
-	return NULL;
+	GList *profiles, *walk;
+	gchar *pipeline_description;
+	GstElement *pipeline;
+	GstElement *encoder;
+	GMAudioProfile *profile;
+	GMAudioProfile *matching_profile = NULL;
+	GError *error = NULL;
+	
+	profiles = gm_audio_profile_get_active_list ();
+	for (walk = profiles; walk; walk = g_list_next (walk)) {
+		profile = (GMAudioProfile *) walk->data;
+		pipeline_description =
+			g_strdup_printf ("fakesrc ! %s ! fakesink", 
+				gm_audio_profile_get_pipeline (profile));
+		pipeline = gst_parse_launch (pipeline_description, &error);	
+		g_free (pipeline_description);
+		if (error) {
+			g_error_free (error);
+			continue;
+		}
+
+		encoder = profile_bin_find_encoder (GST_BIN (pipeline));
+		if (encoder == NULL) {
+			g_object_unref (pipeline);
+			continue;
+		}
+		
+		if (encoder_match_mime (encoder, mime_type)) {
+			matching_profile = profile;
+			gst_object_unref (GST_OBJECT (encoder));
+			gst_object_unref (GST_OBJECT (pipeline));
+			break;
+		}
+		
+		gst_object_unref (GST_OBJECT (encoder));
+		gst_object_unref (GST_OBJECT (pipeline));
+	}
+
+	return matching_profile;
 }
 
 static GstElement *
@@ -510,15 +708,19 @@ create_pipeline_and_source (RBEncoderGst *encoder,
 			    RhythmDBEntry *entry,
 			    GError **error)
 {
-	const char *uri;
+	char *uri;
 	GstElement *src;
 
-	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	uri = rhythmdb_entry_get_playback_uri (entry);
+	if (uri == NULL)
+		return NULL;
+
 	src = gst_element_make_from_uri (GST_URI_SRC, uri, "source");
 	if (src == NULL) {
 		g_set_error (error,
 			     RB_ENCODER_ERROR, RB_ENCODER_ERROR_INTERNAL,
 			     "could not create source element for '%s'", uri);
+		g_free (uri);
 		return NULL;
 	}
 
@@ -527,6 +729,8 @@ create_pipeline_and_source (RBEncoderGst *encoder,
 
 	/* TODO: add progress reporting */
 
+	g_free (uri);
+
 	return src;
 }
 
@@ -561,7 +765,7 @@ extract_track (RBEncoderGst *encoder,
 	       GError **error)
 {
 	/* cdsrc ! encoder ! sink */
-	const char *uri;
+	char *uri;
 	const char *device;
 	const char *profile_name;
 	GMAudioProfile *profile;
@@ -582,10 +786,10 @@ extract_track (RBEncoderGst *encoder,
 	if (src == NULL)
 		return FALSE;
 
-	g_assert (encoder->priv->pipeline != NULL);
-
 	/* setup cd extraction properties */
-	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	uri = rhythmdb_entry_get_playback_uri (entry);
+	if (uri == NULL)
+		return FALSE;
 
 	device = g_utf8_strrchr (uri, -1, '#');
 	g_object_set (G_OBJECT (src),
@@ -598,11 +802,14 @@ extract_track (RBEncoderGst *encoder,
 		paranoia_mode = 255; /* TODO: make configurable */
 		g_object_set (G_OBJECT (src), "paranoia-mode", paranoia_mode, NULL);
 	}
+	g_free (uri);
 
-	end = add_encoding_pipeline (encoder, src, profile, error);
+	end = add_encoding_pipeline (encoder, profile, error);
 	if (end == NULL)
 		return FALSE;
-
+	if (gst_element_link (src, encoder->priv->enc) == FALSE)
+		return FALSE;
+	
 	if (!attach_output_pipeline (encoder, end, dest, error))
 		return FALSE;
 	if (!add_tags_from_entry (encoder, entry, error))
@@ -620,35 +827,37 @@ transcode_track (RBEncoderGst *encoder,
 		 const char *mime_type,
 		 GError **error)
 {
-	/* cdsrc ! encoder ! sink */
+	/* src ! decodebin ! queue ! encoding_profile ! queue ! sink */
 	GMAudioProfile *profile;
-	GstElement *src, *spider, *end;
-
+	GstElement *src, *decoder, *end;
+	
 	g_assert (encoder->priv->pipeline == NULL);
-
+	
 	profile = get_profile_from_mime_type (mime_type);
 	if (profile == NULL) {
 		g_set_error (error,
-			     RB_ENCODER_ERROR, RB_ENCODER_ERROR_FORMAT_UNSUPPORTED,
-			     "Unable to locate encoding profile for mime-type '%s'", mime_type);
+			     RB_ENCODER_ERROR,
+			     RB_ENCODER_ERROR_FORMAT_UNSUPPORTED,
+			     "Unable to locate encoding profile for mime-type "
+			     "'%s'", mime_type);
 		return FALSE;
+	} else {
+		rb_debug ("selected profile %s",
+				gm_audio_profile_get_name (profile));
 	}
 
 	src = create_pipeline_and_source (encoder, entry, error);
 	if (src == NULL)
 		return FALSE;
 
-	spider = gst_element_factory_make ("spider", "spider");
-	if (spider == NULL) {
-		g_set_error (error,
-			     RB_ENCODER_ERROR, RB_ENCODER_ERROR_INTERNAL,
-			     "Could not create encoding profile '%s'", gm_audio_profile_get_id (profile));
+	decoder = add_decoding_pipeline (encoder, error);
+	if (decoder == NULL)
 		return FALSE;
-	}
-	gst_bin_add (GST_BIN (encoder->priv->pipeline), spider);
-	gst_element_link (src, spider);
 
-	end = add_encoding_pipeline (encoder, spider, profile, error);
+	if (gst_element_link (src, decoder) == FALSE)
+		return FALSE;
+
+	end = add_encoding_pipeline (encoder, profile, error);
 	if (end == NULL)
 		return FALSE;
 
@@ -703,6 +912,8 @@ rb_encoder_gst_cancel (RBEncoder *encoder)
 	gst_element_set_state (priv->pipeline, GST_STATE_NULL);
 	g_object_unref (priv->pipeline);
 	priv->pipeline = NULL;
+
+	rb_encoder_gst_emit_completed (RB_ENCODER_GST (encoder));
 }
 
 static gboolean
@@ -737,10 +948,16 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 	}
 
 	if (error) {
-		rb_encoder_cancel (encoder);
-		_rb_encoder_emit_error (encoder, error);
+		RBEncoderGst *enc = RB_ENCODER_GST (encoder);
+		
+		rb_encoder_gst_emit_error (enc, error);
 		g_error_free (error);
-		rb_encoder_gst_emit_completed (RB_ENCODER_GST (encoder));
+		if (enc->priv->pipeline == NULL)
+			rb_encoder_gst_emit_completed (enc);
+		else
+			/* this will unref the pipeline and call emit_completed
+			 */
+			rb_encoder_gst_cancel (encoder);
 	}
 
 	return result;
