@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Authors: Ramiro Estrugo <ramiro@eazel.com>
 */
@@ -64,7 +64,9 @@ eel_gconf_handle_error (GError **error)
 	g_return_val_if_fail (error != NULL, FALSE);
 
 	if (*error != NULL) {
-		rb_error_dialog (_("GConf error:\n  %s"), (*error)->message);
+		rb_error_dialog (NULL,
+				 _("Configuration system error"),
+				 "%s", (*error)->message);
 		g_error_free (*error);
 		*error = NULL;
 
