@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -42,7 +42,7 @@ void rb_debug_init_match       (const char *match);
 void rb_debug_real             (const char *func,
 				const char *file,
 				int line,
-				const char *format, ...);
+				const char *format, ...) G_GNUC_PRINTF (4, 5);
 
 void rb_debug_stop_in_debugger (void);
 
