@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -647,7 +647,11 @@ rb_uri_handle_recursively_cb (const gchar *rel_path,
 	if (data->cancel_flag && *data->cancel_flag)
 		return TRUE;
 	
-	if ((info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) && g_str_has_prefix (rel_path, ".Trash")) {
+	/* skip trash and unreadable directories */
+	if ((info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) && 
+	    (g_str_has_prefix (rel_path, ".Trash") ||
+	     ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) &&
+	      !(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE)))) {
 		*recurse = FALSE;
 		return TRUE;
 	}
@@ -682,7 +686,8 @@ rb_uri_handle_recursively (const char *text_uri,
 
 	flags = GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
 		GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
-		GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
+		GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
+		GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS;
 	result = gnome_vfs_directory_visit (text_uri,
 					    flags,
 					    GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
@@ -746,7 +751,10 @@ rb_canonicalise_uri (const char *uri)
 			gnome_vfs_uri_unref (vfsuri);
 			result = g_strdup (uri);
 		} else {
+			/* this may just mean that gnome-vfs doesn't recognise the
+			 * uri scheme, so return it as is */
 			rb_debug ("Error processing probable URI %s", uri);
+			result = g_strdup (uri);
 		}
 	}
 
