@@ -15,12 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
 #include <gmodule.h>
 #include <gtk/gtk.h>
+#include <string.h>
 
 #include "rb-glade-helpers.h"
 #include "rb-file-helpers.h"
@@ -113,7 +114,15 @@ glade_signal_connect_func (const gchar *cb_name, GObject *obj,
 void
 rb_glade_boldify_label (GladeXML *xml, const char *name)
 {
-	GtkLabel *label = GTK_LABEL (glade_xml_get_widget (xml, name));
+	GtkWidget *widget;
+
+	widget = glade_xml_get_widget (xml, name);
+
+	if (widget == NULL) {
+		g_warning ("widget '%s' not found", name);
+		return;
+	}
+
 	/* this way is probably better, but for some reason doesn't work with
 	 * labels with mnemonics.
 
@@ -128,12 +137,27 @@ rb_glade_boldify_label (GladeXML *xml, const char *name)
 		attr->end_index = G_MAXINT;
 		pango_attr_list_insert (pattrlist, attr);
 	}
-	gtk_label_set_attributes (label, pattrlist);*/
+	gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);*/
 
 	gchar *str_final;
-	str_final = g_strdup_printf ("<b>%s</b>", gtk_label_get_label (label));
-	gtk_label_set_markup_with_mnemonic (label, str_final);
+	str_final = g_strdup_printf ("<b>%s</b>", gtk_label_get_label (GTK_LABEL (widget)));
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (widget), str_final);
 	g_free (str_final);
 }
 
+gboolean
+rb_combo_box_hyphen_separator_func (GtkTreeModel *model,
+				    GtkTreeIter *iter,
+				    gpointer data)
+{
+	const char *s;
+
+	gtk_tree_model_get (model, iter, 0, &s, -1);
+
+	if (s == NULL)
+		return FALSE;
+
+	return (strcmp (s, "-") == 0);
+}
+
 
