@@ -15,16 +15,22 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
+#include "config.h"
 
-#include "rb-util.h"
-#include <gtk/gtk.h>
 #include <string.h>
+#include <stdarg.h>
+#include <stdlib.h>
+
+#include <gtk/gtk.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <gobject/gvaluecollector.h>
+
+#include "rb-util.h"
 #include "rb-debug.h"
 
 static GPrivate * private_is_primary_thread;
@@ -278,8 +284,12 @@ rb_gtk_action_popup_menu (GtkUIManager *uimanager, const char *path)
 	GtkWidget *menu;
 
 	menu = gtk_ui_manager_get_widget (uimanager, path);
+	if (menu == NULL) {
+		g_warning ("Couldn't get menu widget for %s", path);
+	} else {
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, 
 			gtk_get_current_event_time ());
+	}
 }
 
 static GList *
@@ -568,6 +578,55 @@ rb_make_duration_string (guint duration)
 	return str;
 }
 
+char *
+rb_make_elapsed_time_string (guint elapsed, guint duration, gboolean show_remaining)
+{
+	int seconds = 0, minutes = 0, hours = 0;
+	int seconds2 = 0, minutes2 = 0, hours2 = 0;
+
+	if (duration == 0)
+		return rb_make_duration_string (elapsed);
+
+	if (duration > 0) {
+		hours2 = duration / (60 * 60);
+		minutes2 = (duration - (hours2 * 60 * 60)) / 60;
+		seconds2 = duration % 60;
+	}
+
+	if (elapsed > 0) {
+		hours = elapsed / (60 * 60);
+		minutes = (elapsed - (hours * 60 * 60)) / 60;
+		seconds = elapsed % 60;
+	}
+
+	if (show_remaining) {
+		int remaining = duration - elapsed;
+		int remaining_hours = remaining / (60 * 60);
+		int remaining_minutes = (remaining - (remaining_hours * 60 * 60)) / 60;
+		/* remaining could conceivably be negative. This would
+		 * be a bug, but the elapsed time will display right
+		 * with the abs(). */
+		int remaining_seconds = abs (remaining % 60);
+		if (hours2 == 0)
+			return g_strdup_printf (_("%d:%02d of %d:%02d remaining"),
+						remaining_minutes, remaining_seconds,
+						minutes2, seconds2);
+		else
+			return g_strdup_printf (_("%d:%02d:%02d of %d:%02d:%02d remaining"),
+						remaining_hours, remaining_minutes, remaining_seconds,
+						hours2, minutes2, seconds2);
+	} else {
+		if (hours == 0 && hours2 == 0)
+			return g_strdup_printf (_("%d:%02d of %d:%02d"),
+						minutes, seconds,
+						minutes2, seconds2);
+		else
+			return g_strdup_printf (_("%d:%02d:%02d of %d:%02d:%02d"),
+						hours, minutes, seconds,
+						hours2, minutes2, seconds2);
+	}
+}
+
 gboolean
 rb_string_list_equal (GList *a, GList *b)
 {
@@ -729,3 +788,47 @@ rb_mime_get_friendly_name (const gchar *mime_type)
 	return name;
 }
 
+gboolean
+rb_signal_accumulator_object_handled (GSignalInvocationHint *hint,
+				      GValue *return_accu,
+				      const GValue *handler_return,
+				      gpointer dummy)
+{
+	if (handler_return == NULL ||
+	    !G_VALUE_HOLDS_OBJECT (handler_return) ||
+	    g_value_get_object (handler_return) == NULL)
+		return TRUE;
+
+	g_value_unset (return_accu);
+	g_value_init (return_accu, G_VALUE_TYPE (handler_return));
+	g_value_copy (handler_return, return_accu);
+	
+	return FALSE;
+}
+
+void
+rb_value_array_append_data (GValueArray *array, GType type, ...)
+{
+	GValue val = {0,};
+	va_list va;
+	gchar *err = NULL;
+
+	va_start (va, type);
+
+	g_value_init (&val, type);
+	G_VALUE_COLLECT (&val, va, 0, &err);
+	g_value_array_append (array, &val);
+
+	if (err)
+		rb_debug ("unable to collect GValue: %s", err);
+
+	va_end (va);
+}
+
+void
+rb_value_free (GValue *val)
+{
+	g_value_unset (val);
+	g_free (val);
+}
+
