@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -36,7 +36,8 @@ gpointer rb_null_function (gpointer dummy);
 gboolean rb_gvalue_compare (GValue *a, GValue *b);
 
 int rb_compare_gtimeval (GTimeVal *a, GTimeVal *b);
-char * rb_make_duration_string (guint duration);
+char *rb_make_duration_string (guint duration);
+char *rb_make_elapsed_time_string (guint elapsed, guint duration, gboolean show_remaining);
 
 void rb_gtk_action_popup_menu (GtkUIManager *uimanager, const char *path);
 
@@ -62,6 +63,12 @@ GList* rb_collate_hash_table_values (GHashTable *table);
 GList* rb_uri_list_parse (const char *uri_list);
 const gchar* rb_mime_get_friendly_name (const gchar *mime_type);
 
+gboolean rb_signal_accumulator_object_handled (GSignalInvocationHint *hint,
+					       GValue *return_accu,
+					       const GValue *handler_return,
+					       gpointer dummy);
+void rb_value_array_append_data (GValueArray *array, GType type, ...);
+void rb_value_free (GValue *val);
 
 G_END_DECLS
 
