@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA. 
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA 02110-1301  USA. 
  */
 
 
@@ -162,7 +162,7 @@ rb_plugins_engine_load (const gchar *file)
 	{
 		info->lang = RB_PLUGIN_LOADER_PY;
 #ifndef ENABLE_PYTHON
-		g_warning ("Cannot load python extension '%s', Rhythmbox was not "
+		rb_debug ("Cannot load python extension '%s', Rhythmbox was not "
 					"compiled with python support", file);
 		goto error;
 #endif
@@ -213,7 +213,7 @@ rb_plugins_engine_load (const gchar *file)
 						    "Authors",
 						    NULL, NULL);
 	if (info->authors == NULL)
-		rb_debug ("Could not find 'Author' in %s", file);
+		rb_debug ("Could not find 'Authors' in %s", file);
 
 
 	/* Get Copyright */
