@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -435,9 +435,8 @@ rb_podcast_manager_entry_downloaded (RhythmDBEntry *entry)
 {
 	gulong status;
 	const gchar *file_name;
-	gulong type;
+	RhythmDBEntryType type = rhythmdb_entry_get_entry_type (entry);
 
-	type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
 	g_assert (type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST);
 
 	status = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS);
@@ -590,8 +589,8 @@ rb_podcast_manager_copy_post (RBPodcastManager *pd)
 	g_free (conf_dir_name);
 
 	 if (!g_file_test (dir_name, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		if (!g_mkdir_with_parents (dir_name, 0750)) {
-			rb_debug ("Error downloading podcast: could not create local dirs");
+		if (g_mkdir_with_parents (dir_name, 0750) == -1) {
+			rb_debug ("Error downloading podcast: could not create local dirs %s", dir_name);
 			goto next_step;
 		}
 	 }
@@ -701,7 +700,7 @@ rb_podcast_manager_subscribe_feed (RBPodcastManager *pd, const char *url)
 
 	RhythmDBEntry *entry = rhythmdb_entry_lookup_by_location (pd->priv->db, valid_url);
 	if (entry) {
-		if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE) != RHYTHMDB_ENTRY_TYPE_PODCAST_FEED) {
+		if (rhythmdb_entry_get_entry_type (entry) != RHYTHMDB_ENTRY_TYPE_PODCAST_FEED) {
 			/* added as something else, probably iradio */
 			rb_error_dialog (NULL, _("URL already added"),
 					 _("The URL \"%s\" has already been added as a radio station. "
@@ -902,8 +901,8 @@ rb_podcast_manager_save_metadata (RhythmDB *db, RhythmDBEntry *entry, const char
 static void
 rb_podcast_manager_db_entry_added_cb (RBPodcastManager *pd, RhythmDBEntry *entry)
 {
+	RhythmDBEntryType type = rhythmdb_entry_get_entry_type (entry);
 
-	gulong type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
 	if (type != RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
 		return;
 	
@@ -1090,7 +1089,8 @@ download_progress_cb (GnomeVFSXferProgressInfo *info, gpointer cb_data)
 		return GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 	}
 
-	if (info->status != GNOME_VFS_XFER_PROGRESS_STATUS_OK) {
+	if (info->status != GNOME_VFS_XFER_PROGRESS_STATUS_OK ||
+	    ((info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) && (info->file_size == 0))) {
 		GValue val = {0, };
 		rb_debug ("error on download");
 		g_value_init (&val, G_TYPE_ULONG);
@@ -1120,7 +1120,7 @@ download_progress_cb (GnomeVFSXferProgressInfo *info, gpointer cb_data)
 		g_free (canon_uri);
 	}
 	
-	if (info->phase  == GNOME_VFS_XFER_PHASE_COMPLETED) {
+	if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
 		if (data->canceled != TRUE)  {
 			rb_debug ("download completed");
 			data->total_size = info->file_size;
@@ -1215,8 +1215,7 @@ static void
 rb_podcast_manager_db_entry_deleted_cb (RBPodcastManager *pd,
 					RhythmDBEntry *entry)
 {
-
-	gulong type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+	RhythmDBEntryType type = rhythmdb_entry_get_entry_type (entry);
 
 	if ((type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) && (pd->priv->remove_files == TRUE)) {
 		const char *file_name;
@@ -1388,7 +1387,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	/* processing podcast head */
 	entry = rhythmdb_entry_lookup_by_location (db, (gchar *)data->url);
 	if (entry) {
-		if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE) != RHYTHMDB_ENTRY_TYPE_PODCAST_FEED)
+		if (rhythmdb_entry_get_entry_type (entry) != RHYTHMDB_ENTRY_TYPE_PODCAST_FEED)
 			return;
 			
 		rb_debug ("Head found");
@@ -1494,7 +1493,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 							     (gulong) (item->pub_date > 0 ? item->pub_date : data->pub_date),
 							     (gulong) item->duration,
 							     item->filesize);
-			if (item->pub_date >= new_last_post) {
+			if (post_entry && item->pub_date >= new_last_post) {
 				if (item->pub_date > new_last_post) {
 					g_list_free (download_entries);
 					download_entries = NULL;
@@ -1511,13 +1510,11 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 
 		g_value_init (&status, G_TYPE_ULONG);
 		g_value_set_ulong (&status, RHYTHMDB_PODCAST_STATUS_WAITING);
-		t = download_entries;
-		while (t) {
+		for (t = download_entries; t != NULL; t = g_list_next (t)) {
 			rhythmdb_entry_set_uninserted (db, 
 						       (RhythmDBEntry*) t->data,
 						       RHYTHMDB_PROP_STATUS, 
 						       &status);
-			t = t->next;
 		}
 		g_value_unset (&status);
 	}
