@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -75,14 +75,13 @@ gchar *                 rb_podcast_manager_get_podcast_dir	(RBPodcastManager *pd
 gboolean                rb_podcast_manager_subscribe_feed    	(RBPodcastManager *pd, const gchar* url);
 void            	rb_podcast_manager_unsubscribe_feed    	(RhythmDB *db, const gchar* url);
 void			rb_podcast_manager_shutdown 		(RBPodcastManager *pd);
-gboolean                rb_podcast_manager_add_post  	  	(RhythmDB *db,
+RhythmDBEntry *         rb_podcast_manager_add_post  	  	(RhythmDB *db,
                                			         	 const char *name,
 	                                                 	 const char *title,
 	                                                 	 const char *subtitle,
 	                                                 	 const char *generator,
 	                                                 	 const char *uri,
 		        	                               	 const char *description,
-								 gulong status,
 	        	                                       	 gulong date,
 								 gulong duration,
 								 guint64 filesize);
