@@ -16,11 +16,12 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
 #define _XOPEN_SOURCE
+#define __EXTENSIONS__  /* get strptime */
 #include <string.h>
 #include <time.h>
 
@@ -28,6 +29,8 @@
 #include <libxml/SAX.h>
 #include <libxml/parserInternals.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include "rb-debug.h"
 #include "rb-podcast-parse.h" 
@@ -98,8 +101,11 @@ rb_set_channel_value (struct RBPoadcastLoadContext *ctx,
 		ctx->channel_data->summary = dvalue;
 	} else if (!strcmp (name, "description")) {
 		ctx->channel_data->description = dvalue;
-	} else if (!strcmp (name, "generator") ||
-		   !strcmp (name, "itunes:author")) {
+	} else if (!strcmp (name, "generator")) {
+		if (ctx->channel_data->author == NULL)
+			ctx->channel_data->author = dvalue;
+	} else if (!strcmp (name, "itunes:author")) {
+		g_free (ctx->channel_data->author);
 		ctx->channel_data->author = dvalue;
 	} else if (!strcmp (name, "webMaster")) {
 		ctx->channel_data->contact = dvalue;
@@ -297,6 +303,10 @@ rb_podcast_parser_start_element (struct RBPoadcastLoadContext *ctx,
         case RB_PODCAST_PARSER_STATE_CHANNEL_PROPERTY:
         case RB_PODCAST_PARSER_STATE_ITEM_PROPERTY:
         case RB_PODCAST_PARSER_STATE_IMG_PROPERTY:
+		rb_debug ("nested element inside property; treating as unknown");
+		ctx->in_unknown_elt++;
+		break;
+
         case RB_PODCAST_PARSER_STATE_END:
 		break;
 	default:
@@ -401,7 +411,7 @@ rb_podcast_parser_characters (struct RBPoadcastLoadContext *ctx,
 }
 
 
-void
+gboolean
 rb_podcast_parse_load_feed (RBPodcastChannel *data,
 			    const char *file_name)
 {
@@ -410,16 +420,12 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *info;
 	gint file_size;
-	gchar *buffer;
+	gchar *buffer = NULL;
     
 	struct RBPoadcastLoadContext *ctx = NULL;
 
 	data->url = xmlCharStrdup (file_name);
 
-	if (!gnome_vfs_initialized ()) {
-		return;
-	}
-
 	if (!g_str_has_suffix (file_name, ".rss") && !g_str_has_suffix (file_name, ".xml")) {
 		gboolean invalid_mime_type;
 
@@ -436,19 +442,41 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 			invalid_mime_type = FALSE;
 		}
 
-		if ((result != GNOME_VFS_OK) || invalid_mime_type) {
+		if ((result != GNOME_VFS_OK)) {
 			rb_debug ("Invalid mime-type in podcast feed %s", info->mime_type);
 			gnome_vfs_file_info_unref (info);
-			return;
+			return TRUE;
+		}
+
+		if (invalid_mime_type) {
+			GtkWidget *dialog;
+
+			GDK_THREADS_ENTER ();
+			dialog = gtk_message_dialog_new (NULL, 0,
+							 GTK_MESSAGE_QUESTION,
+							 GTK_BUTTONS_YES_NO,
+							 _("The URL '%s' does not appear to be a podcast feed. "
+							 "It may be the wrong URL, or the feed may be broken. "
+							 "Would you like Rhythmbox to attempt to use it anyway?"),
+							 file_name);
+			
+			if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+				invalid_mime_type = FALSE;
+
+			gtk_widget_destroy (dialog);
+			GDK_THREADS_LEAVE ();
 		}
 
 		gnome_vfs_file_info_unref (info);
+
+		if (invalid_mime_type)
+			return FALSE;
 	}
     
 	/* first download file by gnome_vfs for use gnome network configuration */
 	result = gnome_vfs_read_entire_file (file_name, &file_size, &buffer);
 	if (result != GNOME_VFS_OK)
-		return;
+		return TRUE;
 	    
 
 	/* initializing parse */
@@ -464,6 +492,14 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 	ctx->prop_value = g_string_sized_new (512);
                             
 	ctxt = xmlCreateMemoryParserCtxt (buffer, file_size);
+	if (ctx == NULL) {
+		g_free (sax_handler);
+		g_free (buffer);
+		g_string_free (ctx->prop_value, TRUE);
+		g_free (ctx);
+		return FALSE;
+	}
+
 	ctx->xmlctx = ctxt;
 	ctxt->userData = ctx;
 	ctxt->sax = sax_handler;
@@ -478,6 +514,7 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 	g_free (ctx);
 
 	data->posts = g_list_reverse (data->posts);
+	return TRUE;
 }
 
 static uintmax_t
@@ -489,6 +526,18 @@ rb_podcast_parse_date (const char *date_str)
 	/* RFC 2822 date format */
 	result = strptime (date_str, "%a, %d %b %Y %T", &tm);
 
+	/* same as above, but without comma */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a %d %b %Y %T", &tm);
+	}
+
+	/* close-to-RFC 2822, but with extra 0 */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a, %d %b %Y 0%T", &tm);
+	}
+
 	/* format without weekday */
 	if (result == NULL) {
 		memset (&tm, 0, sizeof (struct tm));
@@ -507,6 +556,12 @@ rb_podcast_parse_date (const char *date_str)
 		result = strptime (date_str, "%Y-%m-%d %T", &tm);
 	}
 
+	/* ISO date like without timezone */
+	if (result == NULL) {
+	memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%Y-%m-%d", &tm);
+	}
+
 	/* Broken weekday short names */
 	if (result == NULL) {
 		char *tmp;
@@ -520,6 +575,38 @@ rb_podcast_parse_date (const char *date_str)
 		}
 	}
 
+	/* format with timezone offset from GMT */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a %b %d %T %z %Y", &tm);
+	}
+
+	/* format with timezone name */
+	if (result == NULL) {
+		char *tmp;
+
+		memset (&tm, 0, sizeof (struct tm));
+	
+		/* match first part of time string */
+		result = strptime (date_str, "%a %b %d %T ", &tm);
+
+		/* look for anything with a timezone name-like format
+		   i.e. at least one all caps alphabetical character */
+		if (result != NULL) {
+			size_t n;
+
+			n = strspn(result, "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
+			tmp = result+n;
+
+			/* make sure there was at least one character that matched */
+			if ((tmp != NULL) && n > 0) 
+				/* remaining part must be the year */
+				result = strptime (tmp, "%Y", &tm);
+			else 
+				result = NULL;
+		}
+	}
+
 	if (result == NULL) {
 		memset (&tm, 0, sizeof (struct tm));	
 		rb_debug ("unable to convert date string %s", date_str);
