@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -38,6 +38,18 @@ G_BEGIN_DECLS
 #define RHYTHMDB_IS_QUERY_MODEL_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RHYTHMDB_TYPE_QUERY_MODEL))
 #define RHYTHMDB_QUERY_MODEL_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RHYTHMDB_TYPE_QUERY_MODEL, RhythmDBQueryModelClass))
 
+
+GType rhythmdb_query_model_limit_type_get_type (void);
+#define RHYTHMDB_TYPE_QUERY_MODEL_LIMIT_TYPE (rhythmdb_query_model_limit_type_get_type ())
+
+typedef enum {
+	RHYTHMDB_QUERY_MODEL_LIMIT_NONE,
+	RHYTHMDB_QUERY_MODEL_LIMIT_COUNT,
+	RHYTHMDB_QUERY_MODEL_LIMIT_SIZE,
+	RHYTHMDB_QUERY_MODEL_LIMIT_TIME,
+} RhythmDBQueryModelLimitType;
+
+
 typedef struct RhythmDBQueryModelPrivate RhythmDBQueryModelPrivate;
 
 #define RHYTHMDB_QUERY_MODEL_SUGGESTED_UPDATE_CHUNK 32
@@ -65,6 +77,8 @@ typedef struct
 					 int position);
 	void    (*entry_removed)        (RhythmDBQueryModel *model,
 					 RhythmDBEntry *entry);
+	void	(*post_entry_delete)	(RhythmDBQueryModel *model,
+					 RhythmDBEntry *entry);
 
 } RhythmDBQueryModelClass;
 
@@ -72,11 +86,16 @@ GType			rhythmdb_query_model_get_type	(void);
 
 RhythmDBQueryModel *	rhythmdb_query_model_new	(RhythmDB *db, GPtrArray *query,
 							 GCompareDataFunc sort_func,
-							 RhythmDBPropType sort_prop_id,
+							 gpointer sort_data,
+							 GDestroyNotify sort_data_destroy,
 							 gboolean sort_reverse);
 
 RhythmDBQueryModel *	rhythmdb_query_model_new_empty	(RhythmDB *db);
 
+void			rhythmdb_query_model_chain	(RhythmDBQueryModel *child, 
+							 RhythmDBQueryModel *base, 
+							 gboolean import_entries);
+
 void			rhythmdb_query_model_add_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry, gint index);
 
 gboolean		rhythmdb_query_model_remove_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry);
@@ -101,44 +120,47 @@ RhythmDBEntry *		rhythmdb_query_model_get_next_from_entry(RhythmDBQueryModel *mo
 								 RhythmDBEntry *entry);
 RhythmDBEntry *		rhythmdb_query_model_get_previous_from_entry(RhythmDBQueryModel *model,
 								     RhythmDBEntry *entry);
-char *			rhythmdb_query_model_compute_status_normal (RhythmDBQueryModel *model);
+char *			rhythmdb_query_model_compute_status_normal (RhythmDBQueryModel *model,
+								    const char *singular,
+								    const char *plural);
 
 void			rhythmdb_query_model_set_sort_order (RhythmDBQueryModel *model,
 							     GCompareDataFunc sort_func,
-							     RhythmDBPropType sort_prop_id,
+							     gpointer sort_data,
+							     GDestroyNotify sort_data_destroy,
 							     gboolean sort_reverse);
 
 void			rhythmdb_query_model_reapply_query (RhythmDBQueryModel *model, gboolean filter);
 
 gint 			rhythmdb_query_model_location_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-								 RhythmDBQueryModel *model);
+								 gpointer data);
 
 gint 			rhythmdb_query_model_string_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-							       RhythmDBQueryModel *model);
+							       gpointer data);
 
 gint 			rhythmdb_query_model_title_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-							      RhythmDBQueryModel *model);
+							      gpointer data);
 
 gint 			rhythmdb_query_model_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-							      RhythmDBQueryModel *model);
+							      gpointer data);
 
 gint 			rhythmdb_query_model_artist_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-							       RhythmDBQueryModel *model);
+							       gpointer data);
 				
 gint 			rhythmdb_query_model_genre_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-							      RhythmDBQueryModel *model); 
+							      gpointer data); 
 
 gint 			rhythmdb_query_model_track_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-							      RhythmDBQueryModel *model);
+							      gpointer data);
 
 gint 			rhythmdb_query_model_double_ceiling_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-								       RhythmDBQueryModel *model);
+								       gpointer data);
 
 gint 			rhythmdb_query_model_ulong_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-							      RhythmDBQueryModel *model);
+							      gpointer data);
 
 gint 			rhythmdb_query_model_date_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-							     RhythmDBQueryModel *model);
+							     gpointer data);
 G_END_DECLS
 
 #endif /* __RHYTHMDB_QUERY_MODEL_H */
