@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -40,8 +40,8 @@
 #include <libxml/SAX.h>
 #include <libxml/parserInternals.h>
 
+#include "rhythmdb-private.h"
 #include "rhythmdb-tree.h"
-#include "rhythmdb-query-model.h"
 #include "rhythmdb-property-model.h"
 #include "rb-debug.h"
 #include "rb-util.h"
@@ -74,7 +74,8 @@ static void rhythmdb_tree_entry_delete_by_type (RhythmDB *adb, RhythmDBEntryType
 static RhythmDBEntry * rhythmdb_tree_entry_lookup_by_location (RhythmDB *db, const char *uri);
 static void rhythmdb_tree_entry_foreach (RhythmDB *adb, GFunc func, gpointer user_data);
 static void rhythmdb_tree_do_full_query (RhythmDB *db, GPtrArray *query,
-					 GtkTreeModel *main_model, gboolean *cancel);
+					 RhythmDBQueryResults *results,
+					 gboolean *cancel);
 static gboolean rhythmdb_tree_evaluate_query (RhythmDB *adb, GPtrArray *query,
 				       RhythmDBEntry *aentry);
 
@@ -94,7 +95,7 @@ static void rhythmdb_hash_tree_foreach (RhythmDB *adb,
 					gpointer data);
 
 
-#define RHYTHMDB_TREE_XML_VERSION "1.0"
+#define RHYTHMDB_TREE_XML_VERSION "1.2"
 
 static void destroy_tree_property (RhythmDBTreeProperty *prop);
 static RhythmDBTreeProperty *get_or_create_album (RhythmDBTree *db, RhythmDBTreeProperty *artist,
@@ -208,8 +209,11 @@ struct RhythmDBTreeLoadContext
 	RhythmDBEntry *entry;
 	GString *buf;
 	RhythmDBPropType propid;
+	gint batch_count;
 
+	/* updating */
 	gboolean has_date;
+	gboolean canonicalise_uris;
 };
 
 static void
@@ -228,29 +232,42 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 	{
 	case RHYTHMDB_TREE_PARSER_STATE_START:
 	{
-		if (!strcmp (name, "rhythmdb"))
+		if (!strcmp (name, "rhythmdb")) {
 			ctx->state = RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB;
-		else
+			for (; *attrs; attrs +=2) {
+				if (!strcmp (*attrs, "version")) {
+					const char *version = *(attrs+1);
+
+					if (!strcmp (version, "1.0") || !strcmp (version, "1.1")) {
+						ctx->canonicalise_uris = TRUE;
+						rb_debug ("old version of rhythmdb, performing URI canonicalisation for all entries");
+					} else if (!strcmp (version, "1.2")) {
+						/* current version*/
+					} else {
+						/* too new */
+						g_assert_not_reached ();
+					}
+				} else {
+					g_assert_not_reached ();
+				}
+			}
+
+		} else {
 			ctx->in_unknown_elt = TRUE;
+		}
+
 		break;
 	}
 	case RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB:
 	{
 		if (!strcmp (name, "entry")) {
-			RhythmDBEntryType type = -1;
+			RhythmDBEntryType type = RHYTHMDB_ENTRY_TYPE_INVALID;
 			gboolean type_set = FALSE;
 			for (; *attrs; attrs +=2) {
 				if (!strcmp (*attrs, "type")) {
 					const char *typename = *(attrs+1);
-					if (!strcmp (typename, "song"))
-						type = RHYTHMDB_ENTRY_TYPE_SONG;
-					else if (!strcmp (typename, "iradio"))
-						type = RHYTHMDB_ENTRY_TYPE_IRADIO_STATION;
-					else if (!strcmp (typename, "podcast-post"))
-						type = RHYTHMDB_ENTRY_TYPE_PODCAST_POST;
-					else if (!strcmp (typename, "podcast-feed"))
-						type = RHYTHMDB_ENTRY_TYPE_PODCAST_FEED;
-					else
+					type = rhythmdb_entry_type_get_by_name (typename);
+					if (!type)
 						return;
 					type_set = TRUE;
 					break;
@@ -259,6 +276,7 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 			g_assert (type_set);
 			ctx->state = RHYTHMDB_TREE_PARSER_STATE_ENTRY;
 			ctx->entry = rhythmdb_entry_allocate (RHYTHMDB (ctx->db), type);
+			ctx->entry->flags |= RHYTHMDB_ENTRY_TREE_LOADING;
 			ctx->has_date = FALSE;
 		} else
 			ctx->in_unknown_elt = TRUE;
@@ -283,18 +301,6 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 	}
 }
 
-static gulong
-parse_ulong (const char *buffer)
-{
-	guint64 val;
-
-	val = g_ascii_strtoull (buffer, NULL, 10);
-	if (val == G_MAXUINT64)
-		return 0;
-	else
-		return MIN (val, G_MAXUINT32);
-}
-
 static void
 rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const char *name)
 {
@@ -322,19 +328,49 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 			rb_debug ("pre-Date entry found, causing re-read");
 			ctx->entry->mtime = 0;
 		}
-		if (ctx->entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED && ctx->entry->podcast->post_time == 0) {
+		if (ctx->entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED) {
+			RhythmDBPodcastFields *podcast = RHYTHMDB_ENTRY_GET_TYPE_DATA (ctx->entry, RhythmDBPodcastFields);
 			/* Handle upgrades from 0.9.2.
 			 * Previously, last-seen for podcast feeds was the time of the last post,
 			 * and post-time was unused.  Now, we want last-seen to be the time we
 			 * last updated the feed, and post-time to be the time of the last post.
 			 */
-			ctx->entry->podcast->post_time = ctx->entry->last_seen;
+			if (podcast->post_time == 0) {
+				podcast->post_time = ctx->entry->last_seen;
+			}
 		}
 		
 		if (ctx->entry->location != NULL) {
-			rhythmdb_tree_entry_new (RHYTHMDB (ctx->db), ctx->entry);
-			rhythmdb_entry_insert (RHYTHMDB (ctx->db), ctx->entry);
-			rhythmdb_commit (RHYTHMDB (ctx->db));
+			RhythmDBEntry *entry;
+
+			entry = g_hash_table_lookup (ctx->db->priv->entries, ctx->entry->location);
+			if (entry == NULL) {
+				rhythmdb_tree_entry_new (RHYTHMDB (ctx->db), ctx->entry);
+				rhythmdb_entry_insert (RHYTHMDB (ctx->db), ctx->entry);
+				if (++ctx->batch_count == RHYTHMDB_QUERY_MODEL_SUGGESTED_UPDATE_CHUNK) {
+					rhythmdb_commit (RHYTHMDB (ctx->db));
+					ctx->batch_count = 0;
+				}
+			} else {
+				rb_debug ("found entry with duplicate location %s. merging metadata", ctx->entry->location);
+				entry->play_count += ctx->entry->play_count;
+
+				if (entry->rating < 0.01)
+					entry->rating = ctx->entry->rating;
+				else if (ctx->entry->rating > 0.01)
+					entry->rating = (entry->rating + ctx->entry->rating) / 2;
+
+				if (ctx->entry->last_played > entry->last_played)
+					entry->last_played = ctx->entry->last_played;
+
+				if (ctx->entry->first_seen < entry->first_seen)
+					entry->first_seen = ctx->entry->first_seen;
+
+				if (ctx->entry->last_seen > entry->last_seen)
+					entry->last_seen = ctx->entry->last_seen;
+				
+				rhythmdb_entry_unref (RHYTHMDB (ctx->db), ctx->entry);
+			}
 		} else {
 			rb_debug ("found entry without location");
 			rhythmdb_entry_unref (RHYTHMDB (ctx->db), ctx->entry);
@@ -344,134 +380,41 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 	}
 	case RHYTHMDB_TREE_PARSER_STATE_ENTRY_PROPERTY:
 	{
-		/* Handle indexed properties. */
-		switch (ctx->propid)
-		{
-		case RHYTHMDB_PROP_TYPE:
-			g_assert_not_reached ();
-			break;
-		case RHYTHMDB_PROP_TITLE:
-			ctx->entry->title = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_GENRE:
-			ctx->entry->genre = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_ARTIST:
-			ctx->entry->artist = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_ALBUM:
-			ctx->entry->album = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_TRACK_NUMBER:
-			ctx->entry->tracknum = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_DISC_NUMBER:
-			ctx->entry->discnum = parse_ulong (ctx->buf->str);
-			break;
+		GValue value = {0,};
+		gboolean set = FALSE;
+		gboolean skip = FALSE;
+
+		/* special case some properties for upgrade handling etc. */
+		switch (ctx->propid) {
 		case RHYTHMDB_PROP_DATE:
-		{
-			gulong value = parse_ulong (ctx->buf->str);
-			
-			if (value > 0)
-				ctx->entry->date = g_date_new_julian (value);
-			else
-				;
 			ctx->has_date = TRUE;
 			break;
-		}
-		case RHYTHMDB_PROP_DURATION:
-			ctx->entry->duration = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_FILE_SIZE:
-			ctx->entry->file_size = parse_ulong (ctx->buf->str);
-			break;
 		case RHYTHMDB_PROP_LOCATION:
-			ctx->entry->location = g_strdup (ctx->buf->str);
+			if (ctx->canonicalise_uris) {
+				char *canon = rb_canonicalise_uri (ctx->buf->str);
+
+				g_value_init (&value, G_TYPE_STRING);
+				g_value_take_string (&value, canon);
+				set = TRUE;
+			}
 			break;
 		case RHYTHMDB_PROP_MOUNTPOINT:
-			/* remove this from old podcast-post entries */
-			if (!g_str_has_prefix (ctx->buf->str, "http://"))
-				ctx->entry->mountpoint = rb_refstring_new (ctx->buf->str);
+			/* fix old podcast posts */
+			if (g_str_has_prefix (ctx->buf->str, "http://"))
+				skip = TRUE;
 			break;
-		case RHYTHMDB_PROP_MTIME:
-			ctx->entry->mtime = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_FIRST_SEEN:
-			ctx->entry->first_seen = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_LAST_SEEN:
-			ctx->entry->last_seen = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_RATING:
-			ctx->entry->rating = g_ascii_strtod (ctx->buf->str, NULL);
-			break;
-		case RHYTHMDB_PROP_PLAY_COUNT:
-			ctx->entry->play_count = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_LAST_PLAYED:
-			ctx->entry->last_played = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_BITRATE:
-			ctx->entry->bitrate = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_TRACK_GAIN:
-			ctx->entry->track_gain = g_ascii_strtod (ctx->buf->str, NULL);
-			break;
-		case RHYTHMDB_PROP_TRACK_PEAK:
-			ctx->entry->track_peak = g_ascii_strtod (ctx->buf->str, NULL);
-			break;
-		case RHYTHMDB_PROP_ALBUM_GAIN:
-			ctx->entry->album_gain = g_ascii_strtod (ctx->buf->str, NULL);
-			break;
-		case RHYTHMDB_PROP_ALBUM_PEAK:
-			ctx->entry->album_peak = g_ascii_strtod (ctx->buf->str, NULL);
-			break;
-		case RHYTHMDB_PROP_MIMETYPE:
-			ctx->entry->mimetype = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_STATUS:
-			ctx->entry->podcast->status = parse_ulong (ctx->buf->str);
-			break;			
-		case RHYTHMDB_PROP_DESCRIPTION:
-			ctx->entry->podcast->description = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_SUBTITLE:
-			ctx->entry->podcast->subtitle = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_SUMMARY:
-			ctx->entry->podcast->summary = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_LANG:
-			ctx->entry->podcast->lang = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_COPYRIGHT:
-			ctx->entry->podcast->copyright = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_IMAGE:
-			ctx->entry->podcast->image = rb_refstring_new (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_POST_TIME:			
-			ctx->entry->podcast->post_time = parse_ulong (ctx->buf->str);
-			break;
-		case RHYTHMDB_PROP_TITLE_SORT_KEY:
-		case RHYTHMDB_PROP_GENRE_SORT_KEY:
-		case RHYTHMDB_PROP_ARTIST_SORT_KEY:
-		case RHYTHMDB_PROP_ALBUM_SORT_KEY:
-		case RHYTHMDB_PROP_TITLE_FOLDED:
-		case RHYTHMDB_PROP_GENRE_FOLDED:
-		case RHYTHMDB_PROP_ARTIST_FOLDED:
-		case RHYTHMDB_PROP_ALBUM_FOLDED:
-		case RHYTHMDB_PROP_LAST_PLAYED_STR:
-		case RHYTHMDB_PROP_HIDDEN:
-		case RHYTHMDB_PROP_PLAYBACK_ERROR:
-		case RHYTHMDB_PROP_FIRST_SEEN_STR:
-		case RHYTHMDB_PROP_SEARCH_MATCH:
-		case RHYTHMDB_NUM_PROPERTIES:
-			g_assert_not_reached ();
+		default:
 			break;
 		}
-			
-		rhythmdb_entry_sync_mirrored (RHYTHMDB (ctx->db), ctx->entry, ctx->propid);
+
+		if (!skip) {
+			if (!set) {
+				rhythmdb_read_encoded_property (RHYTHMDB (ctx->db), ctx->buf->str, ctx->propid, &value);
+			}
+
+			rhythmdb_entry_set_internal (RHYTHMDB (ctx->db), ctx->entry, FALSE, ctx->propid, &value);
+			g_value_unset (&value);
+		}
 
 		ctx->state = RHYTHMDB_TREE_PARSER_STATE_ENTRY;
 		break;
@@ -533,6 +476,9 @@ rhythmdb_tree_load (RhythmDB *rdb, gboolean *die)
 		xmlParseDocument (ctxt);
 		ctxt->sax = NULL;
 		xmlFreeParserCtxt (ctxt);
+
+		if (ctx->batch_count)
+			rhythmdb_commit (RHYTHMDB (ctx->db));
 			
 	}
 	g_string_free (ctx->buf, TRUE);
@@ -630,6 +576,13 @@ save_entry_ulong (struct RhythmDBTreeSaveContext *ctx,
 	write_elt_name_close (ctx, elt_name);
 }
 
+static void
+save_entry_boolean (struct RhythmDBTreeSaveContext *ctx,
+		    const xmlChar *elt_name, gboolean val)
+{
+	save_entry_ulong (ctx, elt_name, val ? 1 : 0, FALSE);
+}
+
 static void
 save_entry_uint64 (struct RhythmDBTreeSaveContext *ctx, const xmlChar *elt_name,
 		   guint64 num)
@@ -667,10 +620,15 @@ static void
 save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveContext *ctx)
 {
 	RhythmDBPropType i;
+	RhythmDBPodcastFields *podcast = NULL;
 
 	if (ctx->error)
 		return;
 
+	if (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED ||
+	    entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
+		podcast = RHYTHMDB_ENTRY_GET_TYPE_DATA (entry, RhythmDBPodcastFields);
+
 	RHYTHMDB_FWRITE_STATICSTR ("  <entry type=\"", ctx->handle, ctx->error);
 
 	if (entry->type == RHYTHMDB_ENTRY_TYPE_SONG) {
@@ -681,8 +639,11 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 		RHYTHMDB_FWRITE_STATICSTR ("podcast-post", ctx->handle, ctx->error);
 	} else if (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED) {
 		RHYTHMDB_FWRITE_STATICSTR ("podcast-feed", ctx->handle, ctx->error);
-	} else
+	} else if (entry->type == RHYTHMDB_ENTRY_TYPE_IGNORE) {
+		RHYTHMDB_FWRITE_STATICSTR ("ignore", ctx->handle, ctx->error);
+	} else {
 		g_assert_not_reached ();
+	}
 
 	RHYTHMDB_FWRITE_STATICSTR ("\">\n", ctx->handle, ctx->error);
 		
@@ -718,8 +679,8 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 			save_entry_ulong (ctx, elt_name, entry->discnum, FALSE);
 			break;
 		case RHYTHMDB_PROP_DATE:
-			if (entry->date)
-				save_entry_ulong (ctx, elt_name, g_date_get_julian (entry->date), TRUE);
+			if (g_date_valid (&entry->date))
+				save_entry_ulong (ctx, elt_name, g_date_get_julian (&entry->date), TRUE);
 			else
 				save_entry_ulong (ctx, elt_name, 0, TRUE);
 			break;
@@ -775,37 +736,43 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 		case RHYTHMDB_PROP_LAST_PLAYED:
 			save_entry_ulong (ctx, elt_name, entry->last_played, FALSE);
 			break;
+		case RHYTHMDB_PROP_HIDDEN:
+			{
+				gboolean hidden = ((entry->flags & RHYTHMDB_ENTRY_HIDDEN) != 0);
+				save_entry_boolean (ctx, elt_name, hidden);
+			}
+			break;
 		case RHYTHMDB_PROP_STATUS:
-			if (entry->podcast)
-				save_entry_ulong (ctx, elt_name, entry->podcast->status, FALSE);
+			if (podcast)
+				save_entry_ulong (ctx, elt_name, podcast->status, FALSE);
 			break;
 		case RHYTHMDB_PROP_DESCRIPTION:
-			if (entry->podcast && entry->podcast->description)
-				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->description));
+			if (podcast && podcast->description)
+				save_entry_string(ctx, elt_name, rb_refstring_get (podcast->description));
 			break;
 		case RHYTHMDB_PROP_SUBTITLE:
-			if (entry->podcast && entry->podcast->subtitle)
-				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->subtitle));
+			if (podcast && podcast->subtitle)
+				save_entry_string(ctx, elt_name, rb_refstring_get (podcast->subtitle));
 			break;
 		case RHYTHMDB_PROP_SUMMARY:
-			if (entry->podcast && entry->podcast->summary)
-				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->summary));
+			if (podcast && podcast->summary)
+				save_entry_string(ctx, elt_name, rb_refstring_get (podcast->summary));
 			break;
 		case RHYTHMDB_PROP_LANG:
-			if (entry->podcast && entry->podcast->lang)
-				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->lang));
+			if (podcast && podcast->lang)
+				save_entry_string(ctx, elt_name, rb_refstring_get (podcast->lang));
 			break;
 		case RHYTHMDB_PROP_COPYRIGHT:
-			if (entry->podcast && entry->podcast->copyright)
-				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->copyright));
+			if (podcast && podcast->copyright)
+				save_entry_string(ctx, elt_name, rb_refstring_get (podcast->copyright));
 			break;
 		case RHYTHMDB_PROP_IMAGE:
-			if (entry->podcast && entry->podcast->image)
-				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->image));
+			if (podcast && podcast->image)
+				save_entry_string(ctx, elt_name, rb_refstring_get (podcast->image));
 			break;
 		case RHYTHMDB_PROP_POST_TIME:
-			if (entry->podcast)
-				save_entry_ulong (ctx, elt_name, entry->podcast->post_time, FALSE);
+			if (podcast)
+				save_entry_ulong (ctx, elt_name, podcast->post_time, FALSE);
 			break;			
 		case RHYTHMDB_PROP_TITLE_SORT_KEY:
 		case RHYTHMDB_PROP_GENRE_SORT_KEY:
@@ -816,10 +783,11 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 		case RHYTHMDB_PROP_ARTIST_FOLDED:
 		case RHYTHMDB_PROP_ALBUM_FOLDED:
 		case RHYTHMDB_PROP_LAST_PLAYED_STR:
-		case RHYTHMDB_PROP_HIDDEN:
 		case RHYTHMDB_PROP_PLAYBACK_ERROR:
 		case RHYTHMDB_PROP_FIRST_SEEN_STR:
+		case RHYTHMDB_PROP_LAST_SEEN_STR:
 		case RHYTHMDB_PROP_SEARCH_MATCH:
+		case RHYTHMDB_PROP_YEAR:
 		case RHYTHMDB_NUM_PROPERTIES:
 			break;
 		}
@@ -845,7 +813,7 @@ rhythmdb_tree_save (RhythmDB *rdb)
 	f = fopen (savepath->str, "w");
 
 	if (!f) {
-		g_critical ("Can't save XML: %s", g_strerror (errno));
+		g_warning ("Can't save XML: %s", g_strerror (errno));
 		goto out;
 	}
 
@@ -853,41 +821,44 @@ rhythmdb_tree_save (RhythmDB *rdb)
 	ctx.handle = f;
 	ctx.error = NULL;
 	RHYTHMDB_FWRITE_STATICSTR ("<?xml version=\"1.0\" standalone=\"yes\"?>\n"
-				   "<rhythmdb version=\"" RHYTHMDB_TREE_XML_VERSION "\">", 
+				   "<rhythmdb version=\"" RHYTHMDB_TREE_XML_VERSION "\">\n", 
 				   ctx.handle, ctx.error);
 
-	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_SONG, 
-				    (RBTreeEntryItFunc)save_entry, 
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_SONG,
+				    (RBTreeEntryItFunc)save_entry,
 				    NULL, NULL, NULL, &ctx);
-	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, 
-				    (RBTreeEntryItFunc)save_entry, 
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
+				    (RBTreeEntryItFunc)save_entry,
 				    NULL, NULL, NULL, &ctx);
-	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_POST, 
-				    (RBTreeEntryItFunc)save_entry, 
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
+				    (RBTreeEntryItFunc)save_entry,
 				    NULL, NULL, NULL, &ctx);
-	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_FEED, 
-				    (RBTreeEntryItFunc)save_entry, 
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_FEED,
+				    (RBTreeEntryItFunc)save_entry,
+				    NULL, NULL, NULL, &ctx);
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_IGNORE,
+				    (RBTreeEntryItFunc)save_entry,
 				    NULL, NULL, NULL, &ctx);
 
 	RHYTHMDB_FWRITE_STATICSTR ("</rhythmdb>\n", ctx.handle, ctx.error);
 
 	if (fclose (f) < 0) {
-		g_critical ("Couldn't close %s: %s",
-			    savepath->str,
-			    g_strerror (errno));
+		g_warning ("Couldn't close %s: %s",
+			   savepath->str,
+			   g_strerror (errno));
 		unlink (savepath->str);
 		goto out;
 	}
 
 	if (ctx.error != NULL) {
-		g_critical ("Writing to the database failed: %s", ctx.error);
+		g_warning ("Writing to the database failed: %s", ctx.error);
 		g_free (ctx.error);
 		unlink (savepath->str);
 	} else {
 		if (rename (savepath->str, name) < 0) {
-			g_critical ("Couldn't rename %s to %s: %s",
-				    name, savepath->str,
-				    g_strerror (errno));
+			g_warning ("Couldn't rename %s to %s: %s",
+				   name, savepath->str,
+				   g_strerror (errno));
 			unlink (savepath->str);
 		}
 	}
@@ -960,7 +931,10 @@ rhythmdb_tree_entry_new (RhythmDB *rdb, RhythmDBEntry *entry)
 	artist = get_or_create_artist (db, genre, entry->artist);
 	set_entry_album (db, entry, artist, entry->album);
 
+	/* this accounts for the initial reference on the entry */
 	g_hash_table_insert (db->priv->entries, entry->location, entry);
+	
+	entry->flags &= ~RHYTHMDB_ENTRY_TREE_LOADING;
 }
 
 static RhythmDBTreeProperty *
@@ -978,7 +952,7 @@ get_genres_hash_for_type (RhythmDBTree *db, RhythmDBEntryType type)
 {
 	GHashTable *table;
 
-	table = g_hash_table_lookup (db->priv->genres, GINT_TO_POINTER (type));
+	table = g_hash_table_lookup (db->priv->genres, type);
 	if (table == NULL) {
 		table = g_hash_table_new_full (rb_refstring_hash,
 					       rb_refstring_equal,
@@ -989,7 +963,7 @@ get_genres_hash_for_type (RhythmDBTree *db, RhythmDBEntryType type)
 			return NULL;
 		}
 		g_hash_table_insert (db->priv->genres, 
-				     GINT_TO_POINTER (type), 
+				     type,
 				     table);
 	} 
 	return table;
@@ -1135,6 +1109,12 @@ rhythmdb_tree_entry_set (RhythmDB *adb, RhythmDBEntry *entry,
 
 	type = entry->type;
 
+	/* don't process changes to entries we're loading, we'll get them
+	 * when the entry is complete.
+	 */
+	if (entry->flags & RHYTHMDB_ENTRY_TREE_LOADING)
+		return FALSE;
+
 	/* Handle special properties */
 	switch (propid)
 	{
@@ -1246,6 +1226,7 @@ rhythmdb_tree_entry_delete (RhythmDB *adb, RhythmDBEntry *entry)
 	g_assert (g_hash_table_lookup (db->priv->entries, entry->location) != NULL);
 
 	g_hash_table_remove (db->priv->entries, entry->location);
+	rhythmdb_entry_unref (adb, entry);
 }
 
 typedef struct {
@@ -1262,6 +1243,7 @@ remove_one_song (gchar *uri, RhythmDBEntry *entry,
 	if (entry->type == ctxt->type) {
 		rhythmdb_emit_entry_deleted (ctxt->db, entry);
 		remove_entry_from_album (RHYTHMDB_TREE (ctxt->db), entry); 
+		rhythmdb_entry_unref (ctxt->db, entry);
 		return TRUE;
 	}
 	return FALSE;
@@ -1318,7 +1300,7 @@ rhythmdb_tree_evaluate_query (RhythmDB *adb, GPtrArray *query,
 			if (evaluate_conjunctive_subquery (db, query, last_disjunction, i, entry))
 				return TRUE;
 
-			last_disjunction = i;
+			last_disjunction = i + 1;
 		}
 	}
 	if (evaluate_conjunctive_subquery (db, query, last_disjunction, query->len, entry))
@@ -1353,7 +1335,13 @@ rhythmdb_tree_evaluate_query (RhythmDB *adb, GPtrArray *query,
 				    g_value_get_double (data->val)) \
 					return FALSE; \
 				break; \
+                        case G_TYPE_POINTER: \
+                                if (rhythmdb_entry_get_pointer (entry, data->propid) OP \
+                                    g_value_get_pointer (data->val)) \
+                                        return FALSE; \
+                                break; \
 			default: \
+                                g_warning ("Unexpected type: %s", g_type_name (rhythmdb_get_property_type (db, data->propid))); \
 				g_assert_not_reached (); \
 			}
 
@@ -1440,11 +1428,31 @@ evaluate_conjunctive_subquery (RhythmDBTree *dbtree, GPtrArray *query,
 			relative_time = g_value_get_ulong (data->val);
 			g_get_current_time  (&current_time);
 
-			if (data->type == RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN)
-				return (rhythmdb_entry_get_ulong (entry, data->propid) >= (current_time.tv_sec - relative_time));
-			else
-				return (rhythmdb_entry_get_ulong (entry, data->propid) < (current_time.tv_sec - relative_time));
+			if (data->type == RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN) {
+				if (!(rhythmdb_entry_get_ulong (entry, data->propid) >= (current_time.tv_sec - relative_time)))
+					return FALSE;
+			} else {
+				if (!(rhythmdb_entry_get_ulong (entry, data->propid) < (current_time.tv_sec - relative_time)))
+					return FALSE;
+			}
+			break;
+		}
+		case RHYTHMDB_QUERY_PROP_PREFIX:
+		case RHYTHMDB_QUERY_PROP_SUFFIX:
+		{
+			const char *value_s;
+			const char *entry_s;
+			
+			g_assert (rhythmdb_get_property_type (db, data->propid) == G_TYPE_STRING);
 
+			value_s = g_value_get_string (data->val);
+			entry_s = rhythmdb_entry_get_string (entry, data->propid);
+
+			if (data->type == RHYTHMDB_QUERY_PROP_PREFIX && !g_str_has_prefix (entry_s, value_s))
+				return FALSE;
+			if (data->type == RHYTHMDB_QUERY_PROP_SUFFIX && !g_str_has_suffix (entry_s, value_s))
+				return FALSE;
+			
 			break;
 		}
 		case RHYTHMDB_QUERY_PROP_LIKE:
@@ -1477,10 +1485,8 @@ evaluate_conjunctive_subquery (RhythmDBTree *dbtree, GPtrArray *query,
 			/* Fall through */
 		}
 		case RHYTHMDB_QUERY_PROP_EQUALS:
-		{
 			RHYTHMDB_PROPERTY_COMPARE (!=)
 			break;
-		}
 		case RHYTHMDB_QUERY_PROP_GREATER:
 			RHYTHMDB_PROPERTY_COMPARE (<)
 			break;
@@ -1694,10 +1700,12 @@ conjunctive_query (RhythmDBTree *db, GPtrArray *query,
 		
 		g_ptr_array_remove_index_fast (query, type_query_idx);
 		
-		etype = g_value_get_ulong (qdata->val);
+		etype = g_value_get_pointer (qdata->val);
 		genres = get_genres_hash_for_type (db, etype);
 		if (genres != NULL) {
 			conjunctive_query_genre (db, genres, traversal_data);
+		} else {
+			g_assert_not_reached ();
 		}
 	} else {
 		/* FIXME */
@@ -1750,7 +1758,7 @@ struct RhythmDBTreeQueryGatheringData
 	RhythmDBTree *db;
 	GPtrArray *queue;
 	GHashTable *entries;
-	RhythmDBQueryModel *main_model;
+	RhythmDBQueryResults *results;
 };
 
 static void
@@ -1759,8 +1767,10 @@ do_query_recurse (RhythmDBTree *db, GPtrArray *query, RhythmDBTreeTraversalFunc
 {
 	GList *conjunctions, *tem;
 
+	if (query == NULL)
+		return;
+	
 	conjunctions = split_query_by_disjunctions (db, query);
-
 	rb_debug ("doing recursive query, %d conjunctions", g_list_length (conjunctions));
 
 	if (conjunctions == NULL)
@@ -1796,7 +1806,7 @@ handle_entry_match (RhythmDB *db, RhythmDBEntry *entry,
 
 	g_ptr_array_add (data->queue, entry);
 	if (data->queue->len > RHYTHMDB_QUERY_MODEL_SUGGESTED_UPDATE_CHUNK) {
-		rhythmdb_query_model_add_entries (data->main_model, data->queue);
+		rhythmdb_query_results_add_results (data->results, data->queue);
 		data->queue = g_ptr_array_new ();
 	}
 }
@@ -1804,18 +1814,18 @@ handle_entry_match (RhythmDB *db, RhythmDBEntry *entry,
 static void
 rhythmdb_tree_do_full_query (RhythmDB *adb,
 			     GPtrArray *query,
-			     GtkTreeModel *main_model,
+			     RhythmDBQueryResults *results,
 			     gboolean *cancel)
 {
 	RhythmDBTree *db = RHYTHMDB_TREE (adb);
 	struct RhythmDBTreeQueryGatheringData *data = g_new0 (struct RhythmDBTreeQueryGatheringData, 1);
 
-	data->main_model = RHYTHMDB_QUERY_MODEL (main_model);
+	data->results = results;
 	data->queue = g_ptr_array_new ();
 
 	do_query_recurse (db, query, (RhythmDBTreeTraversalFunc) handle_entry_match, data, cancel);
 
-	rhythmdb_query_model_add_entries (data->main_model, data->queue);
+	rhythmdb_query_results_add_results (data->results, data->queue);
 
 	g_free (data);
 }
