@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -32,6 +32,9 @@
 
 G_BEGIN_DECLS
 
+struct RhythmDB_;
+typedef struct RhythmDB_ RhythmDB;
+
 #define RHYTHMDB_TYPE      (rhythmdb_get_type ())
 #define RHYTHMDB(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RHYTHMDB_TYPE, RhythmDB))
 #define RHYTHMDB_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), RHYTHMDB_TYPE, RhythmDBClass))
@@ -47,6 +50,47 @@ GType rhythmdb_entry_get_type (void);
 #define RHYTHMDB_ENTRY(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RHYTHMDB_TYPE_ENTRY, RhythmDBEntry))
 #define RHYTHMDB_IS_ENTRY(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), RHYTHMDB_TYPE_ENTRY))
 
+
+typedef void (*RhythmDBEntryActionFunc) (RhythmDBEntry *entry, gpointer data);
+typedef char* (*RhythmDBEntryStringFunc) (RhythmDBEntry *entry, gpointer data);
+typedef gboolean (*RhythmDBEntryCanSyncFunc) (RhythmDB *db, RhythmDBEntry *entry, gpointer data);
+typedef void (*RhythmDBEntrySyncFunc) (RhythmDB *db, RhythmDBEntry *entry, GError **error, gpointer data);
+
+typedef struct {
+	char 				*name;
+
+	guint				entry_type_data_size;
+
+	/* virtual functions here */
+	RhythmDBEntryActionFunc		post_entry_create;
+	gpointer			post_entry_create_data;
+	GDestroyNotify			post_entry_create_destroy;
+
+	RhythmDBEntryActionFunc		pre_entry_destroy;
+	gpointer			pre_entry_destroy_data;
+	GDestroyNotify			pre_entry_destroy_destroy;
+
+	RhythmDBEntryStringFunc		get_playback_uri;
+	gpointer			get_playback_uri_data;
+	GDestroyNotify			get_playback_uri_destroy;
+
+	RhythmDBEntryCanSyncFunc	can_sync_metadata;
+	gpointer			can_sync_metadata_data;
+	GDestroyNotify			can_sync_metadata_destroy;
+
+	RhythmDBEntrySyncFunc		sync_metadata;
+	gpointer			sync_metadata_data;
+	GDestroyNotify			sync_metadata_destroy;
+} RhythmDBEntryType_;
+typedef RhythmDBEntryType_ *RhythmDBEntryType;
+
+GType rhythmdb_entry_type_get_type (void);
+#define RHYTHMDB_TYPE_ENTRY_TYPE	(rhythmdb_entry_type_get_type ())
+#define RHYTHMDB_ENTRY_TYPE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), RHYTHMDB_TYPE_ENTRY_TYPE, RhythmDBEntryType_))
+#define RHYTHMDB_IS_ENTRY_TYPE(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), RHYTHMDB_TYPE_ENTRY_TYPE))
+
+
+
 typedef GPtrArray RhythmDBQuery;
 GType rhythmdb_query_get_type (void);
 #define RHYTHMDB_TYPE_QUERY	(rhythmdb_query_get_type ())
@@ -54,14 +98,13 @@ GType rhythmdb_query_get_type (void);
 #define RHYTHMDB_IS_QUERY(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), RHYTHMDB_TYPE_QUERY))
 
 
-typedef gint32 RhythmDBEntryType;
-
 #define RHYTHMDB_ENTRY_TYPE_SONG (rhythmdb_entry_song_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_IRADIO_STATION (rhythmdb_entry_iradio_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_PODCAST_POST (rhythmdb_entry_podcast_post_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_PODCAST_FEED (rhythmdb_entry_podcast_feed_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_IMPORT_ERROR (rhythmdb_entry_import_error_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_IGNORE (rhythmdb_entry_ignore_get_type ())
+#define RHYTHMDB_ENTRY_TYPE_INVALID (GINT_TO_POINTER (-1))
 
 typedef enum
 {
@@ -177,9 +220,13 @@ typedef struct {
 const char *rhythmdb_entry_get_string	(RhythmDBEntry *entry, RhythmDBPropType propid);
 char *rhythmdb_entry_dup_string	(RhythmDBEntry *entry, RhythmDBPropType propid);
 gboolean rhythmdb_entry_get_boolean	(RhythmDBEntry *entry, RhythmDBPropType propid);
-guint64 rhythmdb_entry_get_uint64		(RhythmDBEntry *entry, RhythmDBPropType propid);
+guint64 rhythmdb_entry_get_uint64	(RhythmDBEntry *entry, RhythmDBPropType propid);
 gulong rhythmdb_entry_get_ulong		(RhythmDBEntry *entry, RhythmDBPropType propid);
-double rhythmdb_entry_get_double		(RhythmDBEntry *entry, RhythmDBPropType propid);
+double rhythmdb_entry_get_double	(RhythmDBEntry *entry, RhythmDBPropType propid);
+gpointer rhythmdb_entry_get_pointer     (RhythmDBEntry *entry, RhythmDBPropType propid);
+
+RhythmDBEntryType rhythmdb_entry_get_entry_type (RhythmDBEntry *entry);
+
 
 
 typedef enum
@@ -193,12 +240,12 @@ GQuark rhythmdb_error_quark (void);
 
 typedef struct RhythmDBPrivate RhythmDBPrivate;
 
-typedef struct
+struct RhythmDB_
 {
 	GObject parent;
 
 	RhythmDBPrivate *priv;
-} RhythmDB;
+};
 
 typedef struct
 {
@@ -242,6 +289,7 @@ typedef struct
 						 gboolean *cancel);
 } RhythmDBClass;
 
+
 GType		rhythmdb_get_type	(void);
 
 RhythmDB *	rhythmdb_new		(const char *name);
@@ -273,6 +321,11 @@ void		rhythmdb_entry_set_nonotify	(RhythmDB *db, RhythmDBEntry *entry,
 void		rhythmdb_entry_set_uninserted   (RhythmDB *db, RhythmDBEntry *entry,
 						 guint propid, const GValue *value);
 
+char *		rhythmdb_entry_get_playback_uri	(RhythmDBEntry *entry);
+
+gpointer	rhythmdb_entry_get_type_data (RhythmDBEntry *entry, guint expected_size);
+#define		RHYTHMDB_ENTRY_GET_TYPE_DATA(e,t)	((t*)rhythmdb_entry_get_type_data((e),sizeof(t)))
+
 void		rhythmdb_entry_delete	(RhythmDB *db, RhythmDBEntry *entry);
 void            rhythmdb_entry_delete_by_type (RhythmDB *db, 
 					       RhythmDBEntryType type);
@@ -336,6 +389,8 @@ void		rhythmdb_query_serialize		(RhythmDB *db, GPtrArray *query,
 
 GPtrArray *	rhythmdb_query_deserialize		(RhythmDB *db, xmlNodePtr node);
 
+gboolean	rhythmdb_query_is_time_relative		(RhythmDB *db, GPtrArray *query);
+
 const xmlChar *	rhythmdb_nice_elt_name_from_propid	(RhythmDB *db, RhythmDBPropType propid);
 int		rhythmdb_propid_from_nice_elt_name	(RhythmDB *db, const xmlChar *name);
 
@@ -344,10 +399,13 @@ void		rhythmdb_emit_entry_deleted		(RhythmDB *db, RhythmDBEntry *entry);
 
 gboolean	rhythmdb_is_busy			(RhythmDB *db);
 char *		rhythmdb_compute_status_normal		(gint n_songs, glong duration,
-							 guint64 size);
+							 guint64 size,
+							 const char *singular,
+							 const char *plural);
 
 
-RhythmDBEntryType rhythmdb_entry_register_type          (void);
+RhythmDBEntryType rhythmdb_entry_register_type          (const char *name);
+RhythmDBEntryType rhythmdb_entry_type_get_by_name       (const char *name);
 
 RhythmDBEntryType rhythmdb_entry_song_get_type          (void);
 RhythmDBEntryType rhythmdb_entry_iradio_get_type        (void);
