@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -61,17 +61,25 @@
 #include "rb-util.h"
 
 
-#ifdef WITH_BONOBO
-#include "rb-remote-client-proxy.h"
-#include <bonobo/bonobo-main.h>
-#include "bonobo/rb-remote-bonobo.h"
-#endif
-#ifdef WITH_DBUS
+#if WITH_DBUS
 #include <dbus/dbus-glib.h>
 #include "rb-shell-glue.h"
 #include "rb-shell-player-glue.h"
 #include "rb-playlist-manager.h"
 #include "rb-playlist-manager-glue.h"
+#elif WITH_OLD_DBUS
+#include <dbus/dbus.h>
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+#endif
+
+#include <nautilus-burn-drive.h>
+#ifndef NAUTILUS_BURN_CHECK_VERSION 	 
+#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE 	 
+#endif
+
+#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
+#include <nautilus-burn.h>
 #endif
 
 static gboolean debug           = FALSE;
@@ -81,96 +89,37 @@ static gboolean no_registration = FALSE;
 static gboolean no_update	= FALSE;
 static gboolean dry_run		= FALSE;
 static char *rhythmdb_file = NULL;
-#if WITH_DBUS || WITH_BONOBO
+#if WITH_DBUS
 static gboolean load_uri_args (const char **args, GFunc handler, gpointer user_data);
-#endif
-
-#ifdef WITH_BONOBO
-static gboolean print_playing = FALSE;
-static gboolean print_playing_artist = FALSE;
-static gboolean print_playing_album = FALSE;
-static gboolean print_playing_track = FALSE;
-static gboolean print_playing_genre = FALSE;
-static gboolean print_playing_path = FALSE;
-static gboolean playpause       = FALSE;
-static gboolean play            = FALSE;
-static gboolean do_pause        = FALSE;
-static gboolean focus           = FALSE;
-static gboolean previous        = FALSE;
-static gboolean next            = FALSE;
-static gboolean shuffle         = FALSE;
-static gboolean repeat          = FALSE;
-static gboolean print_play_time = FALSE;
-static gboolean print_song_length = FALSE;
-static long seek_time           = 0;
-static long seek_relative       = 0;
-static double set_rating          = -1.0;
-static float set_volume         = -1.0;
-static gboolean toggle_mute     = FALSE;
-static gboolean toggle_hide     = FALSE;
-
-static void handle_cmdline (RBRemoteClientProxy *proxy, gboolean activated,
-			    const char **args);
-#endif
-#ifdef WITH_DBUS
 static void dbus_load_uri (const char *filename, DBusGProxy *proxy);
 #endif
 
 static void main_shell_weak_ref_cb (gpointer data, GObject *objptr);
 
+#if WITH_OLD_DBUS
+static void register_dbus_handler (DBusConnection *connection, RBShell *shell);
+static gboolean send_present_message (DBusConnection *connection, guint32 current_time);
+
+#endif
+
 int
 main (int argc, char **argv)
 {
 	GnomeProgram *program;
-#ifdef WITH_DBUS
+#if WITH_DBUS || WITH_OLD_DBUS
 	DBusGConnection *session_bus;
+	GError *error = NULL;
 #endif
 	RBShell *rb_shell;
 	char **new_argv;
 	gboolean activated;
 	poptContext poptContext;
         GValue context_as_value = { 0 };
-#ifdef WITH_BONOBO
-	RBRemoteClientProxy *client_proxy;
-	RBRemoteBonobo *bonobo;
-#endif
-#if WITH_DBUS || WITH_BONOBO
-	GError *error = NULL;
-#endif
+
 	rb_profile_start ("starting rhythmbox");
 
 	struct poptOption popt_options[] =
 	{
-		/* These are all legacy; for D-BUS just use dbus-send or Python scripts */ 
-#ifdef WITH_BONOBO
-		{ "print-playing",		0,  POPT_ARG_NONE,	&print_playing,			0, N_("Print the playing song and exit"), NULL },
-		{ "print-playing-artist",	0,  POPT_ARG_NONE,	&print_playing_artist,		0, N_("Print the playing song artist and exit"), NULL },
-		{ "print-playing-album",	0,  POPT_ARG_NONE,	&print_playing_album,		0, N_("Print the playing song album and exit"), NULL },
-		{ "print-playing-track",	0,  POPT_ARG_NONE,	&print_playing_track,		0, N_("Print the playing song track and exit"), NULL },
-		{ "print-playing-genre",	0,  POPT_ARG_NONE,	&print_playing_genre,		0, N_("Print the playing song genre and exit"), NULL },
-		{ "print-playing-path",		0,  POPT_ARG_NONE,	&print_playing_path,		0, N_("Print the playing song URI and exit"), NULL },
-        
-		{ "print-song-length",		0,  POPT_ARG_NONE,	&print_song_length,		0, N_("Print the playing song length in seconds and exit"), NULL },
-		{ "print-play-time",		0,  POPT_ARG_NONE,	&print_play_time,		0, N_("Print the current elapsed time of playing song and exit"), NULL },
-		{ "set-play-time",		0,  POPT_ARG_LONG,	&seek_time,			0, N_("Seek to the specified time in playing song if possible and exit"), NULL },
-		{ "seek",			0,  POPT_ARG_LONG,	&seek_relative,			0, N_("Seek by the specified amount if possible and exit"), NULL },
-		{ "set-rating",			0,  POPT_ARG_DOUBLE,	&set_rating,			0, N_("Set the rating of the currently playing song and exit"), NULL },
-        
-		{ "play-pause",			0,  POPT_ARG_NONE,	&playpause,			0, N_("Toggle play/pause mode"), NULL },
-		{ "pause",			0,  POPT_ARG_NONE,	&do_pause,			0, N_("Pause playback if currently playing"), NULL },
-		{ "play",			0,  POPT_ARG_NONE,	&play,				0, N_("Resume playback if currently paused"), NULL },
-		{ "focus",			0,  POPT_ARG_NONE,	&focus,				0, N_("Focus the running player"), NULL },
-		{ "previous",			0,  POPT_ARG_NONE,	&previous,			0, N_("Jump to previous song"), NULL },
-		{ "next",			0,  POPT_ARG_NONE,	&next,				0, N_("Jump to next song"), NULL },
-		
-		{ "shuffle",			0,  POPT_ARG_NONE,	&shuffle,			0, N_("Toggle shuffling"), NULL },
-		{ "repeat",			0,  POPT_ARG_NONE,	&repeat,			0, N_("Toggle repeat"), NULL },
-
-		{ "set-volume",			0,  POPT_ARG_FLOAT,	&set_volume,			0, N_("Set the volume level"), NULL },
-		{ "toggle-mute",		0,  POPT_ARG_NONE,	&toggle_mute,			0, N_("Mute or unmute playback"), NULL },
-		{ "toggle-hide",		0,  POPT_ARG_NONE,	&toggle_hide,			0, N_("Change visibility of the main Rhythmbox window"), NULL },
-#endif
-
 		{ "debug",			'd',POPT_ARG_NONE,	&debug,				0, N_("Enable debug output"), NULL },
 		{ "debug-match",		'D',POPT_ARG_STRING,	&debug_match,			0, N_("Enable debug output matching a specified string"), NULL },
 		{ "no-update",			0,  POPT_ARG_NONE,	&no_update,			0, N_("Do not update the library with file changes"), NULL },
@@ -202,11 +151,6 @@ main (int argc, char **argv)
         gst_init (NULL, NULL);
 	rb_profile_end ("initializing gstreamer");
 #endif
-#ifdef WITH_BONOBO
-	rb_profile_start ("initializing bonobo remote");
-	rb_remote_bonobo_preinit ();
-	rb_profile_end ("initializing bonobo remote");
-#endif
 
 	gtk_set_locale ();
 	rb_profile_start ("initializing gnome program");
@@ -251,20 +195,7 @@ main (int argc, char **argv)
 
 	activated = FALSE;
 
-#ifdef WITH_BONOBO
-	rb_profile_start ("creating bonobo remote");
-	rb_debug ("going to create Bonobo object");
-	bonobo = rb_remote_bonobo_new ();
-	client_proxy = NULL;
-	if (!no_registration) {
-		if ((activated = rb_remote_bonobo_activate (bonobo))) {
-			rb_debug ("successfully activated Bonobo");
-			client_proxy = RB_REMOTE_CLIENT_PROXY (bonobo);
-		}
-	}
-	rb_profile_end ("creating bonobo remote");
-#endif
-#ifdef WITH_DBUS
+#if WITH_DBUS || WITH_OLD_DBUS
 	rb_debug ("going to create DBus object");
 
 	dbus_g_thread_init ();
@@ -276,6 +207,13 @@ main (int argc, char **argv)
 	} else if (!no_registration) {
 		guint request_name_reply;
 		int flags;
+#ifndef DBUS_NAME_FLAG_DO_NOT_QUEUE
+		flags = DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT;
+#else
+		flags = DBUS_NAME_FLAG_DO_NOT_QUEUE;
+#endif                
+
+#ifndef WITH_OLD_DBUS
 		DBusGProxy *bus_proxy;
 
 		bus_proxy = dbus_g_proxy_new_for_name (session_bus,
@@ -283,12 +221,6 @@ main (int argc, char **argv)
 						       "/org/freedesktop/DBus",
 						       "org.freedesktop.DBus");
 
-#ifndef DBUS_NAME_FLAG_DO_NOT_QUEUE
-		flags = DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT;
-#else
-		flags = DBUS_NAME_FLAG_DO_NOT_QUEUE;
-#endif                
-
 		if (!dbus_g_proxy_call (bus_proxy,
 					"RequestName",
 					&error,
@@ -305,21 +237,44 @@ main (int argc, char **argv)
 		}
 		g_object_unref (bus_proxy);
 
+#else
+		DBusError dbus_error = {0,};
+		DBusConnection *connection;
+
+		connection = dbus_g_connection_get_connection (session_bus);
+		request_name_reply = dbus_bus_request_name (connection,
+			       				    "org.gnome.Rhythmbox",
+							    flags,
+							    &dbus_error);
+		if (dbus_error_is_set (&dbus_error)) {
+			g_warning ("Failed to invoke RequestName: %s",
+				   dbus_error.message);
+		}
+		dbus_error_free (&dbus_error);
+#endif
 		if (request_name_reply == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER
 		    || request_name_reply == DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER)
 			activated = FALSE;
-		else if (request_name_reply == DBUS_REQUEST_NAME_REPLY_EXISTS)
+		else if (request_name_reply == DBUS_REQUEST_NAME_REPLY_EXISTS
+			 || request_name_reply == DBUS_REQUEST_NAME_REPLY_IN_QUEUE)
 			activated = TRUE;
 		else {
 			g_warning ("Got unhandled reply %u from RequestName",
 				   request_name_reply);
 			activated = FALSE;
 		}
+		
+						
 	}
 		
 #endif
 
 	if (!activated) {
+		if (quit) {
+			rb_debug ("was asked to quit, but no instance was running");
+			gdk_notify_startup_complete ();
+			exit (0);
+		}
 #ifdef WITH_RHYTHMDB_GDA
 		gda_init (PACKAGE, VERSION, argc, argv);
 #endif
@@ -333,25 +288,17 @@ main (int argc, char **argv)
 		glade_gnome_init ();
 	
 		rb_stock_icons_init ();
+
+#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
+		nautilus_burn_init ();
+#endif
+
+		gtk_window_set_default_icon_name ("rhythmbox");
 	
 		rb_shell = rb_shell_new (argc, argv, no_registration, no_update, dry_run, rhythmdb_file);
 		g_object_weak_ref (G_OBJECT (rb_shell), main_shell_weak_ref_cb, NULL);
-#ifdef WITH_BONOBO
-		if (!no_registration) {
-			rb_remote_bonobo_acquire (bonobo, RB_REMOTE_PROXY (rb_shell), &error);
-			if (error) {
-				g_warning ("error: %s", error->message);
-			} else {
-				if (rb_remote_bonobo_activate (bonobo))
-					client_proxy = RB_REMOTE_CLIENT_PROXY (bonobo);
-				else
-					g_warning ("acquired bonobo service but couldn't activate!");
-			}
-		}
-
-#endif
-#ifdef WITH_DBUS
 		if (!no_registration && session_bus != NULL) {
+#if WITH_DBUS
 			GObject *obj;
 			const char *path;
 
@@ -369,27 +316,16 @@ main (int argc, char **argv)
 			obj = rb_shell_get_playlist_manager (rb_shell);
 			path = rb_shell_get_playlist_manager_path (rb_shell);
 			dbus_g_connection_register_g_object (session_bus, path, obj);
+#elif WITH_OLD_DBUS
+			register_dbus_handler (dbus_g_connection_get_connection (session_bus), 
+					       rb_shell);
+#endif /* WITH_DBUS */
 		}
-#endif
-#ifdef WITH_DBUS
 	} else if (!no_registration && session_bus != NULL) {
+#if WITH_DBUS
 		DBusGProxy *shell_proxy;
+#endif
 		guint32 current_time;
-
-		shell_proxy = dbus_g_proxy_new_for_name_owner (session_bus,
-							       "org.gnome.Rhythmbox",
-							       "/org/gnome/Rhythmbox/Shell",
-							       "org.gnome.Rhythmbox.Shell",
-							       &error);
-		if (!shell_proxy) {
-			g_warning ("Couldn't create proxy for Rhythmbox shell: %s",
-				   error->message);
-		} else {
-			load_uri_args (poptGetArgs (poptContext), (GFunc) dbus_load_uri, shell_proxy);
-		}
-#ifdef WITH_BONOBO
-		if (!client_proxy) {
-#endif /* WITH_BONOBO */
 #if GTK_MINOR_VERSION >= 8
 		current_time = gdk_x11_display_get_user_time (gdk_display_get_default ());
 #else
@@ -402,30 +338,41 @@ main (int argc, char **argv)
 		 */
 		current_time = GDK_CURRENT_TIME;
 #endif	/* GTK_MINOR_VERSION */
-		dbus_g_proxy_call_no_reply (shell_proxy, "present",
-					    G_TYPE_UINT, current_time,
-					    G_TYPE_INVALID);
-#ifdef WITH_BONOBO
+
+#if WITH_DBUS
+		shell_proxy = dbus_g_proxy_new_for_name_owner (session_bus,
+							       "org.gnome.Rhythmbox",
+							       "/org/gnome/Rhythmbox/Shell",
+							       "org.gnome.Rhythmbox.Shell",
+							       &error);
+		if (!shell_proxy) {
+			g_warning ("Couldn't create proxy for Rhythmbox shell: %s",
+				   error->message);
+		} else {
+			if (quit) {
+				dbus_g_proxy_call_no_reply (shell_proxy, "quit",
+							    G_TYPE_INVALID);
+			} else {
+				load_uri_args (poptGetArgs (poptContext), (GFunc) dbus_load_uri, shell_proxy);
+				dbus_g_proxy_call_no_reply (shell_proxy, "present",
+							    G_TYPE_UINT, current_time,
+							    G_TYPE_INVALID);
+			}
+			g_object_unref (G_OBJECT (shell_proxy));
 		}
-#endif /* WITH_BONOBO */
+#elif WITH_OLD_DBUS
+		if (!send_present_message (dbus_g_connection_get_connection (session_bus), 
+					   current_time))
+			g_warning ("Unable to send dbus message to existing rhythmbox instance");
 #endif /* WITH_DBUS */
 	}
-#ifdef WITH_BONOBO	
-	if (client_proxy)
-		handle_cmdline (client_proxy, activated, poptGetArgs (poptContext));
-#endif
 
 	if (activated) {
 		gdk_notify_startup_complete ();
 	} else {
 
 		rb_profile_start ("mainloop");
-#ifdef WITH_BONOBO
-                /* Unfortunately Bonobo takes over the main loop ... */
-		bonobo_main ();
-#else
 		gtk_main ();
-#endif
 		rb_profile_end ("mainloop");
 
 		rb_debug ("out of toplevel loop");
@@ -435,6 +382,10 @@ main (int argc, char **argv)
 		rb_refstring_system_shutdown ();
 
 		gnome_vfs_shutdown ();
+
+#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
+		nautilus_burn_shutdown ();
+#endif
 	}
 
 	g_strfreev (new_argv);
@@ -444,7 +395,7 @@ main (int argc, char **argv)
 	exit (0);
 }
 
-#if WITH_DBUS || WITH_BONOBO
+#if WITH_DBUS
 static gboolean
 load_uri_args (const char **args, GFunc handler, gpointer user_data)
 {
@@ -468,9 +419,7 @@ load_uri_args (const char **args, GFunc handler, gpointer user_data)
 	}
 	return handled;
 }
-#endif
 
-#ifdef WITH_DBUS
 static void
 dbus_load_uri (const char *filename, DBusGProxy *proxy)
 {
@@ -486,137 +435,90 @@ dbus_load_uri (const char *filename, DBusGProxy *proxy)
 }
 #endif
 
-#ifdef WITH_BONOBO
-
-static void
-bonobo_load_uri (const char *filename, RBRemoteClientProxy *proxy)
-{
-	rb_remote_client_proxy_handle_uri (proxy, filename);
-}
-
-static void
-handle_cmdline (RBRemoteClientProxy *proxy, gboolean activated, const char **args)
-{
-	gboolean grab_focus;
-	RBRemoteSong *song;
-
-	grab_focus = TRUE;
-
-	rb_debug ("handling command line");
-
-	song = NULL;
-	if (print_playing
-	    || print_playing_artist
-	    || print_playing_album
-	    || print_playing_track
-	    || print_playing_genre
-	    || print_playing_path
-	    || print_song_length ) {
-		rb_debug ("retrieving playing song");
-		song = rb_remote_client_proxy_get_playing_song (proxy);
-		grab_focus = FALSE;
-	}
-
-	if (print_playing)
-		g_print ("%s\n", song ? song->title : "");
-
-	if (print_playing_artist)
-		g_print ("%s\n", song ? song->artist : "");
-
-	if (print_playing_album)
-		g_print ("%s\n", song ? song->album : "");
-
-	if (print_playing_track)
-		g_print ("%ld\n", song ? song->track_number : -1);
-
-	if (print_playing_genre)
-		g_print ("%s\n", song ? song->genre : "");
-	
-	if (print_playing_path)
-		g_print ("%s\n", song ? song->uri : "");
-	
-	if (print_song_length)
-		g_print ("%ld\n", song ? song->duration : -1);
-	
-	if (print_play_time
-	    || seek_time > 0
-	    || playpause
-	    || play
-	    || do_pause
-	    || previous
-	    || next
-	    || shuffle)
-		grab_focus = FALSE;
-
-	if (print_play_time)
-		printf ("%ld\n", rb_remote_client_proxy_get_playing_time (proxy));
-
-	if (set_rating != -1)
-		rb_remote_client_proxy_set_rating (proxy, set_rating);
-	
-	if (seek_time > 0)
-		rb_remote_client_proxy_set_playing_time (proxy, seek_time);
-
-	if (seek_relative != 0)
-		rb_remote_client_proxy_seek (proxy, seek_relative);
-	
-	if (play)
-		rb_remote_client_proxy_play (proxy);
-	else if (do_pause)
-		rb_remote_client_proxy_pause (proxy);
-	else if (playpause)
-		rb_remote_client_proxy_toggle_playing (proxy);
-
-	if (previous)
-                rb_remote_client_proxy_jump_previous (proxy);
-
-	if (next)
-                rb_remote_client_proxy_jump_next (proxy);
-
-	if (shuffle)
-		rb_remote_client_proxy_toggle_shuffle (proxy);
-
-	if (repeat)
-		rb_remote_client_proxy_toggle_repeat (proxy);
-
-	if (set_volume > -0.0001) {
-		if (set_volume > 1.0)
-			set_volume = 1.0;
-		else if (set_volume < 0.0)
-			set_volume = 0.0;
-		rb_remote_client_proxy_set_volume (proxy, set_volume);
-	}
-
-	if (toggle_mute)
-		rb_remote_client_proxy_toggle_mute (proxy);
-
-	if (load_uri_args (args, (GFunc) bonobo_load_uri, proxy))
-		grab_focus = FALSE;
-
-	if (quit)
-		rb_remote_client_proxy_quit (proxy);
-
-	if (focus)
-		grab_focus = TRUE;
-
-	if (toggle_hide)
-		rb_remote_client_proxy_toggle_visibility (proxy);
-
-	/* at the very least, we focus the window */
-	if (activated && grab_focus) {
-		rb_debug ("grabbing focus");
-		rb_remote_client_proxy_grab_focus (proxy);
-	}
-}
-#endif
-
 static void
 main_shell_weak_ref_cb (gpointer data, GObject *objptr)
 {
 	rb_debug ("caught shell finalization");
-#ifdef WITH_BONOBO	
-	bonobo_main_quit ();
-#else
 	gtk_main_quit ();
-#endif
 }
+
+
+#ifdef WITH_OLD_DBUS
+/* old dbus support (0.31-0.34) */
+
+static gboolean
+send_present_message (DBusConnection *connection, guint32 current_time)
+{
+	DBusMessage *message;
+	gboolean result;
+
+	message = dbus_message_new_method_call ("org.gnome.Rhythmbox",
+						"/org/gnome/Rhythmbox/Shell",
+						"org.gnome.Rhythmbox.Shell",
+						"present");
+	if (!message)
+		return FALSE;
+
+	if (!dbus_message_append_args (message,
+				       DBUS_TYPE_UINT32, &current_time,
+				       DBUS_TYPE_INVALID)) {
+		dbus_message_unref (message);
+		return FALSE;
+	}
+
+	result = dbus_connection_send (connection, message, NULL);
+	dbus_message_unref (message);
+
+	return result;
+}
+
+static void
+unregister_dbus_handler (DBusConnection *connection, void *data)
+{
+	/* nothing */
+}
+
+static DBusHandlerResult
+handle_dbus_message (DBusConnection *connection, DBusMessage *message, void *data)
+{
+	RBShell *shell = (RBShell *)data;
+
+	rb_debug ("Handling dbus message");
+	if (dbus_message_is_method_call (message, "org.gnome.Rhythmbox.Shell", "present")) {
+		DBusMessageIter iter;
+		guint32 current_time;
+		
+		if (!dbus_message_iter_init (message, &iter))
+			return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+		if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT32)
+			return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+		dbus_message_iter_get_basic (&iter, &current_time);
+
+		rb_shell_present (shell, current_time, NULL);
+		return DBUS_HANDLER_RESULT_HANDLED;
+	} else {
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
+}
+
+static void
+register_dbus_handler (DBusConnection *connection, RBShell *shell)
+{
+	DBusObjectPathVTable vt = {
+		unregister_dbus_handler,
+		handle_dbus_message,
+		NULL, NULL, NULL, NULL
+	};
+
+	dbus_connection_register_object_path (connection,
+					      "/org/gnome/Rhythmbox/Shell",
+					      &vt,
+					      shell);
+	dbus_connection_ref (connection);
+	dbus_connection_setup_with_g_main (connection, NULL);
+}
+
+#endif
+
