@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -189,6 +189,7 @@ struct RBShellPlayerPrivate
 	char *song;
 	gboolean have_url;
 	char *url;
+	long elapsed;
 
 	RBPlayOrder *play_order;
 	RBPlayOrder *queue_play_order;
@@ -596,7 +597,7 @@ rb_shell_player_init (RBShellPlayer *player)
 					    (GConfClientNotifyFunc)gconf_play_order_changed,
 					    player);
 
-	player->priv->header_widget = rb_header_new (player->priv->mmplayer);
+	player->priv->header_widget = rb_header_new (player);
 	gtk_widget_show (GTK_WIDGET (player->priv->header_widget));
 	gtk_box_pack_start (GTK_BOX (player), GTK_WIDGET (player->priv->header_widget), TRUE, TRUE, 0);
 
@@ -1029,16 +1030,17 @@ rb_shell_player_open_location (RBShellPlayer *player,
 static gboolean
 rb_shell_player_open_entry (RBShellPlayer *player, RhythmDBEntry *entry, GError **error)
 {
-	gulong type;
-	const char *location;
+	char *location;
+	gboolean result;
 
-	type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
-	if (type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
-		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
-	else
-		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	location = rhythmdb_entry_get_playback_uri (entry);
+	if (location == NULL)
+		return FALSE;
 
-	return rb_shell_player_open_location (player, location, error);
+	result = rb_shell_player_open_location (player, location, error);
+	g_free (location);
+
+	return result;
 }
 
 
@@ -1553,10 +1555,14 @@ rb_shell_player_cmd_next (GtkAction *action,
 
 void
 rb_shell_player_play_entry (RBShellPlayer *player,
-			    RhythmDBEntry *entry)
+			    RhythmDBEntry *entry,
+			    RBSource *source)
 {
 	GError *error = NULL;
-	rb_shell_player_set_playing_source (player, player->priv->selected_source);
+
+	if (source == NULL)
+		source = player->priv->selected_source;
+	rb_shell_player_set_playing_source (player, source);
 
 	if (!rb_shell_player_set_playing_entry (player, entry, TRUE, &error)) {
 		rb_shell_player_error (player, FALSE, error);
@@ -1585,12 +1591,15 @@ rb_shell_player_cmd_play (GtkAction *action,
 	GError *error = NULL;
 	rb_debug ("play!");
 	if (!rb_shell_player_playpause (player, FALSE, &error))
-		rb_error_dialog (NULL, _("Couldn't start playback: %s"), (error) ? error->message : "(null)");
+		rb_error_dialog (NULL, 
+				 _("Couldn't start playback"), 
+				 "%s", (error) ? error->message : "(null)");
 	g_clear_error (&error);
 }
 
+/* unused parameter can't be removed without breaking dbus interface compatibility */
 gboolean
-rb_shell_player_playpause (RBShellPlayer *player, gboolean ignore_stop, GError **error)
+rb_shell_player_playpause (RBShellPlayer *player, gboolean unused, GError **error)
 {
 	gboolean ret;
 	RBEntryView *songs;
@@ -1610,7 +1619,7 @@ rb_shell_player_playpause (RBShellPlayer *player, gboolean ignore_stop, GError *
 			songs = rb_source_get_entry_view (player->priv->current_playing_source);
 			if (songs)
 				rb_entry_view_set_state (songs, RB_ENTRY_VIEW_PAUSED);
-		} else if (!ignore_stop) {
+		} else {
 			rb_debug ("setting playing source to NULL");
 			rb_shell_player_set_playing_source (player, NULL);
 		}
@@ -1650,6 +1659,8 @@ rb_shell_player_playpause (RBShellPlayer *player, gboolean ignore_stop, GError *
 					entry = (RhythmDBEntry*) selection->data;
 					if (entry)
 						out_of_order = TRUE;
+				
+					g_list_free (selection);
 				}
 			}
 
@@ -1880,6 +1891,7 @@ rb_shell_player_entry_activated_cb (RBEntryView *view,
 	RhythmDBEntry *prev_entry = NULL;
 	GError *error = NULL;
 	gboolean source_set = FALSE;
+	char *playback_uri;
 
 	g_return_if_fail (entry != NULL);
 
@@ -1889,16 +1901,13 @@ rb_shell_player_entry_activated_cb (RBEntryView *view,
 	if (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN))
 		return;
 
-	/* don't play import error entries */
-	if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_IMPORT_ERROR)
+	/* skip entries with no playback uri */
+	playback_uri = rhythmdb_entry_get_playback_uri (entry);
+	if (playback_uri == NULL) {
 		return;
-
-	/* ensure the podcast has been downloaded */
-	if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
-		if (!rb_podcast_manager_entry_downloaded (entry))
-			return;
 	}
-	
+	g_free (playback_uri);
+
 	/* figure out where the previous entry came from */
 	if ((playa->priv->queue_source != NULL) &&
 	    (playa->priv->current_playing_source == RB_SOURCE (playa->priv->queue_source))) {
@@ -2034,7 +2043,6 @@ rb_shell_player_sync_with_source (RBShellPlayer *player)
 	const char *artist = NULL;	
 	char *title;
 	RhythmDBEntry *entry;
-	glong elapsed;
 
 	entry = rb_shell_player_get_playing_entry (player);
 	rb_debug ("playing source: %p, active entry: %p", player->priv->current_playing_source, entry);
@@ -2054,14 +2062,14 @@ rb_shell_player_sync_with_source (RBShellPlayer *player)
 	else
 		title = NULL;
 
-	elapsed = rb_player_get_time (player->priv->mmplayer);
+	player->priv->elapsed = rb_player_get_time (player->priv->mmplayer);
 
 	g_signal_emit (G_OBJECT (player), rb_shell_player_signals[WINDOW_TITLE_CHANGED], 0,
 		       title);
 	g_signal_emit (G_OBJECT (player), rb_shell_player_signals[ELAPSED_CHANGED], 0,
-		       (guint) elapsed);
+		       (guint) player->priv->elapsed);
 
-	/* Sync the player */
+	/* Sync the header */
 	if (player->priv->song)
 		rb_header_set_title (player->priv->header_widget, title);
 	else
@@ -2238,7 +2246,6 @@ rb_shell_player_stop (RBShellPlayer *player)
 
 	g_return_if_fail (RB_IS_SHELL_PLAYER (player));
 
-	rb_shell_player_pause (player, &error);
 	if (error == NULL)
 		rb_player_close (player->priv->mmplayer, &error);
 	if (error) {
@@ -2248,6 +2255,8 @@ rb_shell_player_stop (RBShellPlayer *player)
 		g_error_free (error);
 	}
 
+	rb_shell_player_sync_with_source (player);
+	g_object_notify (G_OBJECT (player), "playing");
 	rb_shell_player_sync_buttons (player);
 }
 
@@ -2295,7 +2304,9 @@ rb_shell_player_get_playing (RBShellPlayer *player,
 char *
 rb_shell_player_get_playing_time_string (RBShellPlayer *player)
 {
-	return rb_header_get_elapsed_string (player->priv->header_widget);
+	return rb_make_elapsed_time_string (player->priv->elapsed,
+					    rb_shell_player_get_playing_song_duration (player),
+					    !eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY));
 }
 
 gboolean
@@ -2413,7 +2424,7 @@ eos_cb (RBPlayer *mmplayer, gpointer data)
 				rb_debug ("Last retry was less than 4 seconds ago...aborting retry playback");
 				rb_shell_player_set_playing_source (player, NULL);
 			} else {
-				rb_shell_player_play_entry (player, entry);
+				rb_shell_player_play_entry (player, entry, NULL);
 			}
 		}
 			break;
@@ -2459,10 +2470,18 @@ rb_shell_player_error (RBShellPlayer *player, gboolean async, const GError *err)
 	if (entry && async)
 		rb_shell_player_set_entry_playback_error (player, entry, err->message);
 
-	if (err->code == RB_PLAYER_ERROR_NO_AUDIO)
+	if (err->code == RB_PLAYER_ERROR_NO_AUDIO) {
+		/* stream has completely ended */
 		rb_shell_player_set_playing_source (player, NULL);
-	else if (player->priv->do_next_idle_id == 0)
+	} else if (rb_source_handle_eos (player->priv->current_playing_source) == RB_SOURCE_EOF_RETRY) {
+		/* receiving an error means a broken stream or non-audio stream, so abort */
+		rb_error_dialog (NULL, 
+				 _("Couldn't start playback"), 
+				 "%s", (err) ? err->message : "(null)");
+		rb_shell_player_set_playing_source (player, NULL);
+	} else if (player->priv->do_next_idle_id == 0) {
 		player->priv->do_next_idle_id = g_idle_add ((GSourceFunc)do_next_idle, player);
+	}
 
 	player->priv->handling_error = FALSE;
 }
@@ -2495,18 +2514,14 @@ tick_cb (RBPlayer *mmplayer, long elapsed, gpointer data)
 
 	GDK_THREADS_ENTER ();
 
-	rb_header_sync_time (player->priv->header_widget);
-
 	if (rb_player_playing (mmplayer)) {
-		static long last_elapsed = -1;
-		if (last_elapsed != elapsed) {
+		if (player->priv->elapsed != elapsed) {
+			player->priv->elapsed = elapsed;
 			g_signal_emit (G_OBJECT (player), rb_shell_player_signals[ELAPSED_CHANGED],
 				       0, (guint) elapsed);
-			last_elapsed = elapsed;
 		}
 	}
 
-
 	GDK_THREADS_LEAVE ();
 }
 
@@ -2521,7 +2536,7 @@ info_available_cb (RBPlayer *mmplayer,
         RhythmDBPropType entry_field = 0;
         gboolean changed = FALSE;
         gboolean set_field = FALSE;
-	gulong type;
+	RhythmDBEntryType type;
 	
         rb_debug ("info: %d", field);
 
@@ -2541,7 +2556,7 @@ info_available_cb (RBPlayer *mmplayer,
                 goto out_unlock;
         }
 
-	type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+	type = rhythmdb_entry_get_entry_type (entry);
 	if (type != RHYTHMDB_ENTRY_TYPE_IRADIO_STATION) {
 		rb_debug ("Got info_available but entry isn't an iradio station");
 		goto out_unlock;
@@ -2628,7 +2643,7 @@ info_available_cb (RBPlayer *mmplayer,
 			bitrate = g_value_get_ulong (value);
 			g_value_set_ulong (value, bitrate/1000);
 			
-			rb_debug ("setting bitrate of iradio station to %d", 
+			rb_debug ("setting bitrate of iradio station to %lu", 
 				  g_value_get_ulong (value));
 			entry_field = RHYTHMDB_PROP_BITRATE;
 			set_field = TRUE;
@@ -2739,7 +2754,7 @@ filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 	player = (RBShellPlayer *)data;
 
 	if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPlay) == key->keycode) {	
-		rb_shell_player_playpause (player, TRUE, NULL);
+		rb_shell_player_playpause (player, FALSE, NULL);
 		return GDK_FILTER_REMOVE;
 	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPause) == key->keycode) {	
 		rb_shell_player_pause (player, NULL);
