@@ -1,7 +1,8 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of audiocd source object (based of the ipod source)
  *
- *  Copyright (C) 2005 James Livingston  <jrl@ids.org.au>
+ *  Copyright (C) 2005-2006 James Livingston  <jrl@ids.org.au>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -23,27 +24,27 @@
 /*
  * TODO
  *    * handle cases where MusicBrainz returns multiple albums
+ *    * save user-edited metadata somewhere (use S-J stuff?)
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkicontheme.h>
 #include <string.h>
-#include "rhythmdb.h"
-#include <libgnome/gnome-i18n.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-volume.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <gst/gst.h>
 #include <totem-disc.h>
+
+#include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
 #include "rb-audiocd-source.h"
 #include "rb-util.h"
 #include "rb-stock-icons.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
-#include "rb-player.h"
-#include "rhythmdb.h"
+
 #ifdef HAVE_MUSICBRAINZ
 #include "sj-metadata-musicbrainz.h"
 #include "sj-structures.h"
@@ -67,7 +68,9 @@ typedef struct
 	gchar *device_path;
 	GList *tracks;
 
-	GstElement *pipeline, *cdda, *fakesink;
+	GstElement *pipeline;
+	GstElement *cdda;
+	GstElement *fakesink;
 	
 #ifdef HAVE_MUSICBRAINZ
 	SjMetadata *metadata;
@@ -132,7 +135,8 @@ rb_audiocd_source_dispose (GObject *object)
 
 
 static GObject *
-rb_audiocd_source_constructor (GType type, guint n_construct_properties,
+rb_audiocd_source_constructor (GType type,
+			       guint n_construct_properties,
 			       GObjectConstructParam *construct_properties)
 {
 	RBAudioCdSource *source; 
@@ -154,7 +158,8 @@ rb_audiocd_source_constructor (GType type, guint n_construct_properties,
 }
 
 RBRemovableMediaSource *
-rb_audiocd_source_new (RBShell *shell, GnomeVFSVolume *volume)
+rb_audiocd_source_new (RBShell *shell,
+		       GnomeVFSVolume *volume)
 {
 	char *device_path;
 	GObject *source;
@@ -162,7 +167,10 @@ rb_audiocd_source_new (RBShell *shell, GnomeVFSVolume *volume)
 
 	g_assert (rb_audiocd_is_volume_audiocd (volume));
 
-	entry_type =  rhythmdb_entry_register_type ();
+	entry_type =  rhythmdb_entry_register_type (NULL);
+	entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc)rb_true_function;
+	/* TODO same the metadata somewhere */
+	entry_type->sync_metadata = (RhythmDBEntrySyncFunc)rb_null_function;
 
 	device_path = gnome_vfs_volume_get_device_path (volume);
 
@@ -171,6 +179,7 @@ rb_audiocd_source_new (RBShell *shell, GnomeVFSVolume *volume)
 			       "volume", volume,
 			       "shell", shell,
 			       "sorting-key", NULL,
+			       "sourcelist-group", RB_SOURCELIST_GROUP_REMOVABLE,
 			       NULL);
 
 	g_free (device_path);
@@ -181,8 +190,11 @@ rb_audiocd_source_new (RBShell *shell, GnomeVFSVolume *volume)
 }
 
 static void 
-entry_set_string_prop (RhythmDB *db, RhythmDBEntry *entry, gboolean is_inserted,
-		       RhythmDBPropType propid, const char *str)
+entry_set_string_prop (RhythmDB *db,
+		       RhythmDBEntry *entry,
+		       gboolean is_inserted,
+		       RhythmDBPropType propid,
+		       const char *str)
 {
 	GValue value = {0,};
 
@@ -203,8 +215,10 @@ entry_set_string_prop (RhythmDB *db, RhythmDBEntry *entry, gboolean is_inserted,
 	g_value_unset (&value);
 }
 
-static RhythmDBEntry*
-rb_audiocd_create_track_entry (RBAudioCdSource *source, RhythmDB *db, guint track_number)
+static RhythmDBEntry *
+rb_audiocd_create_track_entry (RBAudioCdSource *source,
+			       RhythmDB *db,
+			       guint track_number)
 {
 	RhythmDBEntry *entry;
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
@@ -286,7 +300,8 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source, RhythmDB *db, guint trac
 }
 
 static gboolean
-rb_audiocd_get_cd_info (RBAudioCdSource *source, gint64 *num_tracks)
+rb_audiocd_get_cd_info (RBAudioCdSource *source,
+			gint64 *num_tracks)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	GstFormat fmt = gst_format_get_by_nick ("track");
@@ -306,7 +321,8 @@ rb_audiocd_get_cd_info (RBAudioCdSource *source, gint64 *num_tracks)
 }
 
 static gboolean
-rb_audiocd_scan_songs (RBAudioCdSource *source, RhythmDB *db)
+rb_audiocd_scan_songs (RBAudioCdSource *source,
+		       RhythmDB *db)
 {
 	gint64 i, num_tracks;
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
@@ -333,15 +349,14 @@ rb_audiocd_scan_songs (RBAudioCdSource *source, RhythmDB *db)
 	}
 #endif
 
-	if (ok && !rb_audiocd_get_cd_info (source, &num_tracks))
-	{
+	if (ok && !rb_audiocd_get_cd_info (source, &num_tracks)) {
 		rb_error_dialog (NULL, _("Couldn't load Audio CD"),
 					_("Rhythmbox couldn't read the CD information."));
 		ok = FALSE;
 	}
 
 	if (ok) {
-		rb_debug ("importing Audio Cd %s - %d tracks", priv->device_path, num_tracks);
+		rb_debug ("importing Audio Cd %s - %d tracks", priv->device_path, (int)num_tracks);
 		for (i = 1; i <= num_tracks; i++) {
 			RhythmDBEntry* entry = rb_audiocd_create_track_entry (source, db, i);
 
@@ -366,7 +381,10 @@ rb_audiocd_scan_songs (RBAudioCdSource *source, RhythmDB *db)
 
 #ifdef HAVE_MUSICBRAINZ
 static void
-metadata_cb (SjMetadata *metadata, GList *albums, GError *error, RBAudioCdSource *source)
+metadata_cb (SjMetadata *metadata,
+	     GList *albums,
+	     GError *error,
+	     RBAudioCdSource *source)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	GList *cd_track = priv->tracks;
@@ -396,8 +414,9 @@ metadata_cb (SjMetadata *metadata, GList *albums, GError *error, RBAudioCdSource
 	g_value_set_boolean (&true_value, TRUE);
 
 	/*while (albums) {*/
+	{
 		AlbumDetails *album;
-		album = (AlbumDetails*)albums->data;
+		album = (AlbumDetails *)albums->data;
 
 		g_object_set (G_OBJECT (source), "name", album->title, NULL);
 
@@ -406,6 +425,8 @@ metadata_cb (SjMetadata *metadata, GList *albums, GError *error, RBAudioCdSource
 			RhythmDBEntry *entry = cd_track->data;
 			GValue value = {0, };
 
+			rb_debug ("storing metadata for %s - %s - %s", track->artist, album->title, track->title);
+
 			/* record track info in entry*/
 			entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_TITLE, track->title);
 			entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_ARTIST, track->artist);
@@ -445,8 +466,8 @@ metadata_cb (SjMetadata *metadata, GList *albums, GError *error, RBAudioCdSource
 			cd_track = g_list_next (cd_track);
 		}
 
-		/*albums = g_list_next (albums);
-	}*/
+		/*albums = g_list_next (albums);*/
+	}
 
 	g_object_unref (metadata);
 	priv->metadata = NULL;
@@ -454,7 +475,10 @@ metadata_cb (SjMetadata *metadata, GList *albums, GError *error, RBAudioCdSource
 }
 
 static void
-metadata_cancelled_cb (SjMetadata *metadata, GList *albums, GError *error, gpointer old_source)
+metadata_cancelled_cb (SjMetadata *metadata,
+		       GList *albums,
+		       GError *error,
+		       gpointer old_source)
 {
 	/* NOTE: the source may have been finalised, and so should NOT be used*/
 	g_object_unref (metadata);
@@ -462,12 +486,13 @@ metadata_cancelled_cb (SjMetadata *metadata, GList *albums, GError *error, gpoin
 #endif
 
 static void
-rb_audiocd_load_metadata (RBAudioCdSource *source, RhythmDB *db)
+rb_audiocd_load_metadata (RBAudioCdSource *source,
+			  RhythmDB *db)
 {
 #ifdef HAVE_MUSICBRAINZ
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 
-	priv->metadata = (SjMetadata*)sj_metadata_musicbrainz_new();
+	priv->metadata = (SjMetadata*)sj_metadata_musicbrainz_new ();
 	sj_metadata_set_cdrom (priv->metadata, priv->device_path);
 
 	g_signal_connect (G_OBJECT (priv->metadata), "metadata",
@@ -599,7 +624,7 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-static GList*
+static GList *
 impl_get_ui_actions (RBSource *source)
 {
 	GList *actions = NULL;
