@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -87,7 +87,6 @@ static void paned_size_allocate_cb (GtkWidget *widget,
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static GList *impl_get_property_views (RBSource *source);
 static void impl_delete (RBSource *source);
-static void impl_move_to_trash (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
 static void impl_reset_filters (RBSource *source);
 static void impl_song_properties (RBSource *source);
@@ -200,7 +199,6 @@ rb_browser_source_class_init (RBBrowserSourceClass *klass)
 	source_class->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_move_to_trash = impl_move_to_trash;
 	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_search_actions = impl_get_search_actions;
@@ -211,13 +209,10 @@ rb_browser_source_class_init (RBBrowserSourceClass *klass)
 
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
-					 g_param_spec_uint ("entry-type",
-							    "Entry type",
-							    "Type of the entries which should be displayed by this source",
-							    0,
-							    G_MAXINT,
-							    RHYTHMDB_ENTRY_TYPE_SONG,
-							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+					 g_param_spec_pointer ("entry-type",
+							       "Entry type",
+							       "Type of the entries which should be displayed by this source",
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_object_class_install_property (object_class,
 					 PROP_SORTING_KEY,
@@ -238,6 +233,12 @@ rb_browser_source_init (RBBrowserSource *source)
 	source->priv->search_prop = RHYTHMDB_PROP_SEARCH_MATCH;
 
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
+
+	/* Default value, should be overridden at object construction by the 
+	 * "entry-type" property
+	 */
+	source->priv->entry_type = RHYTHMDB_ENTRY_TYPE_INVALID;
+
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
 }
 
@@ -376,8 +377,12 @@ rb_browser_source_constructor (GType type,
 
 	source->priv->paned = gtk_vpaned_new ();
 
-	source->priv->browser = rb_library_browser_new (source->priv->db);
+	source->priv->browser = rb_library_browser_new (source->priv->db, source->priv->entry_type);
 	gtk_paned_pack1 (GTK_PANED (source->priv->paned), GTK_WIDGET (source->priv->browser), TRUE, FALSE);
+	gtk_container_child_set (GTK_CONTAINER (source->priv->paned),
+				 GTK_WIDGET (source->priv->browser),
+				 "resize", FALSE,
+				 NULL);
 	g_signal_connect_object (G_OBJECT (source->priv->browser), "notify::output-model",
 				 G_CALLBACK (rb_browser_source_browser_changed_cb),
 				 source, 0);
@@ -477,7 +482,7 @@ rb_browser_source_set_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
-		source->priv->entry_type = g_value_get_uint (value);
+		source->priv->entry_type = g_value_get_pointer (value);
 		break;
 	case PROP_SORTING_KEY:
 		g_free (source->priv->sorting_key);
@@ -499,7 +504,7 @@ rb_browser_source_get_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
-		g_value_set_uint (value, source->priv->entry_type);
+		g_value_set_pointer (value, source->priv->entry_type);
 		break;
 	case PROP_SORTING_KEY:
 		g_value_set_string (value, source->priv->sorting_key);
@@ -516,12 +521,16 @@ rb_browser_source_cmd_choose_genre (GtkAction *action,
 {
 	GList *props;
 	RBBrowserSource *source;
+	RBPropertyView *view;
 
 	rb_debug ("choosing genre");
 
 	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_source_gather_selected_properties (RB_SOURCE (source), RHYTHMDB_PROP_GENRE);
-	rb_library_browser_set_selection (source->priv->browser, RHYTHMDB_PROP_GENRE, props);
+	view = rb_library_browser_get_property_view (source->priv->browser, RHYTHMDB_PROP_GENRE);
+	if (view)
+		rb_property_view_set_selection (view, props);
+	
 	rb_list_deep_free (props);
 	g_object_unref (source);
 }
@@ -532,12 +541,16 @@ rb_browser_source_cmd_choose_artist (GtkAction *action,
 {
 	GList *props;	
 	RBBrowserSource *source;
+	RBPropertyView *view;
 
 	rb_debug ("choosing artist");
 
 	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_source_gather_selected_properties (RB_SOURCE (source), RHYTHMDB_PROP_ARTIST);
-	rb_library_browser_set_selection (source->priv->browser, RHYTHMDB_PROP_ARTIST, props);
+	view = rb_library_browser_get_property_view (source->priv->browser, RHYTHMDB_PROP_ARTIST);
+	if (view)
+		rb_property_view_set_selection (view, props);
+
 	rb_list_deep_free (props);
 	g_object_unref (source);
 }
@@ -548,12 +561,16 @@ rb_browser_source_cmd_choose_album (GtkAction *action,
 {
 	GList *props;	
 	RBBrowserSource *source;
+	RBPropertyView *view;
 
 	rb_debug ("choosing album");
 
 	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_source_gather_selected_properties (RB_SOURCE (source), RHYTHMDB_PROP_ALBUM);
-	rb_library_browser_set_selection (source->priv->browser, RHYTHMDB_PROP_ALBUM, props);
+	view = rb_library_browser_get_property_view (source->priv->browser, RHYTHMDB_PROP_ALBUM);
+	if (view)
+		rb_property_view_set_selection (view, props);
+
 	rb_list_deep_free (props);
 	g_object_unref (source);
 }
@@ -660,21 +677,6 @@ impl_delete (RBSource *asource)
 	g_list_free (sel);
 }
 
-static void
-impl_move_to_trash (RBSource *asource)
-{
-	RBBrowserSource *source = RB_BROWSER_SOURCE (asource);
-	GList *sel, *tem;
-
-	sel = rb_entry_view_get_selected_entries (source->priv->songs);
-	for (tem = sel; tem != NULL; tem = tem->next) {
-		rhythmdb_entry_move_to_trash (source->priv->db,
-					       (RhythmDBEntry *) tem->data);
-		rhythmdb_commit (source->priv->db);
-	}
-	g_list_free (sel);
-}
-
 static void
 impl_song_properties (RBSource *asource)
 {
@@ -755,10 +757,12 @@ impl_browser_toggled (RBSource *asource, gboolean disclosed)
 {
 	RBBrowserSource *source = RB_BROWSER_SOURCE (asource);
 
-	if (disclosed)
+	if (disclosed) {
 		gtk_widget_show (GTK_WIDGET (source->priv->browser));
-	else
+	} else {
 		gtk_widget_hide (GTK_WIDGET (source->priv->browser));
+		rb_library_browser_reset (source->priv->browser);
+	}
 }
 
 static gboolean
