@@ -16,11 +16,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 
@@ -34,6 +34,7 @@
 #include "rb-stock-icons.h"
 #include "rb-debug.h"
 #include "rb-util.h"
+#include "rb-file-helpers.h"
 #include "rb-dialog.h"
 #include "rb-preferences.h"
 
@@ -54,14 +55,13 @@ static void rb_daap_source_get_property  (GObject *object,
 					  GValue *value,
 				 	  GParamSpec *pspec);
 static void rb_daap_source_activate (RBSource *source);
-static void rb_daap_source_connection_cb (RBDAAPConnection *connection,
-					  gboolean result,
-					  RBSource *source);
-static gboolean rb_daap_source_disconnect (RBSource *source);
+
 static gboolean rb_daap_source_show_popup (RBSource *source);
 static char * rb_daap_source_get_browser_key (RBSource *source);
 static const char * rb_daap_source_get_paned_key (RBBrowserSource *source);
-
+static void rb_daap_source_get_status (RBSource *source, char **text, char **progress_text, float *progress);
+static void rb_daap_source_cmd_disconnect (GtkAction *action, RBShell *shell);
+static void rb_daap_source_disconnect (RBDAAPSource *daap_source);
 
 #define CONF_ENABLE_BROWSING CONF_PREFIX "/sharing/enable_browsing"
 #define CONF_STATE_SORTING CONF_PREFIX "/state/daap/sorting"
@@ -76,20 +76,27 @@ static GHashTable *source_lookup = NULL;
 static guint enable_browsing_notify_id = EEL_GCONF_UNDEFINED_CONNECTION;
 
 static GdkPixbuf *daap_share_pixbuf        = NULL;
-static GdkPixbuf *daap_share_pixbuf_locked = NULL;
+static GdkPixbuf *daap_share_locked_pixbuf = NULL;
+static gboolean daap_was_shutdown = FALSE;
 
 struct RBDAAPSourcePrivate
 {
+	GtkActionGroup *action_group;
+
 	char *service_name;
 	char *host;
 	guint port;
 	gboolean password_protected;
 
-	RBDAAPConnection *connection;
-	GSList *playlist_sources;
-};
+	gpointer connection;
 
-#define RB_DAAP_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_DAAP_SOURCE, RBDAAPSourcePrivate))
+	GSList *playlist_sources;
+
+	const char *connection_status;
+	float connection_progress;
+
+	gboolean disconnecting;
+};
 
 enum {
 	PROP_0,
@@ -101,11 +108,19 @@ enum {
 
 G_DEFINE_TYPE (RBDAAPSource, rb_daap_source, RB_TYPE_BROWSER_SOURCE)
 
+static GtkActionEntry rb_daap_source_actions [] =
+{
+	{ "DaapSourceDisconnect", GTK_STOCK_DISCONNECT, N_("_Disconnect"), NULL, 
+	  N_("Disconnect from DAAP share"), 
+	  G_CALLBACK (rb_daap_source_cmd_disconnect) },
+};
+GtkActionGroup *daap_action_group;
+guint daap_ui_merge_id;
 
 static RhythmDBEntryType
 rhythmdb_entry_daap_type_new (void)
 {
-	return rhythmdb_entry_register_type ();
+	return rhythmdb_entry_register_type (NULL);
 }
 
 static void
@@ -115,12 +130,11 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
-	object_class->dispose = rb_daap_source_dispose;
+	object_class->dispose      = rb_daap_source_dispose;
 	object_class->get_property = rb_daap_source_get_property;
 	object_class->set_property = rb_daap_source_set_property;
 
 	source_class->impl_activate = rb_daap_source_activate;
-	source_class->impl_disconnect = rb_daap_source_disconnect;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
@@ -131,9 +145,9 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	source_class->impl_show_popup = rb_daap_source_show_popup;
 	source_class->impl_get_config_widget = NULL;
 	source_class->impl_get_browser_key = rb_daap_source_get_browser_key;
+	source_class->impl_get_status = rb_daap_source_get_status;
 
 	browser_source_class->impl_get_paned_key = rb_daap_source_get_paned_key;
-	browser_source_class->impl_has_first_added_column = (RBBrowserSourceFeatureFunc) rb_false_function;
 	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
 
 
@@ -176,10 +190,11 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 static void
 rb_daap_source_init (RBDAAPSource *source)
 {
-	source->priv = RB_DAAP_SOURCE_GET_PRIVATE (source);
+	source->priv = G_TYPE_INSTANCE_GET_PRIVATE (source, 
+						    RB_TYPE_DAAP_SOURCE, 
+						    RBDAAPSourcePrivate);
 }
 
-
 static void
 rb_daap_source_dispose (GObject *object)
 {
@@ -263,6 +278,9 @@ rb_daap_get_icon (gboolean password_protected,
 	GtkIconTheme *theme;
 	gint size;
 
+	g_return_val_if_fail (daap_share_pixbuf != NULL, NULL);
+	g_return_val_if_fail (daap_share_locked_pixbuf != NULL, NULL);
+
 	theme = gtk_icon_theme_get_default ();
 	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
 
@@ -271,7 +289,7 @@ rb_daap_get_icon (gboolean password_protected,
 	} else if (connected) {
 		icon = g_object_ref (daap_share_pixbuf);
 	} else {
-		icon = g_object_ref (daap_share_pixbuf_locked);
+		icon = g_object_ref (daap_share_locked_pixbuf);
 	}
 
 	return icon;
@@ -304,6 +322,7 @@ rb_daap_source_new (RBShell *shell,
 					  "visibility", TRUE,
 					  "sorting-key", CONF_STATE_SORTING,
 					  "password-protected", password_protected,
+					  "sourcelist-group", RB_SOURCELIST_GROUP_TRANSIENT,
 					  NULL));
 
 	if (icon != NULL) {
@@ -379,7 +398,8 @@ mdns_service_removed (RBDaapMdnsBrowser *browser,
 static void
 remove_source (RBSource *source)
 {
-	rb_daap_source_disconnect (source);
+	rb_debug ("Removing DAAP source: %s", RB_DAAP_SOURCE (source)->priv->service_name);
+	rb_daap_source_disconnect (RB_DAAP_SOURCE (source));
 	rb_source_delete_thyself (source);
 }
 
@@ -432,6 +452,8 @@ stop_browsing (RBShell *shell)
 		return;
 	}
 
+	rb_debug ("Destroying DAAP source lookup");
+
 	g_hash_table_destroy (source_lookup);
 	source_lookup = NULL;
 
@@ -533,7 +555,7 @@ create_pixbufs (void)
 	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &size, NULL);
 	emblem = gtk_icon_theme_load_icon (theme, "stock_lock", size, 0, NULL);
 
-	daap_share_pixbuf_locked = composite_icons (daap_share_pixbuf, emblem);
+	daap_share_locked_pixbuf = composite_icons (daap_share_pixbuf, emblem);
 }
 
 static void
@@ -544,10 +566,10 @@ destroy_pixbufs (void)
 	}
 	daap_share_pixbuf = NULL;
 
-	if (daap_share_pixbuf_locked != NULL) {
-		g_object_unref (daap_share_pixbuf_locked);
+	if (daap_share_locked_pixbuf != NULL) {
+		g_object_unref (daap_share_locked_pixbuf);
 	}
-	daap_share_pixbuf_locked = NULL;
+	daap_share_locked_pixbuf = NULL;
 }
 
 RBSource *
@@ -556,6 +578,7 @@ rb_daap_sources_init (RBShell *shell)
 	gboolean enabled = TRUE;
 	GConfValue *value;
 	GConfClient *client = eel_gconf_client_get_global ();
+	GtkUIManager *uimanager = NULL;
 
 	value = gconf_client_get_without_default (client,
 						  CONF_ENABLE_BROWSING, NULL);
@@ -577,12 +600,39 @@ rb_daap_sources_init (RBShell *shell)
 
 	create_pixbufs ();
 
+	/* add UI */
+	g_object_get (G_OBJECT (shell),
+		      "ui-manager", &uimanager, 
+		      NULL);
+	daap_action_group = gtk_action_group_new ("DaapActions");
+	gtk_action_group_set_translation_domain (daap_action_group,
+						 GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (daap_action_group,
+				      rb_daap_source_actions, G_N_ELEMENTS (rb_daap_source_actions),
+				      shell);
+	gtk_ui_manager_insert_action_group (uimanager, daap_action_group, 0);
+	daap_ui_merge_id = gtk_ui_manager_add_ui_from_file (uimanager,
+							    rb_file ("daap-ui.xml"),
+							    NULL);
+	g_object_unref (G_OBJECT (uimanager));
+
 	return NULL;
 }
 
 void
 rb_daap_sources_shutdown (RBShell *shell)
 {
+	GtkUIManager *uimanager = NULL;
+
+	rb_debug ("Shutting down DAAP sources");
+
+	g_assert (!daap_was_shutdown);
+	daap_was_shutdown = TRUE;
+
+	g_object_get (G_OBJECT (shell),
+		      "ui-manager", &uimanager, 
+		      NULL);
+
 	if (mdns_browser) {
 		stop_browsing (shell);
 	}
@@ -592,9 +642,13 @@ rb_daap_sources_shutdown (RBShell *shell)
 		enable_browsing_notify_id = EEL_GCONF_UNDEFINED_CONNECTION;
 	}
 
-	destroy_pixbufs ();
+	gtk_ui_manager_remove_ui (uimanager, daap_ui_merge_id);
+	gtk_ui_manager_remove_action_group (uimanager, daap_action_group);
 
+	g_object_unref (G_OBJECT (uimanager));
 	g_object_unref (shell);
+
+	destroy_pixbufs ();
 }
 
 static char *
@@ -612,14 +666,41 @@ connection_auth_cb (RBDAAPConnection *connection,
 }
 
 static void
-connection_connected_cb (RBDAAPConnection *connection,
-			 RBDAAPSource     *source)
+connection_connecting_cb (RBDAAPConnection     *connection,
+			  RBDAAPConnectionState state,
+			  float		        progress,
+			  RBDAAPSource         *source)
 {
 	GdkPixbuf *icon;
+	gboolean   is_connected;
 
-	rb_debug ("DAAP connection connected");
+	rb_debug ("DAAP connection status: %d/%f", state, progress);
 
-	icon = rb_daap_get_icon (source->priv->password_protected, TRUE);
+	switch (state) {
+	case DAAP_GET_INFO:
+	case DAAP_GET_PASSWORD:
+	case DAAP_LOGIN:
+		source->priv->connection_status = _("Connecting to music share");
+		break;
+	case DAAP_GET_REVISION_NUMBER:
+	case DAAP_GET_DB_INFO:
+	case DAAP_GET_SONGS:
+	case DAAP_GET_PLAYLISTS:
+	case DAAP_GET_PLAYLIST_ENTRIES:
+		source->priv->connection_status = _("Retrieving songs from music share");
+		break;
+	case DAAP_LOGOUT:
+	case DAAP_DONE:
+		source->priv->connection_status = NULL;
+		break;
+	}
+
+	source->priv->connection_progress = progress;
+
+	rb_source_notify_status_changed (RB_SOURCE (source));
+
+	is_connected = rb_daap_connection_is_connected (connection);
+	icon = rb_daap_get_icon (source->priv->password_protected, is_connected);
 	g_object_set (source, "icon", icon, NULL);
 	if (icon != NULL) {
 		g_object_unref (icon);
@@ -634,6 +715,10 @@ connection_disconnected_cb (RBDAAPConnection *connection,
 
 	rb_debug ("DAAP connection disconnected");
 
+	if (daap_was_shutdown) {
+		return;
+	}
+
 	icon = rb_daap_get_icon (source->priv->password_protected, FALSE);
 	g_object_set (source, "icon", icon, NULL);
 	if (icon != NULL) {
@@ -642,60 +727,20 @@ connection_disconnected_cb (RBDAAPConnection *connection,
 }
 
 static void
-rb_daap_source_activate (RBSource *source)
+release_connection (RBDAAPSource *daap_source)
 {
-	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
-	RBShell *shell = NULL;
-	RhythmDB *db = NULL;
-	char *name = NULL;
-	RhythmDBEntryType type;
+	rb_debug ("Releasing connection");
 
-	if (daap_source->priv->connection != NULL) 
-		return;
-
-	g_object_get (G_OBJECT (daap_source), 
-		      "shell", &shell, 
-		      "entry-type", &type, 
-		      "name", &name, 
-		      NULL);
-	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-
-	daap_source->priv->connection =
-		rb_daap_connection_new (name,
-					daap_source->priv->host,
-					daap_source->priv->port,
-					daap_source->priv->password_protected,
-					db,
-					type,
-					(RBDAAPConnectionCallback) rb_daap_source_connection_cb,
-					source);
-	g_object_unref (G_OBJECT (db));
-	g_object_unref (G_OBJECT (shell));
-
-	if (daap_source->priv->connection == NULL) {
-		/* XXX can this still happen? */
-		daap_source->priv->playlist_sources = NULL;
-		return;
-	}
-
-        g_signal_connect (daap_source->priv->connection,
-			  "authenticate",
-                          G_CALLBACK (connection_auth_cb),
-			  source);
-        g_signal_connect (daap_source->priv->connection,
-			  "connected",
-                          G_CALLBACK (connection_connected_cb),
-			  source);
-        g_signal_connect (daap_source->priv->connection,
-			  "disconnected",
-                          G_CALLBACK (connection_disconnected_cb),
-			  source);
+	g_object_unref (G_OBJECT (daap_source->priv->connection));
+	/* Weak pointer will set to NULL for us */
+	/*daap_source->priv->connection = NULL;*/
 }
 
 static void
 rb_daap_source_connection_cb (RBDAAPConnection *connection,
-			      gboolean result,
-			      RBSource *source)
+			      gboolean          result,
+			      const char       *reason,
+			      RBSource         *source)
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
 	RBShell *shell = NULL;
@@ -703,8 +748,18 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 	GSList *l;
 	RhythmDBEntryType entry_type;
 
+	rb_debug ("Connection callback result: %s", result ? "success" : "failure");
+
 	if (result == FALSE) {
-		/* FIXME display error?  should get more info from the connection.. */
+		if (reason != NULL) {
+			rb_error_dialog	(NULL, _("Could not connect to shared music"), "%s", reason);
+		}
+
+		/* Don't release the connection if we are already disconnecting */
+		if (! daap_source->priv->disconnecting) {
+			release_connection (daap_source);
+		}
+
 		return;
 	}
 
@@ -712,7 +767,7 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 		      "shell", &shell, 
 		      "entry-type", &entry_type,
 		      NULL);
-	playlists = rb_daap_connection_get_playlists (daap_source->priv->connection);
+	playlists = rb_daap_connection_get_playlists (RB_DAAP_CONNECTION (daap_source->priv->connection));
 	for (l = playlists; l != NULL; l = g_slist_next (l)) {
 		RBDAAPPlaylist *playlist = l->data;
 		RBSource *playlist_source;
@@ -726,25 +781,91 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 	g_object_unref (G_OBJECT (shell));
 }
 
+static void
+rb_daap_source_activate (RBSource *source)
+{
+	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
+	RBShell *shell = NULL;
+	RhythmDB *db = NULL;
+	char *name = NULL;
+	RhythmDBEntryType type;
+
+	if (daap_source->priv->connection != NULL) {
+		return;
+	}
+
+	g_object_get (G_OBJECT (daap_source), 
+		      "shell", &shell, 
+		      "entry-type", &type, 
+		      "name", &name, 
+		      NULL);
+	g_object_get (G_OBJECT (shell), "db", &db, NULL);
+
+	daap_source->priv->connection = rb_daap_connection_new (name,
+								daap_source->priv->host,
+								daap_source->priv->port,
+								daap_source->priv->password_protected,
+								db,
+								type);
+	g_object_add_weak_pointer (G_OBJECT (daap_source->priv->connection), (gpointer *)&daap_source->priv->connection);
+
+	rb_daap_connection_connect (RB_DAAP_CONNECTION (daap_source->priv->connection),
+				    (RBDAAPConnectionCallback) rb_daap_source_connection_cb,
+				    source);
+
+	g_object_unref (G_OBJECT (db));
+	g_object_unref (G_OBJECT (shell));
+
+        g_signal_connect (daap_source->priv->connection,
+			  "authenticate",
+                          G_CALLBACK (connection_auth_cb),
+			  source);
+        g_signal_connect (daap_source->priv->connection,
+			  "connecting",
+                          G_CALLBACK (connection_connecting_cb),
+			  source);
+        g_signal_connect (daap_source->priv->connection,
+			  "disconnected",
+                          G_CALLBACK (connection_disconnected_cb),
+			  source);
+}
+
 static void
 rb_daap_source_disconnect_cb (RBDAAPConnection *connection,
-			      gboolean result,
-			      RBSource *source)
+			      gboolean          result,
+			      const char       *reason,
+			      RBSource         *source)
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
 
 	rb_debug ("DAAP source disconnected");
 
-	g_object_unref (G_OBJECT (connection));
-	daap_source->priv->connection = NULL;
+	release_connection (daap_source);
+
 	g_object_unref (source);
 }
 
-static gboolean
-rb_daap_source_disconnect (RBSource *source)
+static void
+rb_daap_source_cmd_disconnect (GtkAction *action,
+			       RBShell   *shell)
 {
-	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
+	RBSource *source;
 
+	g_object_get (G_OBJECT (shell), 
+		      "selected-source", &source,
+		      NULL);
+
+	if (!RB_IS_DAAP_SOURCE (source)) {
+		g_critical ("got non-Daap source for Daap action");
+		return;
+	}
+		
+	rb_daap_source_disconnect (RB_DAAP_SOURCE (source));
+}
+
+static void
+rb_daap_source_disconnect (RBDAAPSource *daap_source)
+{
 	if (daap_source->priv->connection) {
 		GSList *l;
 		RBShell *shell;
@@ -753,7 +874,9 @@ rb_daap_source_disconnect (RBSource *source)
 
 		rb_debug ("Disconnecting source");
 
-		g_object_get (G_OBJECT (source), "shell", &shell, "entry-type", &type, NULL);
+		daap_source->priv->disconnecting = TRUE;
+
+		g_object_get (G_OBJECT (daap_source), "shell", &shell, "entry-type", &type, NULL);
 		g_object_get (G_OBJECT (shell), "db", &db, NULL);
 		g_object_unref (G_OBJECT (shell));
 
@@ -770,14 +893,27 @@ rb_daap_source_disconnect (RBSource *source)
 		g_slist_free (daap_source->priv->playlist_sources);
 		daap_source->priv->playlist_sources = NULL;
 
+		/* we don't want these firing while we are disconnecting */
+		g_signal_handlers_disconnect_by_func (daap_source->priv->connection,
+						      G_CALLBACK (connection_connecting_cb),
+						      daap_source);
+		g_signal_handlers_disconnect_by_func (daap_source->priv->connection,
+						      G_CALLBACK (connection_auth_cb),
+						      daap_source);
+
 		/* keep the source alive until the disconnect completes */
 		g_object_ref (daap_source);
-		rb_daap_connection_logout (daap_source->priv->connection,
-					   (RBDAAPConnectionCallback) rb_daap_source_disconnect_cb,
-					   daap_source);
-	}
+		rb_daap_connection_disconnect (daap_source->priv->connection,
+					       (RBDAAPConnectionCallback) rb_daap_source_disconnect_cb,
+					       daap_source);
 
-	return TRUE;
+		/* wait until disconnected */
+		rb_debug ("Waiting for DAAP connection to finish");
+		while (daap_source->priv->connection != NULL) {
+			rb_debug ("Waiting for DAAP connection to finish...");
+			gtk_main_iteration ();
+		}
+	}
 }
 
 static gboolean
@@ -882,3 +1018,36 @@ rb_daap_source_get_paned_key (RBBrowserSource *source)
 	return CONF_STATE_PANED_POSITION;
 }
 
+static void
+rb_daap_source_get_status (RBSource *source,
+			   char    **text,
+			   char    **progress_text,
+			   float    *progress)
+{
+	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
+
+	if (text != NULL) {
+		*text = NULL;
+	}
+	if (progress_text != NULL) {
+		*progress_text = NULL;
+	}
+	if (progress != NULL) {
+		*progress = 0.0;
+	}
+
+	if (daap_source->priv->connection_status != NULL) {
+		if (text != NULL) {
+			*text = g_strdup (daap_source->priv->connection_status);
+		}
+
+		if (progress != NULL) {
+			*progress = daap_source->priv->connection_progress;
+		}
+
+		return;
+	}
+	
+	RB_SOURCE_CLASS (rb_daap_source_parent_class)->impl_get_status (source, text, progress_text, progress);
+}
+
