@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -42,6 +42,7 @@
 #include "eel-gconf-extensions.h"
 #include "rb-library-source.h"
 #include "rb-removable-media-manager.h"
+#include "rb-auto-playlist-source.h"
 
 static void rb_library_source_class_init (RBLibrarySourceClass *klass);
 static void rb_library_source_init (RBLibrarySource *source);
@@ -92,6 +93,7 @@ static void rb_library_source_watch_toggled_cb (GtkToggleButton *button,
 static void rb_library_source_songs_show_popup_cb (RBEntryView *view,
 						   gboolean over_entry,
 						   RBLibrarySource *source);						
+static void rb_library_source_sync_child_sources (RBLibrarySource *source);
 #ifdef ENABLE_TRACK_TRANSFER
 static void rb_library_source_path_changed_cb (GtkComboBox *box,
 						RBLibrarySource *source);
@@ -142,6 +144,8 @@ struct RBLibrarySourcePrivate
 
 	GtkWidget *config_widget;
 
+	GList *child_sources;
+
 	GtkWidget *library_location_entry;
 	GtkWidget *watch_library_check;
 #ifdef ENABLE_TRACK_TRANSFER
@@ -183,7 +187,6 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 #endif
 
 	browser_source_class->impl_get_paned_key = impl_get_paned_key;
-	browser_source_class->impl_has_first_added_column = (RBBrowserSourceFeatureFunc) rb_true_function;
 	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_true_function;
 
 	g_type_class_add_private (klass, sizeof (RBLibrarySourcePrivate));
@@ -241,6 +244,14 @@ rb_library_source_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_library_source_parent_class)->finalize (object);
 }
 
+static gboolean
+add_child_sources_idle (RBLibrarySource *source)
+{
+	rb_library_source_sync_child_sources (source);
+
+	return FALSE;
+}
+
 static GObject *
 rb_library_source_constructor (GType type,
 			       guint n_construct_properties,
@@ -275,6 +286,8 @@ rb_library_source_constructor (GType type,
 	g_signal_connect_object (G_OBJECT (rb_source_get_entry_view (RB_SOURCE (source))), "show_popup",
 				 G_CALLBACK (rb_library_source_songs_show_popup_cb), source, 0);
 
+	g_idle_add ((GSourceFunc)add_child_sources_idle, source);
+
 	g_object_unref (G_OBJECT (shell));
 
 	return G_OBJECT (source);
@@ -302,6 +315,7 @@ rb_library_source_new (RBShell *shell)
 					  NULL));
 	rb_shell_register_entry_type_for_source (shell, source,
 						 RHYTHMDB_ENTRY_TYPE_SONG);
+
 	return source;
 }
 
@@ -448,6 +462,8 @@ rb_library_source_library_location_changed (GConfClient *client,
 {
 	if (source->priv->config_widget)
 		rb_library_source_preferences_sync (source);
+
+	rb_library_source_sync_child_sources (source);
 }
 
 static void
@@ -948,7 +964,7 @@ layout_example_label_update (RBLibrarySource *source)
 	format = g_strconcat ("<small><i><b>Example Path:</b> ",
 			      example,
 			      ".",
-			      gm_audio_profile_get_extension (profile),
+			      profile ? gm_audio_profile_get_extension (profile) : "ogg",
 			      "</i></small>", NULL);
 	g_free (example);
   
@@ -1036,7 +1052,7 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	char *realpath;
 	char *filename;
 	char *string;
-	char *extension;
+	char *extension = NULL;
 	GSList *list;
 	const char *layout_path;
 	const char *layout_filename;
@@ -1050,6 +1066,7 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	if (list == NULL || layout_path == NULL || layout_filename == NULL || preferred_format == NULL) {
 		/* emit warning */
 		rb_debug ("Could not retrieve settings from GConf");
+		g_slist_free (list);
 		return NULL;
 	}
 
@@ -1065,8 +1082,11 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	if (g_str_has_prefix (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE), "audio/x-raw")) {
 		GMAudioProfile *profile;
 		profile = gm_audio_profile_lookup (preferred_format);
-		extension = g_strdup (gm_audio_profile_get_extension (profile));
-	} else {
+		if (profile)
+			extension = g_strdup (gm_audio_profile_get_extension (profile));
+	}
+
+	if (extension == NULL) {
 		char *tmp;
 		
 		/* use the old extension. strip anything after a '?' for http/daap/etc */
@@ -1102,13 +1122,17 @@ impl_can_paste (RBSource *asource)
 {
 #ifdef ENABLE_TRACK_TRANSFER
 	GSList *list;
+	gboolean can_paste = TRUE;
 
 	list = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
-	if (list == NULL)
-		return FALSE;
+	can_paste = ((list != NULL) &&
+		     (eel_gconf_get_string (CONF_LIBRARY_LAYOUT_PATH) != NULL ) &&
+		     (eel_gconf_get_string (CONF_LIBRARY_LAYOUT_FILENAME) != NULL) &&
+		     (eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT) != NULL));
+
 
 	g_slist_free (list);
-	return TRUE;
+	return can_paste;
 #else
 	return FALSE;
 #endif
@@ -1127,8 +1151,19 @@ impl_paste (RBSource *asource, GList *entries)
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
 	RBRemovableMediaManager *rm_mgr;
 	GList *l;
+	GSList *sl;
 	RBShell *shell;
 
+	sl = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
+	if ((sl == NULL) ||
+	    (eel_gconf_get_string (CONF_LIBRARY_LAYOUT_PATH) == NULL )||
+	    (eel_gconf_get_string (CONF_LIBRARY_LAYOUT_FILENAME) == NULL) ||
+	    (eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT) == NULL)) {
+		g_slist_free (sl);
+		g_warning ("RBLibrarySource impl_paste called when gconf keys unset");
+		return;
+	}
+
 	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
 	g_object_get (G_OBJECT (shell), "removable-media-manager", &rm_mgr, NULL);
 	g_object_unref (G_OBJECT (shell));
@@ -1140,7 +1175,11 @@ impl_paste (RBSource *asource, GList *entries)
 
 		rb_debug ("pasting entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 
-		entry_type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+		entry_type = rhythmdb_entry_get_entry_type (entry);
+		if (entry_type == RHYTHMDB_ENTRY_TYPE_SONG)
+			/* copying to ourselves would be silly */
+			continue;
+		
 		/* see if the responsible source lets us copy */
 		if (!rb_source_can_copy (rb_shell_get_source_by_entry_type (shell, entry_type)))
 			continue;
@@ -1160,3 +1199,56 @@ impl_paste (RBSource *asource, GList *entries)
 }
 #endif
 
+static void
+rb_library_source_add_child_source (const char *path, RBLibrarySource *library_source)
+{
+	RBSource *source;
+	GPtrArray *query;
+	RBShell *shell;
+	GnomeVFSURI *uri;
+	char *name;
+	GdkPixbuf *icon;
+
+	g_object_get (G_OBJECT (library_source), "shell", &shell, NULL);
+	uri = gnome_vfs_uri_new (path);
+	name = gnome_vfs_uri_extract_short_name (uri);
+	gnome_vfs_uri_unref (uri);
+	
+	source = rb_auto_playlist_source_new (shell, name, FALSE);
+	query = rhythmdb_query_parse (library_source->priv->db,
+				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_SONG,
+				      RHYTHMDB_QUERY_PROP_PREFIX, RHYTHMDB_PROP_LOCATION, path,
+				      RHYTHMDB_QUERY_END);
+	rb_auto_playlist_source_set_query (RB_AUTO_PLAYLIST_SOURCE (source), query,
+					   RHYTHMDB_QUERY_MODEL_LIMIT_NONE, NULL,
+					   NULL, 0);
+	rhythmdb_query_free (query);
+
+	g_object_get (G_OBJECT (library_source), "icon", &icon, NULL);
+	g_object_set (G_OBJECT (source), "icon", g_object_ref (icon), NULL);
+
+	rb_shell_append_source (shell, source, RB_SOURCE (library_source));
+	library_source->priv->child_sources = g_list_prepend (library_source->priv->child_sources, source);
+
+	g_object_unref (shell);
+	g_free (name);
+}
+
+static void
+rb_library_source_sync_child_sources (RBLibrarySource *source)
+{
+	GSList *list;
+
+	list = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
+
+	/* FIXME: don't delete and re-create sources that are still there */
+	g_list_foreach (source->priv->child_sources, (GFunc)rb_source_delete_thyself, NULL);
+	g_list_free (source->priv->child_sources);
+	source->priv->child_sources = NULL;
+
+	if (g_slist_length (list) > 1)
+		g_slist_foreach (list, (GFunc)rb_library_source_add_child_source, source);
+	g_slist_foreach (list, (GFunc) g_free, NULL);
+	g_slist_free (list);
+}
+
