@@ -1,5 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of playlist source recorder object
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
@@ -18,7 +18,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -38,6 +38,14 @@
 #include <nautilus-burn-drive.h>
 #include <nautilus-burn-drive-selection.h>
 
+#ifndef NAUTILUS_BURN_CHECK_VERSION 	 
+#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE 	 
+#endif
+
+#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
+#include <nautilus-burn.h>
+#endif
+
 #include "rb-file-helpers.h"
 #include "rb-glade-helpers.h"
 #include "rb-preferences.h"
@@ -297,8 +305,9 @@ update_speed_combobox (RBPlaylistSourceRecorder *source)
 
 
         default_speed = eel_gconf_get_integer (CONF_STATE_BURN_SPEED);
-        default_speed_index = 0;
+        default_speed_index = -1;
 
+        i = 0;
         if (drive) {
                 const int *write_speeds;
 
@@ -306,7 +315,11 @@ update_speed_combobox (RBPlaylistSourceRecorder *source)
 
                 for (i = 0; write_speeds [i] > 0; i++) {
 
+#ifdef NAUTILUS_BURN_DRIVE_CD_SPEED
+                        name = g_strdup_printf ("%d \303\227", NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds [i]));
+#else
                         name = g_strdup_printf ("%d \303\227", write_speeds [i]);
+#endif
 
                         if (write_speeds [i] == default_speed) {
                                 default_speed_index = i + 1;
@@ -320,8 +333,14 @@ update_speed_combobox (RBPlaylistSourceRecorder *source)
                         g_free (name);
                 }
 
-                /* Disable speed if no items in list */
-                gtk_widget_set_sensitive (combobox, i > 0);
+        }
+
+        /* Disable speed if no items in list */
+        gtk_widget_set_sensitive (combobox, i > 0);
+
+        /* if the default speed was not set then make it the minimum for safety */
+        if (default_speed_index == -1) {
+                default_speed_index = i;
         }
 
         /* for now assume equivalence between index in comboxbox and speed */
@@ -612,9 +631,6 @@ burn_cd (RBPlaylistSourceRecorder *source,
         res = rb_recorder_burn (source->priv->recorder, speed, error);
         source->priv->burning = FALSE;
 
-        if (error && *error)
-                return RB_RECORDER_RESULT_ERROR;
-
         if (res == RB_RECORDER_RESULT_FINISHED) {
                 NautilusBurnDrive *drive;
                 gboolean           do_another;
@@ -639,6 +655,8 @@ burn_cd (RBPlaylistSourceRecorder *source,
                         return res;
                 }
                 set_message_text (source, _("Finished creating audio CD.\nCreate another copy?"));
+        } else if (res == RB_RECORDER_RESULT_ERROR) {
+                set_message_text (source, _("Writing failed.  Try again?"));  
         } else {
                 set_message_text (source, _("Writing cancelled.  Try again?"));  
         }
@@ -658,7 +676,7 @@ get_song_description (RBRecorderSong *song)
         char *desc = NULL;
 
         if (song->artist && song->title)
-                desc = g_strdup_printf ("%s - %s", song->artist, song->title);
+                desc = g_strdup_printf ("%s - %s", song->title, song->artist);
         else if (song->title)
                 desc = g_strdup (song->title);
         else if (song->artist)
@@ -701,8 +719,9 @@ static gboolean
 burn_cd_idle (RBPlaylistSourceRecorder *source)
 {
         GError *error = NULL;
+        int     res;
 
-        burn_cd (source, &error);
+        res = burn_cd (source, &error);
         if (error) {
                 error_dialog (source,
                               _("Audio recording error"),
@@ -866,12 +885,13 @@ response_cb (GtkDialog *dialog,
 
         /* Act only on response IDs we recognize */
         if (!(response_id == GTK_RESPONSE_ACCEPT
-              || response_id == GTK_RESPONSE_CANCEL)) {
+              || response_id == GTK_RESPONSE_CANCEL
+	      || response_id == GTK_RESPONSE_DELETE_EVENT)) {
                 g_signal_stop_emission_by_name (dialog, "response");
                 return;
         }
 
-        if (response_id == GTK_RESPONSE_CANCEL) {
+        if (response_id == GTK_RESPONSE_CANCEL || response_id == GTK_RESPONSE_DELETE_EVENT) {
                 if (source->priv->burning
                     && !source->priv->confirmed_exit) {
                         GtkWidget *interrupt_dialog;
@@ -1035,7 +1055,13 @@ ask_rewrite_disc (RBPlaylistSourceRecorder *source,
         char                 *msg;
         NautilusBurnDrive    *drive;
 
+#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
+        drive = nautilus_burn_drive_monitor_get_drive_for_device (nautilus_burn_get_drive_monitor (),
+                                                                  device);
+#else
         drive = nautilus_burn_drive_new_from_path (device);
+#endif
+
         type = nautilus_burn_drive_get_media_type (drive);
 
         msg = g_strdup_printf (_("This %s appears to have information already recorded on it."),
