@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -50,7 +50,6 @@
 #include "rb-feed-podcast-properties-dialog.h"
 #include "rb-playlist-manager.h"
 #include "rb-debug.h"
-#include "rb-statusbar.h"
 #include "eel-gconf-extensions.h"
 #include "rb-podcast-manager.h"
 #include "rb-static-playlist-source.h"
@@ -232,6 +231,10 @@ static gboolean impl_can_add_to_queue			(RBSource *source);
 static void impl_add_to_queue				(RBSource *source, RBSource *queue);
 static GList *impl_get_ui_actions			(RBSource *source);
 static GList *impl_get_search_actions			(RBSource *source);
+static void impl_get_status				(RBSource *source, 
+							 char **text, 
+							 char **progress_text, 
+							 float *progress);
 
 
 
@@ -365,16 +368,14 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 	source_class->impl_search = impl_search;
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_song_properties = impl_song_properties;
+	source_class->impl_get_status = impl_get_status;
 	
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
-					 g_param_spec_uint ("entry-type",
-							    "Entry type",
-							    "Type of the entries which should be displayed by this source",
-							    0,
-							    G_MAXINT,
-							    RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
-							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+					 g_param_spec_pointer ("entry-type",
+							       "Entry type",
+							       "Type of the entries which should be displayed by this source",
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_object_class_install_property (object_class,
 					 PROP_PODCAST_MANAGER,
@@ -675,6 +676,10 @@ rb_podcast_source_constructor (GType type,
 	g_object_set (G_OBJECT (feed_model), "query-model", query_model, NULL);
 	g_object_unref (G_OBJECT (feed_model));
 	
+	if (source->priv->entry_type == RHYTHMDB_ENTRY_TYPE_INVALID ||
+	    source->priv->entry_type == NULL)
+		source->priv->entry_type = RHYTHMDB_ENTRY_TYPE_PODCAST_POST;
+	
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
 				      RHYTHMDB_PROP_TYPE,
@@ -772,7 +777,7 @@ rb_podcast_source_set_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
-		source->priv->entry_type = g_value_get_uint (value);
+		source->priv->entry_type = g_value_get_pointer (value);
 		break;
 	case PROP_PODCAST_MANAGER:
 		source->priv->podcast_mg = g_value_get_object (value);
@@ -793,7 +798,7 @@ rb_podcast_source_get_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
-		g_value_set_uint (value, source->priv->entry_type);
+		g_value_set_pointer (value, source->priv->entry_type);
 		break;
 	case PROP_PODCAST_MANAGER:
 		g_value_set_object (value, source->priv->podcast_mg);
@@ -1962,3 +1967,26 @@ rb_podcast_source_cmd_new_podcast (GtkAction *action,
 	gtk_widget_destroy (dialog);
 }
 
+static void
+impl_get_status (RBSource *source, char **text, char **progress_text, float *progress)
+{
+	RhythmDBQueryModel *query_model;
+
+	/* hack to get these strings marked for translation */
+	if (0) {
+		ngettext ("%d episode", "%d episodes", 0);
+	}
+
+	g_object_get (G_OBJECT (source), "query-model", &query_model, NULL);
+	if (query_model) {
+		*text = rhythmdb_query_model_compute_status_normal (query_model, 
+								    "%d episode", 
+								    "%d episodes");
+		if (rhythmdb_query_model_has_pending_changes (query_model))
+			*progress = -1.0f;
+		g_object_unref (G_OBJECT (query_model));
+	} else {
+		*text = g_strdup ("");
+	}
+}
+
