@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -63,9 +63,9 @@ static void default_delete_thyself (RBSource *source);
 static RBEntryView* default_get_entry_view (RBSource *source);
 static void default_activate (RBSource *source);
 static void default_deactivate (RBSource *source);
-static gboolean default_disconnect (RBSource *source);
 static void default_add_to_queue (RBSource *source, RBSource *queue);
-static char *default_get_status (RBSource *source);
+static void default_get_status (RBSource *source, char **text, char **progress_text, float *progress);
+static void default_move_to_trash (RBSource *source);
 static GList * default_get_ui_actions (RBSource *source);
 static GList * default_get_search_actions (RBSource *source);
 
@@ -88,6 +88,7 @@ struct _RBSourcePrivate
 	RhythmDBQueryModel *query_model;
 	guint idle_status_changed_id;
 	GdkPixbuf *pixbuf;
+	RBSourceListGroup sourcelist_group;
 };
 
 enum
@@ -98,7 +99,8 @@ enum
 	PROP_SHELL,
 	PROP_UI_MANAGER,
 	PROP_VISIBLE,
-	PROP_QUERY_MODEL
+	PROP_QUERY_MODEL,
+	PROP_SOURCELIST_GROUP
 };
 
 enum
@@ -145,12 +147,12 @@ rb_source_class_init (RBSourceClass *klass)
 	klass->impl_delete_thyself = default_delete_thyself;
 	klass->impl_activate = default_activate;
 	klass->impl_deactivate = default_deactivate;
-	klass->impl_disconnect = default_disconnect;
 	klass->impl_try_playlist = default_try_playlist;
 	klass->impl_add_to_queue = default_add_to_queue;
 	klass->impl_get_status = default_get_status;
 	klass->impl_get_ui_actions = default_get_ui_actions;
 	klass->impl_get_search_actions = default_get_search_actions;
+	klass->impl_move_to_trash = default_move_to_trash;
 
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
@@ -205,6 +207,14 @@ rb_source_class_init (RBSourceClass *klass)
 							      "RhythmDBQueryModel object",
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_SOURCELIST_GROUP,
+					 g_param_spec_enum ("sourcelist-group",
+						 	    "sourcelist group",
+							    "sourcelist group",
+							    RB_TYPE_SOURCELIST_GROUP,
+							    RB_SOURCELIST_GROUP_FIXED,
+							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	rb_source_signals[DELETED] =
 		g_signal_new ("deleted",
@@ -354,6 +364,9 @@ rb_source_set_property (GObject *object,
 		/* g_object_notify (G_OBJECT (source), "query-model"); */
 		rb_source_notify_status_changed (source);
 		break;
+	case PROP_SOURCELIST_GROUP:
+		priv->sourcelist_group = g_value_get_enum (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -394,37 +407,52 @@ rb_source_get_property (GObject *object,
 	case PROP_QUERY_MODEL:
 		g_value_set_object (value, priv->query_model);
 		break;
+	case PROP_SOURCELIST_GROUP:
+		g_value_set_enum (value, priv->sourcelist_group);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-static char *
-default_get_status (RBSource *source)
+static void
+default_get_status (RBSource *source, char **text, char **progress_text, float *progress)
 {
 	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
-	if (priv->query_model)
-		return rhythmdb_query_model_compute_status_normal (priv->query_model);
-
-	return g_strdup ("");
+	/* hack to get these strings marked for translation */
+	if (0) {
+		ngettext ("%d song", "%d songs", 0);
+	}
+	if (priv->query_model) {
+		*text = rhythmdb_query_model_compute_status_normal (priv->query_model, 
+								    "%d song", 
+								    "%d songs");
+		if (rhythmdb_query_model_has_pending_changes (priv->query_model))
+			*progress = -1.0f;
+	} else {
+		*text = g_strdup ("");
+	}
 }
 
 /**
  * rb_source_get_status:
  * @status: a #RBSource
+ * @text: holds the returned status text (allocated)
+ * @progress_text: holds the returned text for the progress bar (allocated)
+ * @progress: holds the progress value
  *
- * FIXME:
- * Some Random comments
- *
- * Returns: The status string
+ * Returns the details to display in the status bar for the source.
+ * If the progress value returned is less than zero, the progress bar
+ * will pulse.  If the progress value is greater than or equal to 1,
+ * the progress bar will be hidden.
  **/
-char *
-rb_source_get_status (RBSource *source)
+void
+rb_source_get_status (RBSource *source, char **text, char **progress_text, float *progress)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
-	return klass->impl_get_status (source);
+	klass->impl_get_status (source, text, progress_text, progress);
 }
 
 static char *
@@ -695,6 +723,27 @@ rb_source_delete (RBSource *source)
 	klass->impl_delete (source);
 }
 
+static void
+default_move_to_trash (RBSource *source)
+{
+	GList *sel, *tem;
+	RBEntryView *entry_view;
+	RhythmDB *db;
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+
+	g_object_get (G_OBJECT (priv->shell), "db", &db, NULL);
+
+	entry_view = rb_source_get_entry_view (source);
+	sel = rb_entry_view_get_selected_entries (entry_view);
+	for (tem = sel; tem != NULL; tem = tem->next) {
+		rhythmdb_entry_move_to_trash (db, (RhythmDBEntry *)tem->data);
+		rhythmdb_commit (db);
+	}
+	g_list_free (sel);
+	g_object_unref (G_OBJECT (db));
+}
+
+
 void
 rb_source_move_to_trash (RBSource *source)
 {
@@ -843,11 +892,6 @@ static void default_deactivate (RBSource *source)
 	return;
 }
 
-static gboolean default_disconnect (RBSource *source)
-{
-	return TRUE;
-}
-
 void rb_source_activate (RBSource *source)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
@@ -866,13 +910,6 @@ void rb_source_deactivate (RBSource *source)
 	return;
 }
 
-gboolean rb_source_disconnect (RBSource *source)
-{
-	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
-
-	return klass->impl_disconnect (source);
-}
-
 static GList *
 default_get_ui_actions (RBSource *source)
 {
@@ -1063,3 +1100,22 @@ rb_source_eof_type_get_type (void)
 	return etype;
 }
 
+GType
+rb_sourcelist_group_get_type (void)
+{
+	static GType etype = 0;
+
+	if (etype == 0) {
+		static const GEnumValue values[] = {
+			ENUM_ENTRY (RB_SOURCELIST_GROUP_FIXED, "Fixed single instance source"),
+			ENUM_ENTRY (RB_SOURCELIST_GROUP_PERSISTANT, "Persistant multiple-instance source"),
+			ENUM_ENTRY (RB_SOURCELIST_GROUP_REMOVABLE, "Source representing a removable device"),
+			ENUM_ENTRY (RB_SOURCELIST_GROUP_TRANSIENT, "Transient source (eg. network shares)"),
+			{ 0, 0, 0 }
+		};
+
+		etype = g_enum_register_static ("RBSourcelistGroupType", values);
+	}
+
+	return etype;
+}
