@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -38,9 +38,23 @@ typedef enum {
 	RB_SOURCE_EOF_NEXT,
 } RBSourceEOFType;
 
+/* groupings in the source list */
+typedef enum {
+	RB_SOURCELIST_GROUP_FIXED,	/* library, iradio, podcast, queue */
+	RB_SOURCELIST_GROUP_PERSISTANT,	/* playlists */
+	RB_SOURCELIST_GROUP_REMOVABLE,	/* ipod, generic audio player, audio CD */
+	RB_SOURCELIST_GROUP_TRANSIENT,	/* DAAP shares */
+
+	RB_SOURCELIST_GROUP_LAST
+} RBSourceListGroup;
+
+
 GType rb_source_eof_type_get_type (void);
 #define RB_TYPE_SOURCE_EOF_TYPE	(rb_source_eof_type_get_type())
 
+GType rb_sourcelist_group_get_type (void);
+#define RB_TYPE_SOURCELIST_GROUP (rb_sourcelist_group_get_type())
+
 #define RB_TYPE_SOURCE         (rb_source_get_type ())
 #define RB_SOURCE(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RB_TYPE_SOURCE, RBSource))
 #define RB_SOURCE_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), RB_TYPE_SOURCE, RBSourceClass))
@@ -71,7 +85,7 @@ struct _RBSourceClass
 	void (*artistalbum_changed)	(RBSource *source);
 
 	/* methods */
-	char *	        (*impl_get_status)	(RBSource *source);
+	void		(*impl_get_status)	(RBSource *source, char **text, char **progress_text, float *progress);
 
 	gboolean	(*impl_can_browse)	(RBSource *source);
 	char *		(*impl_get_browser_key)	(RBSource *source);
@@ -116,7 +130,6 @@ struct _RBSourceClass
 	void		(*impl_delete_thyself)	(RBSource *source);
 	void		(*impl_activate)	(RBSource *source);
 	void		(*impl_deactivate)	(RBSource *source);
-	gboolean	(*impl_disconnect)	(RBSource *source);
 	GList *		(*impl_get_ui_actions)	(RBSource *source);
 	GList *		(*impl_get_search_actions) (RBSource *source);
 };
@@ -132,7 +145,7 @@ void		rb_source_update_play_statistics(RBSource *source, RhythmDB *db,
 
 /* general interface */
 void	        rb_source_set_pixbuf		(RBSource *source, GdkPixbuf *pixbuf);
-char *	        rb_source_get_status		(RBSource *source);
+void	        rb_source_get_status		(RBSource *source, char **text, char **progress_text, float *progress);
 
 gboolean	rb_source_can_browse		(RBSource *source);
 char *		rb_source_get_browser_key	(RBSource *source);
@@ -183,7 +196,6 @@ void		rb_source_delete_thyself	(RBSource *source);
 
 void		rb_source_activate		(RBSource *source);
 void		rb_source_deactivate		(RBSource *source);
-gboolean	rb_source_disconnect		(RBSource *source);
 
 GList *		rb_source_get_ui_actions	(RBSource *source);
 GList *		rb_source_get_search_actions	(RBSource *source);
