@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  *
  */
 
@@ -215,6 +215,9 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (sourcelist->priv->treeview), FALSE);
 	rb_sourcelist_model_set_dnd_targets (RB_SOURCELIST_MODEL (sourcelist->priv->filter_model),
 					     GTK_TREE_VIEW (sourcelist->priv->treeview));
+	gtk_tree_view_set_row_separator_func (GTK_TREE_VIEW (sourcelist->priv->treeview),
+					      (GtkTreeViewRowSeparatorFunc) rb_sourcelist_model_row_is_separator,
+					      sourcelist->priv->filter_model, NULL);
 
 	g_signal_connect_object (G_OBJECT (sourcelist->priv->treeview),
 				 "row_activated",
@@ -362,6 +365,8 @@ rb_sourcelist_append (RBSourceList *sourcelist,
 
 	if (parent) {
 		GtkTreeIter parent_iter;
+
+		rb_debug ("inserting source %p with parent", source);
 		g_assert (rb_sourcelist_source_to_iter (sourcelist, parent, &parent_iter));
 		gtk_tree_store_append (GTK_TREE_STORE (sourcelist->priv->real_model), &iter, &parent_iter);
 
@@ -370,7 +375,54 @@ rb_sourcelist_append (RBSourceList *sourcelist,
 			rb_sourcelist_update_expander_visibility (sourcelist);
 		}
 	} else {
-		gtk_tree_store_append (GTK_TREE_STORE (sourcelist->priv->real_model), &iter, NULL);
+		GtkTreePath *group_path;
+		GtkTreePath *prev_group_path = NULL;
+		RBSourceListGroup group;
+		GtkTreeIter group_iter;
+
+		/* get the marker rows before and after the group for this source */
+		g_object_get (G_OBJECT (source), "sourcelist-group", &group, NULL);
+		group_path = rb_sourcelist_model_get_group_path (RB_SOURCELIST_MODEL (sourcelist->priv->filter_model),
+								 group);
+		g_assert (group_path);
+
+		if (group > 0) {
+			/* there's no marker row before the fixed source group, but we
+			 * don't need one anyway.
+			 */
+			prev_group_path = rb_sourcelist_model_get_group_path (RB_SOURCELIST_MODEL (sourcelist->priv->filter_model),
+									      group-1);
+			g_assert (prev_group_path);
+		}
+
+		/* find the location to insert the source */
+		if (group == RB_SOURCELIST_GROUP_TRANSIENT) {
+			char *check_name = NULL;
+
+			rb_debug ("inserting source %p in group %d in sorted order", source, group);
+			g_assert (prev_group_path);
+
+			gtk_tree_model_get_iter (sourcelist->priv->real_model, &group_iter, prev_group_path);
+			do {
+				g_free (check_name);
+
+				if (!gtk_tree_model_iter_next (sourcelist->priv->real_model, &group_iter))
+					break;
+
+				gtk_tree_model_get (sourcelist->priv->real_model, 
+						    &group_iter, 
+						    RB_SOURCELIST_MODEL_COLUMN_NAME, &check_name,
+						    -1);
+
+			} while (check_name && (strlen (check_name)) > 0 && (g_utf8_collate (name, check_name) > 0));
+			g_free (check_name);
+
+		} else {
+			rb_debug ("inserting source %p in group %d", source, group);
+			gtk_tree_model_get_iter (sourcelist->priv->real_model, &group_iter, group_path);
+		}
+		gtk_tree_store_insert_before (GTK_TREE_STORE (sourcelist->priv->real_model), 
+					      &iter, NULL, &group_iter);
 	}
 
 	gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), &iter,
@@ -547,6 +599,8 @@ rb_sourcelist_selection_changed_cb (GtkTreeSelection *selection,
 	
 	gtk_tree_model_get (cindy, &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
+	if (target == NULL)
+		return;
 	g_return_if_fail (RB_IS_SOURCE (target));
 	source = target;
 	g_signal_emit (G_OBJECT (sourcelist), rb_sourcelist_signals[SELECTED], 0, source);
@@ -575,7 +629,8 @@ row_activated_cb (GtkTreeView *treeview,
 	g_return_if_fail (gtk_tree_model_get_iter (model, &iter, path));
 	gtk_tree_model_get (model, &iter, RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
 
-	g_signal_emit (G_OBJECT (sourcelist), rb_sourcelist_signals[SOURCE_ACTIVATED], 0, target);
+	if (target)
+		g_signal_emit (G_OBJECT (sourcelist), rb_sourcelist_signals[SOURCE_ACTIVATED], 0, target);
 }
 
 static gboolean
@@ -592,6 +647,8 @@ emit_show_popup (GtkTreeView *treeview,
 
 	gtk_tree_model_get (sourcelist->priv->filter_model, &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
+	if (target == NULL)
+		return FALSE;
 	g_return_val_if_fail (RB_IS_SOURCE (target), FALSE);
 
 	g_signal_emit (G_OBJECT (sourcelist), rb_sourcelist_signals[SHOW_POPUP], 0, target, &ret);
@@ -643,6 +700,9 @@ key_release_cb (GtkTreeView *treeview,
 	
 	gtk_tree_model_get (sourcelist->priv->filter_model, &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
+	if (target == NULL)
+		return FALSE;
+
 	g_return_val_if_fail (RB_IS_SOURCE (target), FALSE);
 	if (rb_source_can_rename (target)) {
 		rb_sourcelist_edit_source_name (sourcelist, target);
@@ -736,13 +796,11 @@ rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *
 				    GtkTreeModel *tree_model, GtkTreeIter *iter,
 				    RBSourceList *sourcelist)
 {
-	RBSource *source;
 	char *str;
 	gboolean playing;
 	
 	gtk_tree_model_get (GTK_TREE_MODEL (sourcelist->priv->filter_model), iter,
 			    RB_SOURCELIST_MODEL_COLUMN_NAME, &str,
-			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source,
 			    RB_SOURCELIST_MODEL_COLUMN_PLAYING, &playing,
 			    -1);
 
@@ -770,6 +828,9 @@ source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
 						   &iter, path));
 	gtk_tree_model_get (sourcelist->priv->filter_model,
 			    &iter, RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
+	if (source == NULL)
+		return;
+
 	g_object_set (G_OBJECT (source), "name", text, NULL);
 
 	gtk_tree_path_free (path);
