@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -195,6 +195,7 @@ rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
 	RBPlaylistSource *psource = RB_PLAYLIST_SOURCE (source);
 	RBEntryView *songs;
 	RBShell *shell;
+	RhythmDBEntryType entry_type;
 
 	priv->base_model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (psource));
 	g_object_set (G_OBJECT (priv->base_model), "show-hidden", TRUE, NULL);
@@ -216,7 +217,9 @@ rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
 	priv->search_prop = RHYTHMDB_PROP_SEARCH_MATCH;
 	g_object_unref (G_OBJECT (shell));
 
-	priv->browser = rb_library_browser_new (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)));
+	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
+	priv->browser = rb_library_browser_new (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)), 
+						entry_type);
 	gtk_paned_pack1 (GTK_PANED (priv->paned), GTK_WIDGET (priv->browser), TRUE, FALSE);
 	g_signal_connect_object (G_OBJECT (priv->browser), "notify::output-model",
 				 G_CALLBACK (rb_static_playlist_source_browser_changed_cb),
@@ -256,6 +259,7 @@ rb_static_playlist_source_new (RBShell *shell, const char *name, gboolean local,
 					"shell", shell,
 					"is-local", local,
 					"entry-type", entry_type,
+					"sourcelist-group", RB_SOURCELIST_GROUP_PERSISTANT,
 					NULL));
 }
 
@@ -468,12 +472,11 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	RBStaticPlaylistSource *source = RB_STATIC_PLAYLIST_SOURCE (asource);
 
         if (data->type == gdk_atom_intern ("text/uri-list", TRUE)) {
-                list = gnome_vfs_uri_list_parse ((char *) data->data);
-
-                if (list != NULL)
-                        rb_static_playlist_source_add_list_uri (source, list);
-                else
-                        return FALSE;
+		list = rb_uri_list_parse ((char *)data->data);
+		if (list != NULL) {
+			rb_static_playlist_source_add_list_uri (source, list);
+		} else 
+			return FALSE;
 	}
 
         return TRUE;
@@ -518,13 +521,12 @@ rb_static_playlist_source_add_list_uri (RBStaticPlaylistSource *source,
 	g_return_if_fail (list != NULL);
 
 	for (i = list; i != NULL; i = g_list_next (i)) {
-		char *uri = gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0);
+		char *uri = (char *) i->data;
 		uri_list = g_list_prepend (uri_list, rb_canonicalise_uri (uri));
-		g_free (uri);
 	}
 	uri_list = g_list_reverse (uri_list);
 
-	gnome_vfs_uri_list_free (list);
+	rb_list_deep_free (list);
 
 	if (uri_list == NULL)
 		return;
@@ -562,8 +564,8 @@ rb_static_playlist_source_add_location_internal (RBStaticPlaylistSource *source,
 		RhythmDBEntryType entry_type;
 
 		g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
-		if (entry_type != -1 &&
-		    rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE) != entry_type) {
+		if (entry_type != RHYTHMDB_ENTRY_TYPE_INVALID &&
+		    rhythmdb_entry_get_entry_type (entry) != entry_type) {
 			rb_debug ("attempting to add an entry of the wrong type to playlist");
 			return;
 		}
