@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  */
 
 #include <config.h>
@@ -62,11 +62,11 @@ enum {
 
 #ifdef HAVE_NOTIFY
 struct _Notify {
-#if (LIBNOTIFY_VERSION_MINOR == 2)
+#if (LIBNOTIFY_VERSION_MAJOR == 0 && LIBNOTIFY_VERSION_MINOR == 2)
   NotifyHints *hints;
   NotifyIcon *icon;
   NotifyHandle *handle;
-#elif (LIBNOTIFY_VERSION_MINOR >= 3)
+#elif (LIBNOTIFY_VERSION_MAJOR > 0 || LIBNOTIFY_VERSION_MINOR >= 3)
   NotifyNotification *handle;
 #endif
 };
@@ -289,7 +289,7 @@ egg_tray_icon_unrealize (GtkWidget *widget)
 #ifdef HAVE_NOTIFY
 
   if (EGG_TRAY_ICON (widget)->notify->handle) {
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
+#if (LIBNOTIFY_VERSION_MAJOR > 0 || LIBNOTIFY_VERSION_MINOR >= 3)
     notify_notification_close (EGG_TRAY_ICON (widget)->notify->handle, NULL);
 #elif (LIBNOTIFY_VERSION_MINOR == 2)
     notify_close (EGG_TRAY_ICON (widget)->notify->handle);
@@ -497,7 +497,7 @@ egg_tray_icon_cancel_message (EggTrayIcon *icon,
 #ifdef HAVE_NOTIFY
   if (icon->notify->handle)
   {
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
+#if (LIBNOTIFY_VERSION_MAJOR > 0 || LIBNOTIFY_VERSION_MINOR >= 3)
     notify_notification_close (icon->notify->handle, NULL);
 #elif (LIBNOTIFY_VERSION_MINOR == 2)
     notify_close (icon->notify->handle);
@@ -523,7 +523,7 @@ egg_tray_icon_notify (EggTrayIcon *icon,
 		      const char *secondary)
 {
 #ifdef HAVE_NOTIFY
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
+#if (LIBNOTIFY_VERSION_MAJOR > 0 || LIBNOTIFY_VERSION_MINOR >= 3)
   GtkRequisition size;
   GdkPixbuf *pixbuf;
   int x;
@@ -540,6 +540,15 @@ egg_tray_icon_notify (EggTrayIcon *icon,
       notify_notification_close (icon->notify->handle, NULL);
     }
 
+  if (primary == NULL)
+    {
+      primary = "";
+    }
+  if (secondary == NULL)
+    {
+      secondary = "";
+    }
+
   esc_primary = g_markup_escape_text (primary, strlen (primary));
   esc_secondary = g_markup_escape_text (secondary, strlen (secondary));
   icon->notify->handle = notify_notification_new (esc_primary,
@@ -571,7 +580,7 @@ egg_tray_icon_notify (EggTrayIcon *icon,
 
   if (pixbuf)
     {
-#if (LIBNOTIFY_VERSION_MICRO < 2)
+#if (LIBNOTIFY_VERSION_MAJOR == 0 && LIBNOTIFY_VERSION_MINOR <=3 && LIBNOTIFY_VERSION_MICRO < 2)
       notify_notification_set_icon_data_from_pixbuf (icon->notify->handle, pixbuf);
 #else
       notify_notification_set_icon_from_pixbuf (icon->notify->handle, pixbuf);
@@ -592,7 +601,7 @@ egg_tray_icon_notify (EggTrayIcon *icon,
     }
 
   return;
-#elif (LIBNOTIFY_VERSION_MINOR == 2)
+#elif (LIBNOTIFY_VERSION_MAJOR == 0 && LIBNOTIFY_VERSION_MINOR == 2)
   gint x, y;
   GtkRequisition size;
   NotifyIcon *icon_notify = NULL;
