@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  */
 
 #ifndef __EGG_TRAY_ICON_H__
@@ -29,8 +29,6 @@
 #endif
 #ifdef HAVE_NOTIFY
 #include <libnotify/notify.h>
-#else
-#include "eggnotificationbubble.h"
 #endif
 
 G_BEGIN_DECLS
@@ -64,8 +62,6 @@ struct _EggTrayIcon
   GtkOrientation orientation;
 #ifdef HAVE_NOTIFY
   Notify *notify;
-#else
-  EggNotificationBubble *bubble;
 #endif
 };
 
