@@ -1,5 +1,6 @@
-/* rb-cell-renderer-rating.c
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ * rb-cell-renderer-rating.c
  * arch-tag: Implementation of star rating GtkTreeView cell renderer
  *
  * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
@@ -17,18 +18,17 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- *  $ Id $
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
+ *
  */
 
 #include <config.h>
+
 #include <stdlib.h>
-#include <libgnome/gnome-i18n.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkiconfactory.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include "rb-cell-renderer-rating.h"
 #include "rb-marshal.h"
@@ -77,6 +77,11 @@ struct RBCellRendererRatingClassPrivate
 	RBRatingPixbufs *pixbufs;	
 };
 
+G_DEFINE_TYPE (RBCellRendererRating, rb_cell_renderer_rating, GTK_TYPE_CELL_RENDERER)
+#define RB_CELL_RENDERER_RATING_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
+						RB_TYPE_CELL_RENDERER_RATING, \
+						RBCellRendererRatingPrivate))
+
 
 enum 
 {
@@ -90,43 +95,14 @@ enum
 	LAST_SIGNAL
 };
 
-static GObjectClass *parent_class = NULL;
-
 static guint rb_cell_renderer_rating_signals[LAST_SIGNAL] = { 0 };
 
-GtkType
-rb_cell_renderer_rating_get_type (void)
-{
-	static GtkType cell_rating_type = 0;
-
-	if (!cell_rating_type) {
-		static const GTypeInfo cell_rating_info =
-		{
-			sizeof (RBCellRendererRatingClass),
-			NULL,		/* base_init */
-			NULL,		/* base_finalize */
-			(GClassInitFunc) rb_cell_renderer_rating_class_init,
-			NULL,		/* class_finalize */
-			NULL,		/* class_data */
-			sizeof (RBCellRendererRating),
-			0,              /* n_preallocs */
-			(GInstanceInitFunc) rb_cell_renderer_rating_init,
-		};
-
-		cell_rating_type = g_type_register_static (GTK_TYPE_CELL_RENDERER, 
-							   "RBCellRendererRating", 
-							   &cell_rating_info, 
-							   0);
-	}
-
-	return cell_rating_type;
-}
 
 static void
 rb_cell_renderer_rating_init (RBCellRendererRating *cellrating)
 {
 
-	cellrating->priv = g_new0 (RBCellRendererRatingPrivate, 1);
+	cellrating->priv = RB_CELL_RENDERER_RATING_GET_PRIVATE (cellrating);
 
 	/* set the renderer able to be activated */
 	GTK_CELL_RENDERER (cellrating)->mode = GTK_CELL_RENDERER_MODE_ACTIVATABLE;
@@ -140,7 +116,6 @@ rb_cell_renderer_rating_class_init (RBCellRendererRatingClass *class)
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 	GtkCellRendererClass *cell_class = GTK_CELL_RENDERER_CLASS (class);
 
-	parent_class = g_type_class_peek_parent (class);
 	object_class->finalize = rb_cell_renderer_rating_finalize;
 
 	object_class->get_property = rb_cell_renderer_rating_get_property;
@@ -166,6 +141,8 @@ rb_cell_renderer_rating_class_init (RBCellRendererRatingClass *class)
 			      2,
 			      G_TYPE_STRING,
 			      G_TYPE_DOUBLE);
+
+	g_type_class_add_private (class, sizeof (RBCellRendererRatingPrivate));
 }
 
 static void
@@ -175,9 +152,7 @@ rb_cell_renderer_rating_finalize (GObject *object)
 
 	cellrating = RB_CELL_RENDERER_RATING (object);
 
-	g_free (cellrating->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_cell_renderer_rating_parent_class)->finalize (object);
 }
 
 static void
