@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -108,7 +108,7 @@ void		rb_entry_view_append_column		(RBEntryView *view,
 void		rb_entry_view_append_column_custom	(RBEntryView *view, GtkTreeViewColumn *column,
 							 const char *title, const char *key,
 							 GCompareDataFunc sort_func,
-							 RhythmDBPropType sort_propid);
+							 gpointer data);
 
 void		rb_entry_view_set_columns_clickable	(RBEntryView *view, gboolean clickable);
 
