@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -26,7 +26,7 @@
 #include <gtk/gtkhbox.h>
 
 #include "rhythmdb.h"
-#include "rb-player.h"
+#include "rb-shell-player.h"
 
 G_BEGIN_DECLS
 
@@ -53,29 +53,22 @@ typedef struct
 
 GType		rb_header_get_type		(void);
 
-RBHeader *	rb_header_new			(RBPlayer *player);
+RBHeader *	rb_header_new			(RBShellPlayer *header);
 
-void		rb_header_set_playing_entry	(RBHeader *player,
+void		rb_header_set_playing_entry	(RBHeader *header,
 						 RhythmDBEntry *entry);
 
-void		rb_header_set_title		(RBHeader *player,
+void		rb_header_set_title		(RBHeader *header,
 						 const char *title);
 
-void		rb_header_set_show_artist_album	(RBHeader *player,
-						 gboolean show);
-
-void		rb_header_set_show_position_slider (RBHeader *player,
+void		rb_header_set_show_position_slider (RBHeader *header,
                                                     gboolean show);
 
-void		rb_header_set_urldata		(RBHeader *player,
-						 const char *urltext,
-						 const char *urllink);
+void		rb_header_sync			(RBHeader *header);
 
-void		rb_header_sync			(RBHeader *player);
+gboolean	rb_header_sync_time		(RBHeader *header);
 
-gboolean	rb_header_sync_time		(RBHeader *player);
-
-char *		rb_header_get_elapsed_string	(RBHeader *player);
+char *		rb_header_get_elapsed_string	(RBHeader *header);
 
 G_END_DECLS
 
