@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -641,8 +641,8 @@ static void
 rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 				       RBPropertyView *view)
 {
-	const char *selected_prop;
-	gboolean is_all;
+	char *selected_prop = NULL;
+	gboolean is_all = TRUE;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 
@@ -695,6 +695,8 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 				       is_all ? NULL : selected_prop);
 		}
 	}
+
+	g_free (selected_prop);
 }
 
 static gboolean 
