@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -45,6 +45,7 @@
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
 #include "rb-source.h"
+#include "rb-shell.h"
 
 static void rb_song_info_class_init (RBSongInfoClass *klass);
 static void rb_song_info_init (RBSongInfo *song_info);
@@ -68,6 +69,7 @@ static void rb_song_info_populate_dialog (RBSongInfo *song_info);
 static void rb_song_info_populate_dialog_multiple (RBSongInfo *song_info);
 static void rb_song_info_update_duration (RBSongInfo *song_info);
 static void rb_song_info_update_location (RBSongInfo *song_info);
+static void rb_song_info_update_filesize (RBSongInfo *song_info);
 static void rb_song_info_update_play_count (RBSongInfo *song_info);
 static void rb_song_info_update_last_played (RBSongInfo *song_info);
 static void rb_song_info_update_bitrate (RBSongInfo *song_info);
@@ -105,6 +107,7 @@ struct RBSongInfoPrivate
 	/* the dialog widgets */
 	GtkWidget   *backward;
 	GtkWidget   *forward;
+	GtkWidget   *notebook;
 
 	GtkWidget   *title;
 	GtkWidget   *artist;
@@ -120,6 +123,7 @@ struct RBSongInfoPrivate
 	GtkWidget   *duration;
 	GtkWidget   *name;
 	GtkWidget   *location;
+	GtkWidget   *filesize;
 	GtkWidget   *play_count;
 	GtkWidget   *last_played;
 	GtkWidget   *rating;
@@ -142,7 +146,8 @@ enum
 {
 	PROP_0,
 	PROP_SOURCE,
-	PROP_ENTRY_VIEW
+	PROP_ENTRY_VIEW,
+	PROP_CURRENT_ENTRY
 };
 
 static guint rb_song_info_signals[LAST_SIGNAL] = { 0 };
@@ -175,6 +180,12 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 					                      "RBEntryView object",
 					                      RB_TYPE_ENTRY_VIEW,
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_CURRENT_ENTRY,
+					 g_param_spec_pointer ("current-entry",
+					                       "RhythmDBEntry",
+					                       "RhythmDBEntry object",
+					                       G_PARAM_READABLE));
 
 	object_class->finalize = rb_song_info_finalize;
 
@@ -254,11 +265,13 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	gtk_window_set_title (GTK_WINDOW (song_info), _("Song Properties"));
 
 	/* get the widgets from the XML */
+	song_info->priv->notebook      = glade_xml_get_widget (xml, "song_info_vbox");
 	song_info->priv->title         = glade_xml_get_widget (xml, "song_info_title");
 	song_info->priv->track_cur     = glade_xml_get_widget (xml, "song_info_track_cur");
 	song_info->priv->bitrate       = glade_xml_get_widget (xml, "song_info_bitrate");
 	song_info->priv->duration      = glade_xml_get_widget (xml, "song_info_duration");
 	song_info->priv->location = glade_xml_get_widget (xml, "song_info_location");
+	song_info->priv->filesize = glade_xml_get_widget (xml, "song_info_filesize");
 	song_info->priv->play_count    = glade_xml_get_widget (xml, "song_info_playcount");
 	song_info->priv->last_played   = glade_xml_get_widget (xml, "song_info_lastplayed");
 	song_info->priv->name = glade_xml_get_widget (xml, "song_info_name");
@@ -267,6 +280,7 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	rb_glade_boldify_label (xml, "trackn_label");
 	rb_glade_boldify_label (xml, "name_label");
 	rb_glade_boldify_label (xml, "location_label");
+	rb_glade_boldify_label (xml, "filesize_label");
 	rb_glade_boldify_label (xml, "last_played_label");
 	rb_glade_boldify_label (xml, "play_count_label");
 	rb_glade_boldify_label (xml, "duration_label");
@@ -297,6 +311,8 @@ rb_song_info_construct_multiple (RBSongInfo *song_info, GladeXML *xml,
 	gtk_window_set_title (GTK_WINDOW (song_info),
 			      _("Multiple Song Properties"));
 	gtk_widget_grab_focus (song_info->priv->artist);
+
+	song_info->priv->notebook = glade_xml_get_widget (xml, "song_info_notebook");
 }
 
 static void
@@ -321,6 +337,7 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	GList *selected_entries;
 	GList *tem;
 	gboolean editable = TRUE;
+	RBShell *shell;
 
 	klass = RB_SONG_INFO_CLASS (g_type_class_peek (RB_TYPE_SONG_INFO));
 
@@ -356,12 +373,11 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 				   glade_xml_get_widget (xml, "song_info_vbox"));
 	} else {
 		xml = rb_glade_xml_new ("song-info-multiple.glade",
-					"song_info_basic",
+					"song_info_notebook",
 					song_info);
 		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (song_info)->vbox),
-				   glade_xml_get_widget (xml, "song_info_basic"));
+				   glade_xml_get_widget (xml, "song_info_notebook"));
 	}
-		
 	
 	glade_xml_signal_autoconnect (xml);
 
@@ -427,6 +443,9 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 		rb_song_info_construct_multiple (song_info, xml, editable);
 		rb_song_info_populate_dialog_multiple (song_info);
 	}
+	g_object_get (G_OBJECT (song_info->priv->source), "shell", &shell, NULL);
+	g_signal_emit_by_name (G_OBJECT (shell), "create_song_info", song_info, (song_info->priv->current_entry == NULL));
+	g_object_unref (G_OBJECT (shell));
 
 	gtk_dialog_add_button (GTK_DIALOG (song_info),
 			       GTK_STOCK_CLOSE,
@@ -553,6 +572,9 @@ rb_song_info_get_property (GObject *object,
 	case PROP_ENTRY_VIEW:
 		g_value_set_object (value, song_info->priv->entry_view);
 		break;
+	case PROP_CURRENT_ENTRY:
+		g_value_set_pointer (value, song_info->priv->current_entry);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -582,6 +604,21 @@ rb_song_info_new (RBSource *source, RBEntryView *entry_view)
 	return GTK_WIDGET (song_info);
 }
 
+guint
+rb_song_info_append_page (RBSongInfo *info, const char *title, GtkWidget *page)
+{
+	GtkWidget *label;
+	guint page_num;
+
+	label = gtk_label_new (title);
+	page_num = gtk_notebook_append_page (GTK_NOTEBOOK (info->priv->notebook),
+					     page,
+					     label);
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (info->priv->notebook), TRUE);
+
+	return page_num;
+}
+
 typedef void (*RBSongInfoSelectionFunc)(RBSongInfo *info,
 					RhythmDBEntry *entry,
 					void *data);
@@ -750,7 +787,7 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 {
 	const char *text;
 	char *tmp;
-	ulong num;
+	gulong num;
 	
 	g_assert (song_info->priv->current_entry);
 
@@ -778,6 +815,7 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 
 	rb_song_info_update_duration (song_info);
 	rb_song_info_update_location (song_info);
+	rb_song_info_update_filesize (song_info);
 	rb_song_info_update_play_count (song_info);
 	rb_song_info_update_last_played (song_info);
 	rb_song_info_update_bitrate (song_info);
@@ -839,6 +877,17 @@ rb_song_info_update_duration (RBSongInfo *song_info)
 	g_free (text);
 }
 
+static void
+rb_song_info_update_filesize (RBSongInfo *song_info)
+{
+	char *text = NULL;
+	guint64 filesize = 0;
+	filesize = rhythmdb_entry_get_uint64 (song_info->priv->current_entry, RHYTHMDB_PROP_FILE_SIZE);
+	text = gnome_vfs_format_file_size_for_display (filesize);
+	gtk_label_set_text (GTK_LABEL (song_info->priv->filesize), text);
+	g_free (text);
+}
+
 static void
 rb_song_info_update_location (RBSongInfo *song_info)
 {
@@ -913,6 +962,7 @@ rb_song_info_backward_clicked_cb (GtkWidget *button,
 	rb_entry_view_scroll_to_entry (song_info->priv->entry_view, new_entry);
 
 	rb_song_info_populate_dialog (song_info);
+	g_object_notify (G_OBJECT (song_info), "current-entry");
 }
 
 static void
@@ -932,6 +982,7 @@ rb_song_info_forward_clicked_cb (GtkWidget *button,
 	rb_entry_view_scroll_to_entry (song_info->priv->entry_view, new_entry);
 
 	rb_song_info_populate_dialog (song_info);
+	g_object_notify (G_OBJECT (song_info), "current-entry");
 }
 
 /*
@@ -1063,7 +1114,7 @@ rb_song_info_update_rating (RBSongInfo *song_info)
 static void
 rb_song_info_update_year (RBSongInfo *song_info)
 {
-	ulong year;
+	gulong year;
 	char *text;
 
 	year = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_YEAR);
@@ -1183,7 +1234,7 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		g_value_set_ulong (&val, discn);
 
 		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
-			ulong entry_disc_num;
+			gulong entry_disc_num;
 			
 			entry = (RhythmDBEntry *)tem->data;
 			entry_disc_num = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
